/* #[info:		*/
/************************************************************************
*									*
*		    |\							*
*		     \\		 _		 '  '			*
*		    / \\	< \,  _-_,  _-_ \\ \\			*
*		   || ||	/-|| ||_.  ||	|| ||			*
*		   || ||       (( ||  ~ || ||	|| ||			*
*		    \\/		\/\\ ,-_-  \\,/ \\ \\			*
*			 _____						*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : d_ascii.c							*
* Purpose : Dump a Go data structure to ASCII				*
* Version : 1.16							*
* Modified: 22 Mar 1995 09:42:47					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "tools.h"
#include "d_ascii.h"

/* #]include:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

static void	ASCII_header	PROTO((GOGAME *g, char *s));

#undef PROTO

/* #]prototype:		*/ 
/* #[extern:		*/

extern	FILE *		fw;		/* Output stream		*/
extern	int		kibitz;		/* Do we have kibitz or text	*/
extern	int		labels;		/* Labels with move numbers	*/

/* #]extern:		*/ 

/* #[ASCII_header:	*/

static void
ASCII_header(g, s)
GOGAME *g;
char   *s;
{
    int indent, n;

    indent = n = (2*g->gm_sz - (int) strlen(s))/2;
    if (indent < 0) indent = n = 0;

    fputc('\t', fw); while (indent--) fprintf(fw, " "); fprintf(fw, "%s\n", s);
    fputc('\t', fw); while (n--     ) fprintf(fw, " ");
    n = (int) strlen(s);
    while (n--     ) fprintf(fw, "-"); fprintf(fw, "\n");
}

/* #]ASCII_header:	*/ 

/* #[ASCII_init:	*/

int
ASCII_init()
{
    return 1;
}

/* #]ASCII_init:	*/ 
/* #[ASCII_exit:	*/

void
ASCII_exit()
{
    if (fw != stdin) fclose(fw);
}

/* #]ASCII_exit:	*/ 
/* #[ASCII_alloc:	*/

int
ASCII_alloc(g)
GOGAME *g;
{
    return 1;
}

/* #]ASCII_alloc:	*/ 
/* #[ASCII_free:	*/

void
ASCII_free(g)
GOGAME *g;
{
}

/* #]ASCII_free:	*/ 
/* #[ASCII_onmove:	*/

void
ASCII_onmove(g, n1, c1, n2, c2)
GOGAME *g;
int	n1;
int	c1;
int	n2;
int	c2;
{
    con_printf("%d at %d", n1, n2);
}

/* #]ASCII_onmove:	*/ 
/* #[ASCII_onstone:	*/

void
ASCII_onstone(g, n, c, p)
GOGAME *g;
int	n;
int	c;
PLACE	p;
{
    int i = P2a(g,p);
    int j = g->gm_sz - P2J(g,p);

    if (i >= 'i') i++;	/* Skip the 'i' */

    con_printf("%d at (%c,%d)", n, i, j);
}

/* #]ASCII_onstone:	*/ 
/* #[ASCII_pass:	*/

void
ASCII_pass(g, n, c)
GOGAME *g;
int	n;
int	c;
{
    con_printf("%d pass", n);
}

/* #]ASCII_pass:	*/ 
/* #[ASCII_time:	*/

void
ASCII_time(g, btime, wtime)
/*
 * For the moment we ignore the time on the clock
 */
GOGAME *g;
char *btime;
char *wtime;
{
}

/* #]ASCII_time:	*/ 
/* #[ASCII_emit:	*/

#define COORDINATES	"a b c d e f g h j k l m n o p q r s t"

int
ASCII_emit(g, m1, m2, t1, t2)
GOGAME *g;
GOMOVE *m1, *m2;	/* Diagram from m1 to m2	*/
GOMOVE *t1, *t2;	/* Text    from t1 to t2	*/
{
    GOMOVE *m;
    PLACE   p;
    char    username[MAXUSER];		/* Name of last kibitzer */
    int     size = g->gm_sz;		/* The board size	*/
    int     n1   = 999;			/* The 1st move number	*/
    int     n2   = 0;			/* The lst move number	*/
    int     i;
    char *  name = board_caption(g, m1, m2);

/*	#[foldline:		*/

    fprintf(fw, "\n");

/*	#]foldline:		*/ 
/*	#[position:		*/

    /*
     * Print the coordinates
     */
    fprintf(fw, "\t   ");
    for (i = 0; i < size; i++) fprintf(fw, "%c ", *(COORDINATES + 2*i));
    fputc('\n', fw);

    /*
     * Print the position
     */
    for (p = 0; p < size*size; p++) {
	if (p % size == 0) fprintf(fw, "\t%2d ", size - (p/size));

	m = g->gm_board + p;
	if (m->mv_letter)
		fprintf(fw, "%c ", m->mv_letter->ch_letter);
	else
	if (m->mv_mark) {
	    if (*(m->mv_mark->ch_mark) >= LETTER_MIN &&
		*(m->mv_mark->ch_mark) <= LETTER_MAX) {
		fprintf(fw, "%s", m->mv_mark->ch_mark);
	    } else {
		switch(m->mv_side) {
		    case FREE : fprintf(fw, "MK"); break;
		    case BLACK: fprintf(fw, "x "); break;
		    case WHITE: fprintf(fw, "o "); break;
		}
	    }
	}
	else
	if (m->mv_side) {
	    if (m->mv_nr) {
		if (m->mv_nr < n1) n1 = m->mv_nr;
		if (m->mv_nr > n2) n2 = m->mv_nr;
		fprintf(fw, "%-2d", m->mv_nr % 100);
	    }
	    else
		fprintf(fw, "%c ", (m->mv_side == BLACK) ? '@' : 'O');
	} else  fprintf(fw, ". ");

	if ((p+1) % size == 0) fprintf(fw, " %2d\n", size - (p/size));
    }

    /*
     * Print the coordinates
     */
    fprintf(fw, "\t   ");
    for (i = 0; i < size; i++) fprintf(fw, "%c ", *(COORDINATES + 2*i));
    fputc('\n', fw);

/*	#]position:		*/ 
/*	#[caption:		*/

    /*
     * Show the title of this position
     */
    fputc('\n', fw);
    ASCII_header(g, name);

/*	#]caption:		*/ 
/*	#[connections:		*/

    /*
     * Flush the ko/connect messages
     */
    {
	char *connects = con_cat((char *) 0);

	if (*connects) fprintf(fw, "\n%s\n", connects);
    }
    fprintf(fw, "\n");

/*	#]connections:		*/ 
/*	#[text:			*/
    /*
     * Print the text associated with the position
     */
    if (t1) for (m = t1; m != t2; m = m->mv_next) {
	TEXT *t;

	if (!m->mv_text) continue;
/*
	  #[statistics:
*/
	if (kibitz) kib_add(m->mv_text);
/*
	  #]statistics: 
	  #[labels:
*/
	if (m->mv_nr) {
	    username[0] = 0;
	    if (labels && n1 != n2) {
		fprintf(fw, "\n%s %3d:\n",
			(m->mv_side == BLACK) ? "Black" : "White",
			m->mv_nr);
		fprintf(fw, "---------\n");
	    }
	}
/*
	  #]labels: 
	  #[text:
*/
	for (t = m->mv_text; t; t = t->cc_next) {
	    char line[1024];
/*
	  #[preparation:
*/
	    strcpy(line, t->cc_text);
	    if (*(line) &&
		*(line + strlen(line)-1) == ' ')
		*(line + strlen(line)-1) = '\n';
	    if (kibitz	&&
		*(line) &&
		*(line + strlen(line)-1) == '\n')
		*(line + strlen(line)-1) = 0;
/*
	  #]preparation: 
	  #[non-kibitz:
*/
	    if (!kibitz) { fprintf(fw, "%s", line); continue; }
/*
	  #]non-kibitz: 
	  #[kibitz:
*/
	    if (strcmp(username, t->cc_name)) {
		strcpy(username, t->cc_name);
		fprintf(fw, "%s[%s]: %s\n",
			t->cc_name,
			rank2str(t->cc_spec),
			line);
	    } else {
		/*
		 * 4 = strlen("[]: ")
		 */
		int n = (int) (
			    strlen(username) +
			    strlen(rank2str(t->cc_spec)) + 4);

		while (n--) fputc(' ', fw);
		fprintf(fw, "%s\n", line);
	    }
/*
	  #]kibitz: 
*/
	}
/*
	  #]text: 
*/
    }
/*	#]text:			*/ 

    return 1;
}

/* #]ASCII_emit:	*/ 

