/* #[info:		*/
/************************************************************************
*									*
*			 |\	     |\	  ,,				*
*			  \\	      \\  ||				*
*			 / \\	     / \\ ||/|,				*
*			|| ||	    || || || ||				*
*			|| ||	    || || || |'				*
*			 \\/	     \\/  \\/				*
*			      _____					*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : d_db.c							*
* Purpose : Get ascii information from a Go game			*
* Version : 1.13							*
* Modified: 03 May 1995 10:33:01					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "d_db.h"
#include "patch.h"

/* #]include:		*/ 
/* #[define:		*/

/*
 * Write complete data structure
 */
#define DUMPN(t,f) fprintf(fw, "%s[%d]\n", t, (g) ? ((g->f) ? (g->f) : 0) : 0)
#define DUMPS(t,f)  if (g && g->f && *(g->f)) {				\
			fprintf(fw, "%s[", t);				\
			text2sgf((g) ? ((g->f) ? (g->f) : "") : "");	\
			fputc(']', fw); fputc('\n', fw);		\
		    }

/* #]define:		*/ 
/* #[extern:		*/

extern	FILE *	fw;		/* Output stream			*/
extern	int	longkey;	/* Long or short SGF keys while writing	*/

/* #]extern:		*/ 

/* #[game2db:		*/

void
game2db(g, filename)
/*
 * Dump the ASCII information contained in an SGF file
 */
GOGAME *g;
char   *filename;
{
    static int done = 0;

    if (done) return;

    /*
     * The following properties are not interesting for databases
     */
    if (!filename) {
	/*
	 * Game number
	 */
	DUMPS(GM_KEY, gm_gamenr);

	/*
	 * Clipping rectangle of the board
	 */
	if (g->gm_vw[0] || g->gm_vw[1]) {
	    fprintf(fw, "%s[%c%c]", VW_KEY,
		    'a' + g->gm_vw[0],
		    'a' + g->gm_vw[1]);
	    if (g->gm_vw[2] || g->gm_vw[3]) {
		fprintf(fw, "[%c%c]",
			'a' + g->gm_vw[2],
			'a' + g->gm_vw[3]);
		}
	    fputc('\n', fw);
	} else
	    fprintf(fw, "%s[]\n", VW_KEY);

	/*
	 * Region rectangle of the board
	 */
	if (g->gm_rg[0] || g->gm_rg[1]) {
	    fprintf(fw, "%s[%c%c]", RG_KEY,
		    'a' + g->gm_rg[0],
		    'a' + g->gm_rg[1]);
	    if (g->gm_rg[2] || g->gm_rg[3]) {
		fprintf(fw, "[%c%c]",
			'a' + g->gm_rg[2],
			'a' + g->gm_rg[3]);
		}
	    fputc('\n', fw);
	}

	/*
	 * Round
	 */
	if (g->gm_ro) fprintf(fw, "%s[%d]\n", RO_KEY, g->gm_ro);

	DUMPS(US_KEY, gm_us);		/* Messages for user	*/
	fprintf(fw, "%s[Converted by sgf2misc-%d.%d.%d]\n",
		US_KEY, RELEASE, PATCHLEVEL/10, PATCHLEVEL%10);
    }
    DUMPS(GN_KEY	, gm_gn); /* Game name			*/
    DUMPS(EV_KEY	, gm_ev); /* Event specification	*/
    DUMPS(ID_KEY	, gm_id); /* Game ID			*/
    DUMPS(FF_KEY	, gm_ff); /* File format 		*/
    DUMPS(PB_KEY	, gm_pb); /* Black player 		*/
    DUMPS(BR_KEY	, gm_br); /* Black rank			*/
    DUMPS(PW_KEY	, gm_pw); /* White player 		*/
    DUMPS(WR_KEY	, gm_wr); /* White rank			*/
    DUMPS(DT_KEY	, gm_dt); /* Date 			*/
    DUMPS(PC_KEY	, gm_pc); /* Place			*/
    DUMPS(RE_KEY	, gm_re); /* Result			*/
    if (g->gm_sz != 19)
	DUMPN(SZ_KEY	, gm_sz); /* Board size			*/
    DUMPS(HA_KEY	, gm_ha); /* Handicap 			*/
    DUMPS(KM_KEY	, gm_km); /* Komi 			*/
    DUMPS(SO_KEY	, gm_so); /* Game source 		*/
    DUMPS(TM_KEY	, gm_tm); /* Alloted time		*/
    if (filename) {
	/*
	 * Not SGF compliant but interesting for databases
	 */
	DUMPN(MS_KEY	, gm_movenr  ); /* Number of moves in the game	*/
	fprintf(fw, "%s[%s]\n",		/* Source of the game on disk	*/
		FN_KEY, filename);
    }

    done = 1;
}

/* #]game2db:		*/ 
/* #[text2sgf:		*/

void
text2sgf(t)
/*
 * Copy t to fw while taking care of SGF special characters.
 */
char *t;
{
    for (; *t; t++) {
	if (*t == ']' || *t == '\\') fputc('\\', fw);
	fputc(*t, fw);
    }
}

/* #]text2sgf:		*/ 

