/* #[info:		*/
/************************************************************************
*									*
*		        |\                    /\			*
*		         \\          _    '  || 			*
*		        / \\        / \\ \\ =||=			*
*		       || ||       || || ||  || 			*
*		       || ||       || || ||  || 			*
*		        \\/        \\_-| \\  \\,			*
*		             _____  /  \        			*
*		                   '----`        			*
*									*
*		 Copyright (c) 1993  Jan van der Steen			*
*	      Centre for Mathematics and Computer Science		*
*		      Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File    : d_gif.c							*
* Purpose : Dump GIF diagrams contained by a Go data structure		*
* Version : 1.11							*
* Modified: 03 May 1995 12:06:53					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include "sgf2misc.h"
#include "d_gif.h"
#include "gogame.h"
#include "tools.h"

/* #]include:		*/ 
/* #[define:		*/

/* #]define:		*/ 
/* #[static:		*/

static	FILE *	fp;			/* GIF generating script	*/
static	char	mkgif[1024];		/* Name of GIF script		*/
static	int	xvmin = 0, xvmax;	/* View port */
static	int	yvmin = 0, yvmax;
static	int	xrmin = 0, xrmax;	/* Selected region */
static	int	yrmin = 0, yrmax;
static	int	caption = 0;

/* #]static:		*/ 
/* #[extern:		*/

extern	FILE *		fw;		/* Output stream		*/
extern	EPSFINFO	epsf;		/* The eps filename/directory	*/
extern	char *		program;	/* Name of this program		*/
extern	char *		ppm;		/* Location of ppm bitmaps	*/
extern	char *		pbmbin;		/* Location of pbmplus package	*/
extern	char *		rgb_b;		/* Black			*/
extern	char *		rgb_w;		/* White			*/
extern	char *		rgb_g;		/* Board			*/
extern	char *		rgb_r;		/* Region			*/
extern	char *		rgb_o;		/* Outside			*/
extern	int		fntsz;		/* Bitmap size			*/
extern	int		hold;		/* Don't remove script		*/

/* #]extern:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

static int	gif_init	PROTO((GOGAME *g, int n, char *title));
static void	gif_exit	PROTO((int f, int n, char *title));
static char *	border		PROTO((GOGAME *g, int p));
static int	hoshi		PROTO((GOGAME *g, int p));

#undef PROTO

/* #]prototype:		*/ 

/* #[gif_init:		*/

static int
gif_init(g, n, title)
/*
 * Open the GIF generating script file and print the header
 */
GOGAME *g;
int	n;
char *	title;
{
    char *thetime;
    long  sec;

    /*
     * Get the creation time
     */
    time(&sec);
    thetime = ctime(&sec);

    /*
     * Construct the name of the GIF generating script
     */
    sprintf(mkgif, "%s%c%s.%c%02d",
	    epsf.eps_dir,
	    DIRSEP,
	    epsf.eps_name,
	    (g->gm_parent) ? 'd' : 'f',
	    n);
    if ((fp = fopen(mkgif, "w")) == (FILE *) 0) {
	message("Couldn't create: \"%s\"\n", mkgif);
	exit(1);
    }

    fprintf(fp, "#\n");
    fprintf(fp, "# Title	: %s-%s (%s)\n",
		(g->gm_pw) ? g->gm_pw : "unknown",
		(g->gm_pb) ? g->gm_pb : "unknown",
		title);
    fprintf(fp, "# Creator	: %s\n", program);
    fprintf(fp, "# CreationDate	: %s", thetime);
    fprintf(fp, "#\n");
    /*
     * Construct the PATH including the PBM area.
     * Construct the temporary directory
     * Construct the gif filename for output
     */
    fprintf(fp, "PATH=${PATH}:%s\n", pbmbin);
    fprintf(fp, "TMP=\"%s\"\n", epsf.eps_dir);
    fprintf(fp, "GIF=\"${TMP}/%s-%c%02d.gif\"\n",
	    epsf.eps_name, (g->gm_parent) ? 'd' : 'f', n);
    /*
     * Remember old directory.
     * Change directory to the PPM directory to avoid excessively
     * lengthy pathnames.
     */
    fprintf(fp, "CWD=\"`/bin/pwd`\"\n");
    fprintf(fp, "cd %s/%d\n", ppm, fntsz);

    return 1;
}

/* #]gif_init:		*/ 
/* #[gif_exit:		*/

static void
gif_exit(f, n, title)
int	f;
int	n;
char *	title;
{
    char cmd[1024];
    int  i;

    /*
     * Collate the lines
     */
    fprintf(fp, "#\n");
    fprintf(fp, "# * Collate the lines\n");
    fprintf(fp, "# * Convert the colors according to the user specs\n");
    fprintf(fp, "# * Convert to GIF\n");
    fprintf(fp, "#\n");
    fprintf(fp, "pnmcat -tb\t\\\n");
    for (i = yvmin; i <= yvmax; i++)
	fprintf(fp, "\t${TMP}/%02d.ppm\t\\\n", i);
    fflush(fp);

    /*
     * Convert the colors:
     *		0 = black
     *		1 = white
     *		2 = board
     *		3 = outside
     *		4 = region
     */
    fprintf(fp, "| ppmchange rgb\\:%02x/%02x/%02x %s\\\n", 0,0,0, rgb_b);
    fprintf(fp, "| ppmchange rgb\\:%02x/%02x/%02x %s\\\n", 1,1,1, rgb_w);
    fprintf(fp, "| ppmchange rgb\\:%02x/%02x/%02x %s\\\n", 2,2,2, rgb_g);
    fprintf(fp, "| ppmchange rgb\\:%02x/%02x/%02x %s\\\n", 3,3,3, rgb_o);
    fprintf(fp, "| ppmchange rgb\\:%02x/%02x/%02x %s\\\n", 4,4,4, rgb_r);

    fprintf(fp, "| ppmtogif > ${GIF}\n");

    /*
     * Move output to original directory.
     */
    fprintf(fp, "/bin/mv ${GIF} ${CWD}\n");

    fprintf(fp, "#\n");
    fprintf(fp, "# * Remove intermediate files\n");
    fprintf(fp, "#\n");
    fprintf(fp, "/bin/rm ");
    for (i = yvmin; i <= yvmax; i++) fprintf(fp, "\\\n\t${TMP}/%02d.ppm", i);
    fprintf(fp, "\n");
    fclose (fp);

    /*
     * 1. Prepare command for /bin/sh.
     * 2. Execute it.
     * 3. Remove script.
     */
    sprintf(cmd, "/bin/sh %s", mkgif);
    if (caption) { fputc('\n', stderr); caption = 0; }
    fprintf(stderr, "[%s %d (%s) WIDTH=%d HEIGHT=%d]\n",
	    (f) ? "Figure" : "Diagram", n,
	    title, (xvmax-xvmin+1)*fntsz, (yvmax-yvmin+1)*fntsz);
    if (system(cmd) || hold)
	fprintf(stderr, "Left script in \"%s\"\n", mkgif);
    else
	unlink(mkgif);
}

/* #]gif_exit:		*/ 

/* #[GIF_init:		*/

int
GIF_init()
{
    if (fntsz != 15 && fntsz != 19 && fntsz != 23) {
	fprintf(stderr, "Illegal pointsize: %d\n", fntsz);
	return 0;
    }

    return 1;
}

/* #]GIF_init:		*/ 
/* #[GIF_exit:		*/

void
GIF_exit()
{
    if (fw != stdout) fclose(fw);
}

/* #]GIF_exit:		*/ 
/* #[GIF_alloc:		*/

int
GIF_alloc(g)
GOGAME *g;
{
    return 1;
}

/* #]GIF_alloc:		*/ 
/* #[GIF_free:		*/

void
GIF_free(g)
GOGAME *g;
{
}

/* #]GIF_free:		*/ 
/* #[GIF_onmove:	*/

void
GIF_onmove(g, n1, c1, n2, c2)
GOGAME *g;
int	n1;
int	c1;
int	n2;
int	c2;
{
    if (caption) fprintf(stderr, ", ");
    fprintf(stderr, "%c%d at %c%d",
	    (c1 == BLACK) ? 'B' : 'W', n1,
	    (c2 == BLACK) ? 'B' : 'W', n2);
    caption++;
}

/* #]GIF_onmove:	*/ 
/* #[GIF_onstone:	*/

void
GIF_onstone(g, n, c, p)
GOGAME *g;
int	n;
int	c;
PLACE	p;
{
    int i = P2a(g,p);
    int j = g->gm_sz - P2J(g,p);

    if (i >= 'i') i++;	/* Skip the 'i' */

    if (caption) fprintf(stderr, ", ");
    fprintf(stderr, "%c%d at (%c,%d)",
	    (c == BLACK) ? 'B' : 'W', n, i, j);
    caption++;
}

/* #]GIF_onstone:	*/ 
/* #[GIF_pass:		*/

void
GIF_pass(g, n, c)
GOGAME *g;
int	n;
int	c;
{
    if (caption) fprintf(stderr, ", ");
    fprintf(stderr, "%c%d pass", (c == BLACK) ? 'B' : 'W', n);
    caption++;
}

/* #]GIF_pass:		*/ 
/* #[GIF_time:		*/

void
GIF_time(g, btime, wtime)
/*
 * For the moment we ignore the time on the clock
 */
GOGAME *g;
char *btime;
char *wtime;
{
}

/* #]GIF_time:		*/ 
/* #[GIF_emit:		*/

int
GIF_emit(g, m1, m2, t1, t2)
GOGAME *g;
GOMOVE *m1, *m2;	/* Diagram from m1 to m2	*/
GOMOVE *t1, *t2;	/* Text    from t1 to t2	*/
{
    PLACE	p;
    char *	name	   = board_caption(g, m1, m2);
    static int	fignr	   = 0;
    static int	dianr	   = 0;
    int		n1	   = 999;		/* The 1st move number	*/
    int		n2	   = 0;			/* The lst move number	*/
    int		inreg	   = 0;

    /*
     * Print the position
     */
    if (g->gm_parent)	gif_init(g, ++dianr, name);
    else		gif_init(g, ++fignr, name);
    xvmin = g->gm_vw[0];
    yvmin = g->gm_vw[1];
    xvmax = g->gm_vw[2]; if (!xvmax) xvmax = g->gm_sz-1;
    yvmax = g->gm_vw[3]; if (!yvmax) yvmax = g->gm_sz-1;
    xrmin = g->gm_rg[0];
    yrmin = g->gm_rg[1];
    xrmax = g->gm_rg[2]; if (!xrmax) xrmax = g->gm_sz-1;
    yrmax = g->gm_rg[3]; if (!yrmax) yrmax = g->gm_sz-1;
    for (p = 0; p < g->gm_sz*g->gm_sz; p++) {
	GOMOVE *m = g->gm_board + p;

	/*
	 * Check view port
	 */
	if (p % g->gm_sz < xvmin ||
	    p % g->gm_sz > xvmax) continue;
	if (p / g->gm_sz < yvmin ||
	    p / g->gm_sz > yvmax) continue;
	if (p % g->gm_sz == xvmin) fprintf(fp, "pnmcat -lr");

	/*
	 * Set region flag
	 */
	inreg = (xrmin <= p % g->gm_sz && p % g->gm_sz <= xrmax &&
		 yrmin <= p / g->gm_sz && p / g->gm_sz <= yrmax);

	if (m->mv_letter)
		fprintf(fp, " l/%c", m->mv_letter->ch_letter);
	else
	if (m->mv_mark) {
	    if ((int)(*(m->mv_mark->ch_mark)) < LETTER_MIN ||
		(int) strlen(m->mv_mark->ch_mark) == 1) {
		char *mark  = NULL;

		switch ((int)(*(m->mv_mark->ch_mark))) {
		case SQUARE_EMPTY   : mark = "e"	; break;
		case SQUARE_FILLED  : mark = "sf"	; break;
		case TRIANGLE_EMPTY : mark = "t"	; break;
		case TRIANGLE_FILLED: mark = "tf"	; break;
		case '+'	    : mark = "p"	; break;
		case '-'	    : mark = "m"	; break;
		default		    :			  break;
		}
		if (mark) {
		    fprintf(fp, " %cm/%s",
			    (m->mv_side == BLACK) ? 'b' : 'w', mark);
		} else {
		    fprintf(fp, " n/%c",
			    (m->mv_side == BLACK) ? 'b' : 'w');
		    fprintf(stderr,
			    "Unable to convert mark at (%d,%d)\n",
			    P2I(g, p), P2J(g, p));
		}
	    }
	    else {
		    fprintf(fp, " n/%c",
			    (m->mv_side == BLACK) ? 'b' : 'w');
		    fprintf(stderr,
			    "Unable to convert mark at (%d,%d)\n",
			    P2I(g, p), P2J(g, p));
	    }
	}
	else
	if (m->mv_side) {
	    if (m->mv_nr) {
		int c;

		/*
		 * numbered stone
		 */
		if (m->mv_nr < n1) n1 = m->mv_nr;
		if (m->mv_nr > n2) n2 = m->mv_nr;

		c = ((m->mv_nr-1) % 100) + 1;
		fprintf(fp, " %cn/%d", (m->mv_side == BLACK) ? 'b' : 'w', c);
	    } else {
		/*
		 * stone
		 */
		fprintf(fp, " n/%c", (m->mv_side == BLACK) ? 'b' : 'w');
	    }
	}
	else {
	    /*
	     * Empty point
	     */
	    if (hoshi(g, p)) fprintf(fp, " n/h");
	    else {
		char *s = border(g, p);
		if (s) fprintf(fp, " n/%s", s);
		else   fprintf(fp, " n/%c", 'e');
	    }
	}

	if (p % g->gm_sz == xvmax) {
	    /*
	     * We're done with this scanline
	     */
	    fprintf(fp, "> ${TMP}/%02d.ppm\n", p / g->gm_sz);
	}
    }
    gif_exit((g->gm_parent == NULL),
	     (g->gm_parent == NULL) ? fignr : dianr, name);
    

    return 1;
}

/* #]GIF_emit:		*/ 
/* #[border:		*/

static char *
border(g, p)
GOGAME *g;
int     p;
{
    int sz = g->gm_sz;
    int x  = (p % sz)+1;
    int y  = (p / sz)+1;

    if (x ==  1 && y ==  1) return "ul";
    if (x == sz && y ==  1) return "ur";
    if (x ==  1 && y == sz) return "ll";
    if (x == sz && y == sz) return "lr";
    if (x ==  1)	    return "l" ;
    if (x == sz)	    return "r" ;
    if (y ==  1)	    return "u" ;
    if (y == sz)	    return "d" ;

    return NULL;
}

/* #]border:		*/ 
/* #[hoshi:		*/

#define ABS(n)	(((n) > 0) ? (n) : -(n))

static int
hoshi(g, p)
GOGAME *g;
int     p;
{
    int sz = g->gm_sz;
    int x  = (p % sz)+1;
    int y  = (p / sz)+1;
    int w  = (sz == 19) ? 6 : (sz == 13) ? 4 : 1000;

    return (ABS(x - ((sz+1)/2)) % w == 0 &&
	    ABS(y - ((sz+1)/2)) % w == 0);
}

/* #]hoshi:		*/ 
