/* #[info:		*/
/************************************************************************
*                                                                       *
*                   |\         ,,      ,           ,,                   *
*                    \\        ||     ||           ||                   *
*                   / \\       ||/\\ =||= \\/\\/\\ ||                   *
*                  || ||       || ||  ||  || || || ||                   *
*                  || ||       || ||  ||  || || || ||                   *
*                   \\/        \\ |/  \\, \\ \\ \\ \\                   *
*                        _____   _/                                     *
*                                                                       *
*                                                                       *
*                Copyright (c) 1993/94 Jan van der Steen                *
*              Centre for Mathematics and Computer Science              *
*                      Amsterdam, the Netherlands                       *
*                                                                       *
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.    *
*   You are allowed to use this software in free software under the     *
*   condition that you maintain this integral message in the source.    *
*-----------------------------------------------------------------------*
* File    : d_html.h							*
* Purpose : Write an HTML (and GIF) file from internal game structure	*
* Version : 0                                                           *
* Modified: 5/16/95                                                     *
* Author  : Thierry.EXCOFFIER@ligia.univ-lyon1.fr                       *
* Author  : Jan van der Steen (jansteen@cwi.nl)                         *
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "tools.h"
#include "d_html.h"
#include "d_gif.h"
#include "textools.h"

extern EPSFINFO epsf ;

void text2HTML() ;

extern	FILE *		fw;		/* Output stream		*/

extern	EPSFINFO	epsf;		/* The eps filename/directory	*/
extern	char *		program;	/* Name of this program		*/
extern	int		statout;	/* Statistics report ?		*/
extern	int		kibitz;		/* Are we processing kibitz ?	*/
extern	int		labels;		/* Labels with move numbers	*/
extern	int		zhuge;		/* Apply the "zhuge" conversion	*/
extern  int             flatten;

static	int		dianr = 0;
static	int		fignr = 0;


/* #[HTML_header:	*/

void
HTML_header(g)
GOGAME *g;
{
    static GOGAME *	gg;
    static int		done = 0;

    if (done) g = gg;
    /*
     * Title page
     */
    fprintf(fw, "<HTML>\n");
    fprintf(fw, "<HEAD>\n");
    fprintf(fw, "<TITLE>\n");
	text2HTML((g->gm_pb) ? g->gm_pb : "NN");
	fprintf(fw, ", ");
	text2HTML((g->gm_br) ? g->gm_br : "NR");
	fprintf(fw, " versus ");
	text2HTML((g->gm_pw) ? g->gm_pw : "NN");
	fprintf(fw, ", ");
	text2HTML((g->gm_wr) ? g->gm_wr : "NR");
	if (g->gm_dt) {
	    fprintf(fw, " (");
	    text2HTML(g->gm_dt);
	    fprintf(fw, ")");
	}
    fprintf(fw, "</TITLE>\n");
    fprintf(fw, "</HEAD>\n");

    fprintf(fw, "<BODY>\n");
    fprintf(fw, "<H1>");
	text2HTML((g->gm_pb) ? g->gm_pb : "NN");
	fprintf(fw, ", ");
	text2HTML((g->gm_br) ? g->gm_br : "NR");
	fprintf(fw, " <EM>versus</EM> ");
	text2HTML((g->gm_pw) ? g->gm_pw : "NN");
	fprintf(fw, ", ");
	text2HTML((g->gm_wr) ? g->gm_wr : "NR");
	if (g->gm_dt) {
	    fprintf(fw, " (");
	    text2HTML( g->gm_dt );
	    fprintf(fw, ")");
	}
    fprintf(fw, "</H1>\n");

    /*
     * Game header (players, rank, date, place and result)
     */
    if (!done) {
	fprintf(fw, "Black: ");
		    text2HTML((g->gm_pb) ? g->gm_pb : "NN");
		    fprintf(fw, " ");
		    text2HTML((g->gm_br) ? g->gm_br : "NR");
	fprintf(fw, "<BR>\n");

	fprintf(fw, "White: ");
		    text2HTML((g->gm_pw) ? g->gm_pw : "NN");
                    fprintf(fw, " ");
		    text2HTML((g->gm_wr) ? g->gm_wr : "NR");
	fprintf(fw, "<BR>\n");

	if (g->gm_dt) {
	    fprintf(fw, "Date: ");
	    text2HTML(g->gm_dt);
	    fprintf(fw, "<BR>\n");
	}
	
	if (g->gm_pc) {
	    fprintf(fw, "Place: ");
	    text2HTML(g->gm_pc);
	    fprintf(fw, "<BR>\n");
	}
	
	if (g->gm_ha) {
	    fprintf(fw, "Handicap: ");
	    text2HTML(g->gm_ha);
	    fprintf(fw, "<BR>\n");
	}
	
	if (g->gm_km) {
	    fprintf(fw, "Komi: ");
	    text2HTML(g->gm_km) ;
	    fprintf(fw, "<BR>\n");
	}
	
	if (g->gm_re) {
	    fprintf(fw, "Result: ");
	    text2HTML(g->gm_re) ;
	    fprintf(fw, "<BR>\n");
	}
	fprintf(fw, "<PRE>\n");
    }
    done = 1;
    gg   = g;
}

/* #]HTML_header:	*/ 
/* #[HTML_trailer:	*/

void
HTML_trailer()
{
    static int done = 0;

    if (done) return;
    fprintf(fw, "</PRE>\n");

    if (statout) {
	/*#[Name:*/

	fprintf(stdout, "<H2>Kibitzers sorted by name</H2>\n");
	kib_list(0, "kdict");

	/*#]Name:*/ 
	/*#[Rank:*/

	fprintf(stdout, "<H2>Kibitzers sorted by rank</H2>\n");
	kib_list(1, "krank");

	/*#]Rank:*/ 
	/*#[Freq:*/

	fprintf(stdout, "<H2>Kibitzers sorted by frequency</H2>\n");
	kib_list(2, "kfreq");

	/*#]Freq:*/ 
	/*#[Byte:*/

	fprintf(stdout, "<H2>Kibitzers sorted by amount</H2>\n");
	kib_list(3, "kbyte");

	/*#]Byte:*/ 
    }
    fprintf(fw, "</BODY>\n");
    fprintf(fw, "</HTML>\n");
    fputc('\n', stderr);	/* To flush the messages on stderr */

    done = 1;
}

/* #]HTML_trailer:	*/ 
/* #[HTML_text:		*/

void
HTML_text(t1, t2, n1, n2)
GOMOVE *t1;
GOMOVE *t2;
int	n1;
int	n2;
/*
 * Print the text associated with a position
 */
{
    GOMOVE *m;
    char    username[MAXUSER];		/* Name of last kibitzer */

    if (!t1) return;

    fprintf(fw, "<DL>\n");

    for (m = t1; m != t2; m = m->mv_next) {
	TEXT   *t;

	if (!m->mv_text) continue;

/*
	  #[statistics:
*/
	if (kibitz) kib_add(m->mv_text);
/*
	  #]statistics: 
	  #[labels:
*/
	if (m->mv_nr) {
	    username[0] = 0;
	    if (labels && n1 != n2) {
		fprintf(fw, "<DT>" ) ;
		fprintf(fw, "<STRONG>%c%d</STRONG>",
			(m->mv_side == BLACK) ? 'B' : 'W', m->mv_nr);
		fprintf(fw, "<DD> %s\n", (kibitz) ? "<P>" : "");
	    }
	}
/*
	  #]labels: 
	  #[text:
*/
	for (t = m->mv_text; t; t = t->cc_next) {
/*
	  #[kibitz:
*/
	    if (kibitz) {
		char rankstr[10];
		char *r;

		if (strcmp(username, t->cc_name)) {
		    strcpy(username, t->cc_name);
		    strcpy(rankstr, rank2str(t->cc_spec));
		    for (r = rankstr; *r == ' '; r++);
		    fprintf(fw, "<STRONG>%s %s</STRONG>\n<LISTING>",
			    t->cc_name, r);
		    text2HTML(t->cc_text);
		    fprintf(fw, "</LISTING>\n");
		} else {
		    fprintf(fw, "<LISTING>");
		    text2HTML(t->cc_text);
		    fprintf(fw, "</LISTING>\n");
		}
		continue;
	    }
/*
	  #]kibitz: 
	  #[non-zhuge:
*/
	    if (!zhuge) { text2HTML(t->cc_text); continue; }
/*
	  #]non-zhuge: 
	  #[zhuge:
*/
	    {
		/*
		 * Author's notes are in "(*" "*)" blocks
		 */
		char *s		 = t->cc_text;
		char *note_start = strstr(t->cc_text, "(*");
		char *note_end	 = strstr(t->cc_text, "*)");
		int   restore    = 0;

		if (note_start) {
		    fprintf(fw, "<EM>\n");
		    s = note_start+2;
		    if (*s == ' ') s++;
		}
		if (note_end) {
		if (*(note_end-1) == ' ') note_end--;
		    restore = *note_end;
		    *note_end = 0;
		}
		text2HTML(s);
		if (note_end) {
		    fprintf(fw, "\n</EM>\n");
		    *note_end = restore; /* restore line */
		}
	    }
/*
	  #]zhuge: 
*/
	}
/*
	  #]text: 
*/

    }
    fprintf(fw,"\n</DL>\n");		/* create a paragraph */
}

/* #]HTML_text:		*/ 
/* #[text2HTML:		*/

void
text2HTML(t)
/*
 * Copy t to fw while taking care of HTML special characters.
 */
char *t;
{
    for (; *t; t++) {
	if (*t > '~')
	    fprintf(fw, "&#%o;", *t);
	else
	    switch(*t) {
	    case '&'  : fputs("&amp;"	,fw); break;
	    case '<'  : fputs("&lt;"	,fw); break;
	    case '>'  : fputs("&gt;"	,fw); break;
	    case '\r' :
	    case '\n' : fputs(" "	,fw); break;
	    default   : putc(*t, fw);
	    }
    }
}

/* #]text2HTML:		*/ 
/* #[define:		*/

#define P2HANNA_I(g,p)	(P2b(g,p) + (P2J(g,p) > 8 ? 1 : 0))
#define P2HANNA_J(g,p)	(P2I(g,p) + 1)

/* #]define:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

static char *	n2hanna		PROTO((int n, int c));

#undef PROTO

/* #]prototype:		*/ 
 
/* #[n2hanna:		*/

static char *
n2hanna(n, color)
/*
 * The Hanna fonts have numbered stones up to 252.
 * We want to use them in captions if possibly.
 */
int n;
int color;
{
    static char HTMLnum[128];

    if (n <= 252)
	sprintf(HTMLnum,
		"%c%d",
		(color == BLACK) ? 'B' : 'W',
		n);
    else
	sprintf(HTMLnum,
		"%c%d",
		(color == BLACK) ? 'B' : 'W',
		n);

    return HTMLnum;
}

/* #]n2hanna:		*/ 

/* #[HTML_init:		*/

int
HTML_init()
{
    return GIF_init();
}

/* #]HTML_init:		*/ 
/* #[HTML_exit:		*/

void
HTML_exit()
{
#ifndef NO_GIF
    GIF_exit();
#endif
    HTML_trailer();
    if (fw != stdout) fclose(fw);
}

/* #]HTML_exit:		*/ 
/* #[HTML_alloc:	*/

int
HTML_alloc(g)
GOGAME *g;
{
    return GIF_alloc(g);
}

/* #]HTML_alloc:	*/ 
/* #[HTML_free:		*/

void
HTML_free(g)
GOGAME *g;
{
    GIF_free(g);
}

/* #]HTML_free:		*/ 
/* #[HTML_onmove:	*/

void
HTML_onmove(g, n1, c1, n2, c2)
GOGAME *g;
int	n1;
int	c1;
int	n2;
int	c2;
{
    char onmove[128];
#ifndef NO_GIF
    GIF_onmove(g, n1, c1, n2, c2) ;
#endif
    strcpy(onmove, n2hanna(n1, c1));
    strcat(onmove, " at ");
    strcat(onmove, n2hanna(n2, c2));
    con_printf(onmove);
}

/* #]HTML_onmove:	*/ 
/* #[HTML_onstone:	*/

void
HTML_onstone(g, n, c, p)
GOGAME *g;
int	n;
int	c;
PLACE	p;
{
#ifndef NO_GIF
    GIF_onstone(g, n, c, p) ;
#endif
    con_printf("%s at (%c,%c)", n2hanna(n, c), P2a(g,p), P2b(g,p));
}

/* #]HTML_onstone:	*/ 
/* #[HTML_pass:		*/

void
HTML_pass(g, n, c)
GOGAME *g;
int	n;
int	c;
{
    char pass[128];
#ifndef NO_GIF
    GIF_pass(g, n, c) ;
#endif
    strcpy(pass, n2hanna(n, c));
    strcat(pass, " pass");
    con_printf(pass);
}

/* #]HTML_pass:		*/ 
/* #[HTML_time:		*/

void
HTML_time(g, btime, wtime)
/*
 * For the moment we ignore the time on the clock
 */
GOGAME *g;
char *btime;
char *wtime;
{
#ifndef NO_GIF
    GIF_time(g, btime, wtime) ;
#endif
}

/* #]HTML_time:		*/ 


void numbering(g)
GOGAME *g;
{
    GOMOVE *m;
    static int diagnr = 1;

    if (!g) return;
    g->gm_did = (g->gm_parent) ? diagnr++ : 0;

    for (m = g->gm_move; m; m = m->mv_next) numbering(m->mv_variation);
    numbering(g->gm_next);
}


void HTML_anchor(n)
int n;
{
    if (n == 0) n = -fignr;
    fprintf(fw, "<A HREF=\"%s-%c%02d.html\">",
	    epsf.eps_name,
	    (n < 0) ? 'f' : 'd',
	    (n < 0) ? -n  : n);
}


void HTML_anchor_text(g, t)
GOGAME *g;
char   *t;
{
    char nn[1024];
    char *n;
    int i;

    strcpy(nn, board_caption(g, g->gm_move, g->gm_move));

    n = nn;

    if ((n[0] == 'B' || n[0] == 'W') && n[1] == ' ') {
	for (i = 2; n[i]; i++)
	    if (n[i] < '0' || n[i] > '9') break;
    }
    else i = 0;

    if (n[i]) {
	while (n[0] == ' ') n++;
	if (n[0]) while (n[strlen(n)-1] == ' ') n[strlen(n)-1] = 0; 
	fprintf(fw, "%s (%s)</A>", t, n) ;
    }
    else fprintf(fw, "%s</A>", t);
}


void HTML_index(g,name)
GOGAME *g ;
char *name ;
{
    GOGAME *r;
    static int first = 1;

    if (first) {
	first = 0 ;
	HTML_header(g); 
	fprintf(fw,"<P>\n") ;
    }
    if (g->gm_parent) {
	for (r = g; r->gm_parent; r = r->gm_parent) fprintf(fw," |");
	HTML_anchor(dianr);
	fprintf(fw,"Diagram %d ",dianr);
	HTML_anchor_text(g,""); 
    }
    else {
	HTML_anchor(-fignr);
	fprintf(fw,"Figure %d",fignr);
	HTML_anchor_text(g,"");
	fprintf(fw," (%s)",name);
    }
    fprintf(fw,"<BR>\n") ;
}


/* #[HTML_emit:		*/

int
HTML_emit(g, m1, m2, t1, t2)
GOGAME *g;
GOMOVE *m1, *m2;	/* Diagram from m1 to m2	*/
GOMOVE *t1, *t2;	/* text    from t1 to t2	*/
{
    char *	name ;
    int		n1 = 999;			/* The 1st move number	*/
    int		n2 = 0;				/* The lst move number	*/
    FILE *	save;
    GOGAME *	gg;
    GOMOVE *	m;
    static int  numbered = 0;
    char        filenam[1024];
    char        figname[1024];
    int		fwr;

#ifndef NO_GIF
    GIF_emit(g, m1, m2, t1, t2) ; 
#endif

    if (!numbered) {
	if (fw == stdout) {
	    sprintf(filenam, "%s.html", epsf.eps_name);
	    if ((fw = fopen(filenam,"w")) == NULL) {
		perror(filenam);
		exit(1);
	    }
	}
	numbering(g);
	numbered = 1;
	message("d_html: numbered\n");
	if (!flatten) {
	    message("\nYou could use -flatten options\n");
	    message("in order to have the best result\n\n");
	}
    }

    if (g->gm_parent) {
	++dianr;
	sprintf(filenam, "%s-d%02d.html", epsf.eps_name, dianr);
	if (dianr != g->gm_did) {
	    fprintf(stderr, "dianr %d number=%d\n", dianr, g->gm_did);
	    exit(1);
	}
    }
    else {
	++fignr;
	sprintf(filenam, "%s-f%02d.html", epsf.eps_name, fignr);
	for (m = m1; m != m2; m = m->mv_next) {
	    for (gg = m->mv_variation; gg ; gg = gg->gm_next)
		gg->gm_fid = fignr;
	}
    }
    strcpy(figname, board_caption(g, m1, m2));
    HTML_index(g, figname); 
    name = board_caption(g, m1, m2);

    save = fw;
    if ((fw = fopen(filenam, "w")) == NULL) {
	perror("fopen");
	exit(1);
    }
    HTML_header(g);

/*	#[position:		*/
    /*
     * Print the position
     */

    fprintf(fw, "\n<H2>");

    if (g->gm_parent) fprintf(fw, "Diagram %d: \n", dianr);
    		 else fprintf(fw, "Figure %d: \n", fignr);
    text2HTML(name);

    fprintf(fw, "</H2>\n");
    message("[%d (%s)]", dianr, name);

    /*
     * Index/Backward/Forward
     */
    fprintf(fw, "In the real game: <A HREF=\"%s.html\">Index</A> *\n",
	    epsf.eps_name);

    if (g->gm_parent==0 && fignr != 1) {
        HTML_anchor     (-(fignr-1)) ;
        HTML_anchor_text(g,"Backward");
    }
    else {
	fprintf(fw,"Backward") ;
    }
    
    if (g->gm_parent == 0 && t2 && t2->mv_next) {
	fprintf(fw," * ") ;	       
	HTML_anchor     (-(fignr+1)) ;
	HTML_anchor_text(g,"Forward");
	fwr = 1 ;
    }
    else {
	fprintf(fw," * Forward") ;
    }

    /*
     * Parent/Next/Previous
     */
    fprintf(fw,"<BR>Variations: ");

    if (g->gm_parent) {
	if (g->gm_parent->gm_parent)
	    HTML_anchor(g->gm_parent->gm_did);
	else
	    HTML_anchor(-g->gm_fid);
      HTML_anchor_text(g->gm_parent,"Parent");
    }
    else {
	fprintf(fw,"Parent") ;
    }
    fprintf(fw, " * ");
    
    if (g->gm_prev) {
	HTML_anchor     (g->gm_prev->gm_did);
	HTML_anchor_text(g->gm_prev,"Previous");
    }
    else {
	fprintf(fw,"Previous");
    }
    fprintf(fw, " * ");

    if (g->gm_next) {
	HTML_anchor     (g->gm_next->gm_did);
	HTML_anchor_text(g->gm_next,"Next");
    }
    else {
	fprintf(fw,"Next");
    }

    /*
     * Parent/Next/Previous
     */
    fprintf(fw, "\n<BR>From this diagram: ");
    gg  = g;
    fwr = 1;
    for (m = m1; m != m2->mv_next; m = m->mv_next) {
	for (gg = m->mv_variation; gg; gg = gg->gm_next) {
	    if (fwr != 1) fprintf(fw," * ");
	    sprintf(filenam, "Variation %d", fwr++);
	    HTML_anchor     (gg->gm_did);
	    HTML_anchor_text(gg, filenam);
	}
    }
    fprintf(fw, "\n<BR>\n");

    if (g->gm_parent) {
	fprintf(fw,
		"<P>\n<IMG SRC=\"%s-d%02d.gif\" ALT=\"diagram %d\">\n<P>\n",
		epsf.eps_name, dianr, dianr);
    }
    else {
	fprintf(fw, 
		"<P>\n<IMG SRC=\"%s-f%02d.gif\" ALT=\"figure %d\">\n<P>\n",
		epsf.eps_name, fignr, fignr);
    }

/*	#]position:		*/ 
/*	#[connections/caption:	*/

    /*
     * Flush the ko/connect messages and give it a title
     */
    fprintf(fw, "<TT>%s</TT><BR>\n", con_cat((char *) 0));


/*	#]connections/caption:	*/ 
/*	#[text:			*/

    /*
     * Print the text associated with the position
     */
    HTML_text(t1, t2, n1, n2);

/*	#]text:			*/ 

    fprintf(fw, "</BODY>\n");
    fprintf(fw, "</HTML>\n");
    fclose(fw);
    fw = save;
    return 1;
}

/* #]HTML_emit:		*/ 
