/* #[info:		*/
/************************************************************************
*									*
*		 |\			  ,				*
*		  \\			 ||	   ,			*
*		 / \\	    -_-_   _-_, =||=  _-_  \\ /`		*
*		|| ||	    || \\ ||_.	 ||  || \\  \\			*
*		|| ||	    || ||  ~ ||	 ||  ||/    /\\			*
*		 \\/	    ||-'  ,-_-	 \\, \\,/  /  \;		*
*		      _____ |/						*
*			    '						*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : d_pstex.c							*
* Purpose : Dump a Go data structure to PS (diagrams) and TeX (text)	*
* Version : 1.33							*
* Modified: 04 Apr 1995 11:49:25					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "textools.h"
#include "tools.h"
#include "d_pstex.h"

/* #]include:		*/ 
/* #[define:		*/

#define	EPSF_I(g,p)	((p) % g->gm_sz)
#define	EPSF_J(g,p)	((g->gm_sz-1) - ((p)/g->gm_sz))

/* #]define:		*/ 
/* #[typedef:		*/

typedef struct afm_info {
    int		afm_w;
    int		afm_llx;
    int		afm_lly;
    int		afm_urx;
    int		afm_ury;
} AFM;

typedef struct fnt_info {
    char *	fnt_name;
    AFM		fnt_afm[10];
    int		fnt_base;
    int		fnt_top;
    int		fnt_kern;	/* '1' kerning */
} FNT;

typedef struct fm_info {
    char *	fm_psname;	/* PS variable	*/
    int		fm_def;		/* Default	*/
    int		fm_set;		/* Set value	*/
} FNTMAP;

/* #]typedef:		*/ 
/* #[font metrics:	*/

static FNT fnt[] = {
    {
	"Helvetica", {
	    { 556,  43, -23, 507, 709},	/* 0 */
	    { 556, 102,   0, 347, 709},	/* 1 */
	    { 556,  34,   0, 511, 710},	/* 2 */
	    { 556,  32, -23, 506, 709},	/* 3 */
	    { 556,  28,   0, 520, 709},	/* 4 */
	    { 556,  35, -23, 513, 709},	/* 5 */
	    { 556,  43, -23, 513, 709},	/* 6 */
	    { 556,  46,   0, 520, 709},	/* 7 */
	    { 556,  37, -23, 513, 709},	/* 8 */
	    { 556,  38, -23, 509, 709},	/* 9 */
	}, -23, 710, 74
    },
    {
	"Helvetica-Bold", {
	    { 556,  29, -23, 517, 725},	/* 0 */
	    { 556,  68,   0, 378, 709},	/* 1 */
	    { 556,  30,   0, 515, 726},	/* 2 */
	    { 556,  29, -23, 516, 726},	/* 3 */
	    { 556,  24,   0, 522, 709},	/* 4 */
	    { 556,  27, -24, 517, 709},	/* 5 */
	    { 556,  32, -23, 519, 727},	/* 6 */
	    { 556,  29,   0, 528, 709},	/* 7 */
	    { 556,  22, -23, 525, 726},	/* 8 */
	    { 556,  28, -23, 516, 728},	/* 9 */
	}, -24, 728, 55
    },
    {
	"Futura-Condensed", {
	    { 411, 34, -10, 376, 764},	/* 0 */
	    { 411, 83,   0, 255, 754},	/* 1 */
	    { 411, 37,   0, 376, 764},	/* 2 */
	    { 411, 41, -10, 373, 764},	/* 3 */
	    { 411, 29,   0, 378, 754},	/* 4 */
	    { 411, 42, -10, 371, 754},	/* 5 */
	    { 411, 43, -10, 368, 754},	/* 6 */
	    { 411, 38,   0, 375, 754},	/* 7 */
	    { 411, 43, -10, 369, 764},	/* 8 */
	    { 411, 44,   0, 368, 764},	/* 9 */
	}, -10, 764, 0
    },
    {
	"Futura-CondensedBold", {
	    { 490,  33, -12, 457, 766},	/* 0 */
	    { 490, 110,   0, 327, 754},	/* 1 */
	    { 490,  41,   0, 450, 766},	/* 2 */
	    { 490,  34, -12, 455, 766},	/* 3 */
	    { 490,  28,   0, 453, 754},	/* 4 */
	    { 490,  49, -12, 442, 754},	/* 5 */
	    { 490,  37, -12, 454, 754},	/* 6 */
	    { 490,  41,   0, 480, 754},	/* 7 */
	    { 490,  29, -12, 462, 766},	/* 8 */
	    { 490,  37,   0, 454, 766},	/* 9 */
	}, -12, 766, 0
    },
};
#define PHVR	0
#define PHVB	1
#define PFURRC	2
#define PFUBRC	3

static FNTMAP fnt_map[4] = {
    { "wFont", PHVR, PHVR },	/* White stone numbers	*/
    { "bFont", PHVB, PHVB },	/* Black stone numbers	*/
    { "fFont", PHVR, PHVR },	/* Grid markers		*/
    { "mFont", PHVR, PHVR },	/* Metamark font	*/
};

#define	FONT(c)	(((c) == WHITE) ? fnt_map[0].fm_set : fnt_map[1].fm_set)

/* #]font metrics:	*/ 
/* #[static:		*/

static	FILE *	fp;			/* EPSF file			*/
static	char	epsffile[1024];		/* Name of current EPSF file	*/

/* #]static:		*/ 
/* #[extern:		*/

extern	FILE *		fw;		/* Output stream		*/
extern	int		prn_dev;	/* The device number		*/
extern	EPSFINFO	epsf;		/* The eps filename/directory	*/
extern	char *		program;	/* Name of this program		*/
extern	int		caching;	/* Save on output ?		*/
extern	int		coords;		/* Diagrams with coordinates ?	*/

/* #]extern:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

static int	eps_init	PROTO((GOGAME *g, int n, char *title));
static void	eps_exit	PROTO((int n, char *title));
static char *	eps_nstone	PROTO((GOGAME *g, int n, int c));
static void	eps_metric	PROTO((FILE *fp, int n, int c, int m[]));

#undef PROTO

/* #]prototype:		*/ 

/* #[eps_metric:	*/

static void
eps_metric(fp, N, c, m)
/*
 * Return metrics of black blob created by number n in c-colored stone.
 * We fill x_offset, y_offset, width and height.
 */
FILE *fp;
int   N;
int   c;
int   m[];
{
    int   fi  = FONT(c);
    AFM * afm = fnt[fi].fnt_afm;
    AFM * a   = NULL;
    int   n;
    int   d;
    int   string = 0;
    char  stack[10];

    fprintf(fp, "mark ");
    m[3] = fnt[fi].fnt_top;
    for (n = N; n; n /= 10) {
	a     = afm+(n % 10);
	if (n == N) m[2]  = a->afm_urx;
	else	    m[2] += a->afm_w;
	if (n % 10 == 1) {
	    if ((n/10) % 10 == 1) d = fnt[fi].fnt_kern;
	    else		  d = fnt[fi].fnt_kern/2;
	} else d = 0;
	if (d) {
	    if (n/10) m[2] -= d;
	    if (string) {
		do { fputc(stack[--string], fp); } while (string);
		fprintf(fp, ") 0 ");
	    }
	    fprintf(fp, "(1) %d ", d);
	} else {
	    if (!string) fprintf(fp, "(");
	    stack[string++] = '0' + n % 10;
	}
    }
    if (string) {
	do { fputc(stack[--string], fp); } while (string);
	fprintf(fp, ") 0 ");
    }
    m[0]  = a->afm_llx;
    m[2] -= m[0];
    m[1]  = fnt[fi].fnt_base;
    m[3] -= m[1];
}

/* #]eps_metric:	*/ 
/* #[eps_nstone:	*/

static char *
eps_nstone(g, n, c)
/*
 * Create an EPSF file to print a numbered stone
 */
GOGAME *g;
int	n;
int	c;
{
    FILE *	fp;
    long	tenth_mm  = 254L*epsf.eps_size/72;
    int		m[4];
    int		i;
    static char	epsfname[1024];

    /*
     * Construct the epsf filename
     */
    sprintf(epsfname, "%s%c%c.%03d",
	    epsf.eps_dir,
	    DIRSEP,
	    (c == BLACK) ? 'b' : 'w', n);

    if ((fp = fopen(epsfname, "w")) == (FILE *) 0) {
	message("Couldn't create: \"%s\"\n", epsfname);
	exit(1);
    }

    fprintf(fp, "%%!PS-Adobe-2.0 EPSF-1.2\n");
    fprintf(fp, "%%%%Title: %s stone numbered %d\n",
		(c == BLACK) ? "Black" : "White", n);
    fprintf(fp, "%%%%Creator: %s\n", program);
    fprintf(fp, "%%%%BoundingBox: 0 0 %d %d\n", 9, 9);
    fprintf(fp, "%%%%DocumentFonts: %s %s %s %s\n",
	    fnt[fnt_map[0].fm_set].fnt_name,
	    fnt[fnt_map[1].fm_set].fnt_name,
	    fnt[fnt_map[2].fm_set].fnt_name,
	    fnt[fnt_map[3].fm_set].fnt_name);
    fprintf(fp, "%%%%EndComments\n");
    fprintf(fp, "Liberty begin\n");
    fprintf(fp, "/cache false def\n");
    fprintf(fp, "/axis  false def\n");
    for (i = 0; i < sizeof(fnt_map)/sizeof(FNTMAP); i++) {
	if (fnt_map[i].fm_def != fnt_map[i].fm_set) {
	    fprintf(fp, "/%s /%-20s def\n",
		    fnt_map[i].fm_psname,
		    fnt[fnt_map[i].fm_set].fnt_name);
	}
    }
    fprintf(fp, "%d %ld.%02ld boardinit\n", g->gm_sz,
		tenth_mm/100, tenth_mm - 100*(tenth_mm/100));

    /*
     * Get metrics
     */
    eps_metric(fp, n, c, m);

    if (c == BLACK) {
	fputc('\n', fp);
	fprintf(fp, "/bB %d def\n", m[1]);
	fprintf(fp, "/bH %d def\n", m[3]);
    } else {
	fputc('\n', fp);
	fprintf(fp, "/wB %d def\n", m[1]);
	fprintf(fp, "/wH %d def\n", m[3]);
    }
    fprintf(fp, "%d %d 0 0 %cst\n", m[0], m[2], (c == BLACK) ? 'b' : 'w');
    fprintf(fp, "end\n");
    fprintf(fp, "%%%%Trailer\n");
    fclose (fp);

    return epsfname;
}

/* #]eps_nstone:	*/ 
/* #[eps_init:		*/

static int
eps_init(g, n, title)
/*
 * Open the EPSF file and print the header
 */
GOGAME *g;
int	n;
char *	title;
{
    char *	thetime;
    long	seconds;
    long	tenth_mm  = 254L*epsf.eps_size/72;
    int		i;

    /*
     * Get the creation time
     */
    time(&seconds);
    thetime = ctime(&seconds);

    /*
     * Construct the epsf filename
     */
    sprintf(epsffile, "%s%c%s.%c%02d",
	    epsf.eps_dir,
	    DIRSEP,
	    epsf.eps_name,
	    (g->gm_parent) ? 'd' : 'f',
	    n);
    if ((fp = fopen(epsffile, "w")) == (FILE *) 0) {
	message("Couldn't create: \"%s\"\n", epsffile);
	exit(1);
    }

    /*
     * Clipping region
     */
    if (!g->gm_vw[0]) g->gm_vw[0] = 0;
    if (!g->gm_vw[1]) g->gm_vw[1] = 0;
    if (!g->gm_vw[2]) g->gm_vw[2] = g->gm_sz-1;
    if (!g->gm_vw[3]) g->gm_vw[3] = g->gm_sz-1;

    fprintf(fp, "%%!PS-Adobe-2.0 EPSF-1.2\n");
    fprintf(fp, "%%%%Title: %s-%s (%s)\n",
		(g->gm_pw) ? g->gm_pw : "unknown",
		(g->gm_pb) ? g->gm_pb : "unknown",
		title);
    fprintf(fp, "%%%%Creator: %s\n", program);
    fprintf(fp, "%%%%CreationDate: %s", thetime);
    fprintf(fp, "%%%%BoundingBox: 0 0 %d %d\n",
	    ((g->gm_vw[2]-g->gm_vw[0]+1) * epsf.eps_width) / g->gm_sz,
	    ((g->gm_vw[3]-g->gm_vw[1]+1) * epsf.eps_width) / g->gm_sz);
    fprintf(fp, "%%%%DocumentFonts: %s %s %s %s\n",
	    fnt[fnt_map[0].fm_set].fnt_name,
	    fnt[fnt_map[1].fm_set].fnt_name,
	    fnt[fnt_map[2].fm_set].fnt_name,
	    fnt[fnt_map[3].fm_set].fnt_name);
    fprintf(fp, "%%%%EndComments\n");
    fprintf(fp, "Liberty begin\n");
    fprintf(fp, "/cache %s def\n", (caching) ? "true" : "false");
    fprintf(fp, "/axis  %s def\n", (coords ) ? "true" : "false");
    for (i = 0; i < sizeof(fnt_map)/sizeof(FNTMAP); i++) {
	if (fnt_map[i].fm_def != fnt_map[i].fm_set) {
	    fprintf(fp, "/%s /%-20s def\n",
		    fnt_map[i].fm_psname,
		    fnt[fnt_map[i].fm_set].fnt_name);
	}
    }
    fprintf(fp, "%d %ld.%02ld boardinit\n", g->gm_sz,
		tenth_mm/100, tenth_mm - 100*(tenth_mm/100));
    /*
     * Clipping region
     */
    fprintf(fp, "/llbdx %2d def\n", g->gm_vw[0]);
    fprintf(fp, "/llbdy %2d def\n", g->gm_sz-1-g->gm_vw[1]);
    fprintf(fp, "/urbdx %2d def\n", g->gm_vw[2]);
    fprintf(fp, "/urbdy %2d def\n", g->gm_sz-1-g->gm_vw[3]);

    /*
     * Region indication
     */
    fprintf(fp, "/llrgx %2d def\n", g->gm_rg[0]);
    fprintf(fp, "/llrgy %2d def\n", g->gm_sz-1-g->gm_rg[1]);
    fprintf(fp, "/urrgx %2d def\n", g->gm_rg[2]);
    fprintf(fp, "/urrgy %2d def\n", g->gm_sz-1-g->gm_rg[3]);

    fprintf(fp, "board\n");

    return 1;
}

/* #]eps_init:		*/ 
/* #[eps_exit:		*/

static void
eps_exit(n, title)
int	n;
char *	title;
{
    fprintf(fp, "end\n");
    fprintf(fp, "%%%%Trailer\n");
    fclose (fp);
    message("[%d (%s)]", n, title);
}

/* #]eps_exit:		*/ 

/* #[PSTeX_init:	*/

int
PSTeX_init()
{
    return 1;
}

/* #]PSTeX_init:	*/ 
/* #[PSTeX_exit:	*/

void
PSTeX_exit()
{
    tex_trailer();
    if (fw != stdout) fclose(fw);
}

/* #]PSTeX_exit:	*/ 
/* #[PSTeX_alloc:	*/

int
PSTeX_alloc(g)
GOGAME *g;
{
    return 1;
}

/* #]PSTeX_alloc:	*/ 
/* #[PSTeX_free:	*/

void
PSTeX_free(g)
GOGAME *g;
{
}

/* #]PSTeX_free:	*/ 
/* #[PSTeX_onmove:	*/

void
PSTeX_onmove(g, n1, c1, n2, c2)
GOGAME *g;
int	n1;
int	c1;
int	n2;
int	c2;
{
#if 0
    con_printf("%d at %d", n1, n2);
#else
    {
	char stone1[1024];

	strcpy(stone1, eps_nstone(g, n1, c1));
	con_printf("\\stone{%s} at \\stone{%s}",
		    stone1, eps_nstone(g, n2, c2));
    }
#endif
}

/* #]PSTeX_onmove:	*/ 
/* #[PSTeX_onstone:	*/

void
PSTeX_onstone(g, n, c, p)
GOGAME *g;
int	n;
int	c;
PLACE	p;
{
    int i = P2a(g,p);
    int j = g->gm_sz - P2J(g,p);

    if (i >= 'i') i++;	/* Skip the 'i' */

#if 0
    con_printf("%d at (%c,%d)", n, i, j);
#else
    con_printf("\\stone{%s} at (%c,%d)", eps_nstone(g, n, c), i, j);
#endif
}

/* #]PSTeX_onstone:	*/ 
/* #[PSTeX_pass:	*/

void
PSTeX_pass(g, n, c)
GOGAME *g;
int	n;
int	c;
{
#if 0
    con_printf("%d pass", n);
#else
    con_printf("\\stone{%s} pass", eps_nstone(g, n, c));
#endif
}

/* #]PSTeX_pass:	*/ 
/* #[PSTeX_time:	*/

void
PSTeX_time(g, btime, wtime)
/*
 * For the moment we ignore the time on the clock
 */
GOGAME *g;
char *btime;
char *wtime;
{
}

/* #]PSTeX_time:	*/ 
/* #[PSTeX_emit:	*/

int
PSTeX_emit(g, m1, m2, t1, t2)
GOGAME *g;
GOMOVE *m1, *m2;	/* Diagram from m1 to m2	*/
GOMOVE *t1, *t2;	/* Text    from t1 to t2	*/
{
    PLACE	p;
    char *	name	   = board_caption(g, m1, m2);
    static int	fignr	   = 0;
    static int	dianr	   = 0;
    int		n1	   = 999;		/* The 1st move number	*/
    int		n2	   = 0;			/* The lst move number	*/
    int		bm	   = 0;
    int		wm	   = 0;

    tex_header(g);

/*	#[position:		*/
    /*
     * Print the position
     */
    if (g->gm_parent)
	eps_init(g, ++dianr, name);
    else
	eps_init(g, ++fignr, name);
    for (p = 0; p < g->gm_sz*g->gm_sz; p++) {
	GOMOVE *m = g->gm_board + p;

	if (m->mv_letter)
		fprintf(fp, "(%c) %d %d fs\n",		/* field sign	*/
			m->mv_letter->ch_letter,
			EPSF_I(g,p), EPSF_J(g,p));
	else
	if (m->mv_mark) {
	    if ((int)(*(m->mv_mark->ch_mark)) < LETTER_MIN ||
		(int) strlen(m->mv_mark->ch_mark) == 1) {
		char *mark  = NULL;
		int   shade = 0;

		switch ((int)(*(m->mv_mark->ch_mark))) {
		case SQUARE_EMPTY   : mark = "square_e"	; break;
		case SQUARE_FILLED  : mark = "square_f"	; break;
		case CIRCLE_EMPTY   : mark = "circle_e"	; break;
		case CIRCLE_FILLED  : mark = "circle_f"	; break;
		case TRIANGLE_EMPTY : mark = "triang_e"	; break;
		case TRIANGLE_FILLED: mark = "triang_f"	; break;
		case SHADE_00	    : mark = "shade"; shade =   0; break;
		case SHADE_10	    : mark = "shade"; shade =  10; break;
		case SHADE_20	    : mark = "shade"; shade =  20; break;
		case SHADE_30	    : mark = "shade"; shade =  30; break;
		case SHADE_40	    : mark = "shade"; shade =  40; break;
		case SHADE_50	    : mark = "shade"; shade =  50; break;
		case SHADE_60	    : mark = "shade"; shade =  60; break;
		case SHADE_70	    : mark = "shade"; shade =  70; break;
		case SHADE_80	    : mark = "shade"; shade =  80; break;
		case SHADE_90	    : mark = "shade"; shade =  90; break;
		case SHADE_100	    : mark = "shade"; shade = 100; break;
		case TERR_B	    : mark = "bterr"	; break;
		case TERR_W	    : mark = "wterr"	; break;
		case '+'	    : mark = "ps"	; break;
		case '-'	    : mark = "ms"	; break;
		default		    :			  break;
		}
		if (mark) {
		    if (!strcmp(mark, "bterr") || !strcmp(mark, "wterr"))
			fprintf(fp, "%d %d %s\n",
				EPSF_I(g,p), EPSF_J(g,p), mark);
		    else
		    if (!strcmp(mark, "shade"))
			fprintf(fp, "%d %d %d shade\n",
				shade, EPSF_I(g,p), EPSF_J(g,p));
		    else
			fprintf(fp, "%d %d %d %s\n",
				(m->mv_side == BLACK) ? 1 : 0,
				EPSF_I(g,p), EPSF_J(g,p), mark);
		} else
		    fprintf(fp, "%d %d %d (%s) metamark\n",
			    (m->mv_side == BLACK) ? 1 : 0,
			    EPSF_I(g,p), EPSF_J(g,p), m->mv_mark->ch_mark);
	    }
	    else {
		fprintf(fp, "%d %d %d (%s) metamark\n",
			(m->mv_side == BLACK) ? 1 : 0,
			EPSF_I(g,p), EPSF_J(g,p), m->mv_mark->ch_mark);
	    }
	}
	else
	if (m->mv_side) {
	    if (m->mv_nr) {
		int bb[4];

		if (m->mv_nr < n1) n1 = m->mv_nr;
		if (m->mv_nr > n2) n2 = m->mv_nr;

		/*
		 * Get metrics
		 */
		eps_metric(fp, ((m->mv_nr-1)%100)+1, m->mv_side, bb);
		if (!bm && m->mv_side == BLACK) {
		    fputc('\n', fp);
		    fprintf(fp, "/bB %d def\n", bb[1]);
		    fprintf(fp, "/bH %d def\n", bb[3]);
		    bm = 1;
		}
		if (!wm && m->mv_side == WHITE) {
		    fputc('\n', fp);
		    fprintf(fp, "/wB %d def\n", bb[1]);
		    fprintf(fp, "/wH %d def\n", bb[3]);
		    wm = 1;
		}
		fprintf(fp, "%d %d %d %d %cst\n",	/* black/white stone */
			bb[0], bb[2],
			EPSF_I(g,p), EPSF_J(g,p),
			(m->mv_side == BLACK) ? 'b' : 'w');
	    } else {
		fprintf(fp, "%d %d %cs\n",		/* black/white stone */
			EPSF_I(g,p), EPSF_J(g,p),
			(m->mv_side == BLACK) ? 'b' : 'w');
	    }
	}
    }
    eps_exit(dianr, name);

/*	#]position:		*/ 
/*	#[connections/caption:	*/

    /*
     * Flush the ko/connect messages and the epsffile call
     */
    fprintf(fw, "\\connect{%s}\n", con_cat((char *) 0));
    fprintf(fw, "\\epsfflush{%s}{", epsffile);
	    text2tex(name);
    fprintf(fw, "}\n");

/*	#]connections/caption:	*/ 
/*	#[text:			*/

    /*
     * Print the text associated with the position
     */
    tex_text(t1, t2, n1, n2);

/*	#]text:			*/ 

    return 1;
}

/* #]PSTeX_emit:	*/ 

