/* #[info:		*/
/************************************************************************
*									*
*	      |\							*
*	       \\							*
*	      / \\	  _-_,	_-_ ,._-_  _-_	 _-_  \\/\\		*
*	     || ||	 ||_.  ||    ||	  || \\ || \\ || ||		*
*	     || ||	  ~ || ||    ||	  ||/	||/   || ||		*
*	      \\/	 ,-_-  \\,/  \\,  \\,/	\\,/  \\ \\		*
*		   _____						*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : d_screen.c							*
* Purpose : Dump a Go data structure onto the screen (using curses)	*
* Version : 1.26							*
* Modified: 22 Mar 1995 09:42:51					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdlib.h>
#include <string.h>
#if defined (ATARI)
#include <vt52.h>
#endif
#include <curses.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "tools.h"
#include "d_screen.h"
#include "gofont.h"

/* #]include:		*/ 
/* #[define:		*/

/*
 * Game --> widget macros
 */
#define G2HW(g)		(((INSTANCE *) g->gm_instance)->in_title)
#define G2IW(g)		(((INSTANCE *) g->gm_instance)->in_info)
#define G2BW(g)		(((INSTANCE *) g->gm_instance)->in_board)
#define G2TW(g)		(((INSTANCE *) g->gm_instance)->in_text)
#define wplace(g,p)	wmove(G2BW(g)->win_id, P2J(g,p), 2*P2I(g,p))
#define wput(g,p,s)	wplace(g,p);	\
			waddstr(G2BW(g)->win_id, s);

/* #]define:		*/ 
/* #[typedef:		*/

typedef struct win_info {
    void *	win_id;		/* Object representation on the screen	*/
    int		win_x;		/* X pos  of the object (approp. unit)	*/
    int		win_y;		/* Y pos  of the object (approp. unit)	*/
    int		win_w;		/* Width  of the object (approp. unit)	*/
    int		win_h;		/* Height of the object	(approp. unit)	*/
    /*
     * And more if necessary...
     */
} WIN;

typedef struct inst_info {
    WIN *	in_title;	/* Screen representation of the title	*/
    WIN *	in_info;	/* Screen representation of the info	*/
    WIN *	in_board;	/* Screen representation of the board	*/
    WIN *	in_text;	/* Screen representation of the text	*/
} INSTANCE;

/* #]typedef:		*/ 
/* #[extern:		*/

extern	int	kibitz;		/* Are we processing kibitz ?		*/
extern	int	labels;		/* Labels with move numbers		*/

/* #]extern:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

static int	scr_Init	PROTO((void));
static void	scr_exit	PROTO((void));
static int	scr_alloc	PROTO((GOGAME *g));
static void	scr_free	PROTO((GOGAME *g));
static void	scr_update	PROTO((GOGAME *g));
static int	scr_getch	PROTO((void));
static void	scr_empty	PROTO((GOGAME *g, PLACE p));
static void	scr_move	PROTO((GOGAME *g, PLACE p, int c, int n));
static void	scr_stone	PROTO((GOGAME *g, PLACE p, int    c));
static void	scr_mark	PROTO((GOGAME *g, PLACE p, int c, MARK m));
static void	scr_letter	PROTO((GOGAME *g, PLACE p, LETTER l));
static void	scr_text	PROTO((GOGAME *g, char *s));
static void	scr_header	PROTO((GOGAME *g, char *s));
static void	scr_clear	PROTO((GOGAME *g));
static void	scr_footer	PROTO((GOGAME *g, char *s));
static int	scr_decode	PROTO((GOGAME *g, PLACE p));
static int	scr_getcmd	PROTO((GOGAME *g, char *explain));
static void	scr_bold	PROTO((GOGAME *g, char *s));
static void	scr_scroll	PROTO((WIN *h));
static int	scr_bop		PROTO((WIN *h));

#undef PROTO

/* #]prototype:		*/ 
/* #[static:		*/

static int	go = 0;

/* #]static:		*/ 

/* #[screen_init:	*/

int
screen_init()
{
    return scr_Init();
}

/* #]screen_init:	*/ 
/* #[screen_exit:	*/

void
screen_exit()
{
    scr_exit();
}

/* #]screen_exit:	*/ 
/* #[screen_alloc:	*/

int
screen_alloc(g)
GOGAME *g;
{
    return scr_alloc(g);
}

/* #]screen_alloc:	*/ 
/* #[screen_free:	*/

void
screen_free(g)
GOGAME *g;
{
    scr_free(g);
}

/* #]screen_free:	*/ 
/* #[screen_onmove:	*/

void
screen_onmove(g, n1, c1, n2, c2)
GOGAME *g;
int	n1;
int	c1;
int	n2;
int	c2;
{
    con_printf("%d on %d", n1, n2);
}

/* #]screen_onmove:	*/ 
/* #[screen_onstone:	*/

void
screen_onstone(g, n, c, p)
GOGAME *g;
int	n;
int	c;
PLACE	p;
{
    con_printf("%d at (%c,%c)", n, P2a(g,p), P2b(g,p));
}

/* #]screen_onstone:	*/ 
/* #[screen_pass:	*/

void
screen_pass(g, n, c)
GOGAME *g;
int	n;
int	c;
{
    con_printf("%d pass", n);
}

/* #]screen_pass:	*/ 
/* #[screen_time:	*/

void
screen_time(g, btime, wtime)
/*
 * For the moment we use the text widget to display the time on the clock
 */
GOGAME *g;
char *btime;
char *wtime;
{
#if 0
    if (btime) scr_text(g, btime);
    if (wtime) scr_text(g, wtime);
#endif
}

/* #]screen_time:	*/ 
/* #[screen_emit:	*/

int
screen_emit(g, m1, m2, t1, t2)
GOGAME *g;
GOMOVE *m1, *m2;	/* Diagram from m1 to m2	*/
GOMOVE *t1, *t2;	/* Text    from t1 to t2	*/
{
    GOMOVE *	m;
    PLACE	p;
    char	username[MAXUSER];	/* Last kibitzer	*/
    int		n1 = MAXNUM;		/* The 1st move number	*/
    int		n2 = 0;			/* The lst move number	*/

/*	#[caption:		*/
    /*
     * Show the title of this position
     */
    scr_header(g, board_caption(g, m1, m2));

/*	#]caption:		*/ 
/*	#[position:		*/
    /*
     * Show the position
     */
    for (p = 0; p < g->gm_sz*g->gm_sz; p++) {
	GOMOVE *b = g->gm_board + p;

	if (b->mv_letter) scr_letter(g, p, b->mv_letter->ch_letter);
	else if (b->mv_mark)
	    scr_mark(g, p, b->mv_side, b->mv_mark->ch_mark);
	else if (b->mv_side  ) {
	    if (b->mv_nr) {
		scr_move  (g, p, b->mv_side, b->mv_nr);
		if (b->mv_nr < n1) n1 = b->mv_nr;
		if (b->mv_nr > n2) n2 = b->mv_nr;
	    }
	    else {
		scr_stone (g, p, b->mv_side);
	    }
	} else scr_empty (g, p);
    }

/*	#]position:		*/ 
/*	#[connections:		*/
    /*
     * Show the connections/ko moves in this position
     */
    scr_footer(g, con_cat((char *) 0));

/*	#]connections:		*/ 
/*	#[update:		*/

    scr_update(g);

/*	#]update:		*/ 
/*	#[text:			*/
    /*
     * Next, show the text associated with the position
     */
    if (t1) for (m = t1; m != t2; m = m->mv_next) {
	TEXT   *t;
	char	msg[1024];

	if (!m->mv_text) continue;

	if (m->mv_nr) {
	    username[0] = 0;
	    if (labels && n1 != n2) {
		sprintf(msg, "\n%s %3d:\n",
			(m->mv_side == BLACK) ? "Black" : "White",
			m->mv_nr);
		scr_bold(g, msg);
	    }
	}
	for (t = m->mv_text; t; t = t->cc_next) {
	    if (!kibitz) { scr_text(g, t->cc_text); continue; }

	    if (strcmp(username, t->cc_name)) {
		strcpy(username, t->cc_name);
		sprintf(msg, "%s[%s]: %s",
			t->cc_name,
			rank2str(t->cc_spec),
			t->cc_text);
	    } else {
		/*
		 * 4 = strlen("[]: ")
		 */
		int n = (int) (
			    strlen(username) +
			    strlen(rank2str(t->cc_spec)) + 4);

		msg[0] = 0;
		while (n--) strcat(msg, " ");
		strcat(msg, t->cc_text);
	    }
	    scr_text(g, msg);
	}
    }

/*	#]text:			*/ 

    /*
     * Get a command and return
     */
    return scr_getcmd(g, "[(q)uit, (g)o, or <any>]");
}

/* #]screen_emit:	*/ 

/* #[scr_Init:		*/

static int
scr_Init()
{
    font_init	();
    initscr	();
    noecho	();
    nl		();	/* '\n' will set cursor at BOL	*/
    raw		();	/* '\n' will set cursor at BOL	*/
#if defined (ATARI)
    CURSOR_OFF;
#endif
    return 1;
}

/* #]scr_Init:		*/ 
/* #[scr_exit:		*/

static void
scr_exit()
{
#if defined (ATARI)
    CURSOR_ON;
#endif
    wclear	(stdscr);
    wrefresh	(stdscr);
    endwin	();
    font_exit	();
}

/* #]scr_exit:		*/ 
/* #[scr_alloc:		*/

#define OFF_X	1
#define OFF_Y	0

static int
scr_alloc(g)
GOGAME *g;
{
    INSTANCE *	I;
    WIN      *	wh;	/* Title window	*/
    WIN      *	wb;	/* Board window	*/
    WIN      *	wi;	/* Info  window	*/
    WIN      *	wt;	/* Text  window	*/

    if ((I = (INSTANCE *) calloc(1, sizeof(INSTANCE))) == (INSTANCE *) 0) {
	message("Out of memory in scr_alloc()\n");
	return 0;
    }
    wh = I->in_title = (WIN *) calloc(1, sizeof(WIN));
    wb = I->in_board = (WIN *) calloc(1, sizeof(WIN));
    wi = I->in_info  = (WIN *) calloc(1, sizeof(WIN));
    wt = I->in_text  = (WIN *) calloc(1, sizeof(WIN));
    if (!wh || !wb || !wi || !wt) {
	message("Out of memory in scr_alloc()\n");
	return 0;
    }

/* #[explanation:	*/
/*
 *	-------------------------------	-------------------------------
 *	| ~~~~~~~~~   title  ~~~~~~~~ |	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	-------------------------------	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . + . . . . + . . . . + . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . + . . .  Goban  . + . . .|	| ~~~~~~~~~   text   ~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . + . . . . + . . . . + . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|. . . . . . . . . . . . . . .|	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	-------------------------------	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~| | ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|~~~~~~~~~~~~ info ~~~~~~~~~~~| | ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~| | ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	------------------------------- -------------------------------
 *
 *                                   [Game]             
 *
 *	-------------------------------	-------------------------------
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	| ~~~~~~~~~   title  ~~~~~~~~ |
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	-------------------------------
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . + . . . . + . . . . + . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~   text   ~~~~~~~~ |	|. . + . . .  Goban  . + . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . + . . . . + . . . . + . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |	|. . . . . . . . . . . . . . .|
 *	| ~~~~~~~~~~~~~~~~~~~~~~~~~~~~|	-------------------------------
 *	|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~| | ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~| | ~~~~~~~~~~ info ~~~~~~~~~~~ |
 *	|~~~~~~~~~~~~~~~~~~~~~~~~~~~~~| | ~~~~~~~~~~~~~~~~~~~~~~~~~~~ |
 *	------------------------------- -------------------------------
 *
 *                                [Variations]             
 */
/* #]explanation:	*/ 
    /*
     * y-position fixed
     */
    wh->win_y = OFF_Y;			/* Fixed Y offset		*/
    wh->win_h = 1;			/* Fixed height			*/
    wb->win_h = g->gm_sz;		/* height decided by board size	*/

    wb->win_y = wh->win_y + wh->win_h;	/* One line below the title	*/
    wi->win_y = wb->win_y + wb->win_h;	/* One line below the title	*/
    wt->win_y = wh->win_y;		/* Same height as the title	*/

    /*
     * width decided by board size
     */
    wh->win_w = 2 * g->gm_sz;
    wb->win_w = wh->win_w;
    wi->win_w = wh->win_w;
    wt->win_w = COLS - wh->win_w - 2*OFF_X - 2;

    /*
     * height of info and text window depends on screen size
     */
    wt->win_h = LINES - wt->win_y - 1;	/* take maximum	*/
    wi->win_h = LINES - wi->win_y - 1;	/* take maximum	*/

    /*
     * x-position depends on game/variation
     */
    if (g->gm_parent) {
	/*
	 * Variation(s)
	 */
	wh->win_x = COLS - wh->win_w - OFF_X;	/* right side	*/
	wb->win_x = wh->win_x;			/* right side	*/
	wi->win_x = wh->win_x;			/* right side	*/
	wt->win_x = OFF_X;			/* left  side	*/
    } else {
	/*
	 * Game
	 */
	wh->win_x = OFF_X;			/* left  side	*/
	wb->win_x = wh->win_x;			/* left  side	*/
	wi->win_x = wh->win_x;			/* left  side	*/
	wt->win_x = COLS - wt->win_w - OFF_X;	/* right side	*/
    }

/* #[creation:		*/
    /*
     * Create the windows
     */
    wclear(stdscr);
    wh->win_id = subwin(stdscr, wh->win_h, wh->win_w, wh->win_y, wh->win_x);
    wb->win_id = subwin(stdscr, wb->win_h, wb->win_w, wb->win_y, wb->win_x);
    wi->win_id = subwin(stdscr, wi->win_h, wi->win_w, wi->win_y, wi->win_x);
    wt->win_id = subwin(stdscr, wt->win_h, wt->win_w, wt->win_y, wt->win_x);

    if (wh->win_id == NULL) { return 0; }
    if (wb->win_id == NULL) { return 0; }
    if (wi->win_id == NULL) { return 0; }
    if (wt->win_id == NULL) { return 0; }

scrollok(((WINDOW *) (wh->win_id)), FALSE);/* wclear((WINDOW *) (wh->win_id));*/
scrollok(((WINDOW *) (wb->win_id)), FALSE);/* wclear((WINDOW *) (wb->win_id));*/
scrollok(((WINDOW *) (wi->win_id)), TRUE );/* wclear((WINDOW *) (wi->win_id));*/
scrollok(((WINDOW *) (wt->win_id)), TRUE );/* wclear((WINDOW *) (wt->win_id));*/

/* #]creation:		*/ 

    g->gm_instance = I;

    return 1;
}

/* #]scr_alloc:		*/ 
/* #[scr_free:		*/

static void
scr_free(g)
GOGAME *g;
{
    WINDOW *w;

    /*
     * Free architecture specific resources
     */
    w = G2BW(g)->win_id; wclear(w); delwin(w);
    w = G2TW(g)->win_id; wclear(w); delwin(w);
    w = G2HW(g)->win_id; wclear(w); delwin(w);

    /*
     * Free private resources
     */
    free(g->gm_instance);
    g->gm_instance = (INSTANCE *) 0;
}

/* #]scr_free:		*/ 
/* #[scr_update:	*/

static void
scr_update(g)
/*
 * Update the board
 */
GOGAME *g;
{
#if 0
    wrefresh(stdscr);
#endif
#if 0
    wrefresh((WINDOW *) (G2HW(g)->win_id));
    wrefresh((WINDOW *) (G2BW(g)->win_id));
    wrefresh((WINDOW *) (G2IW(g)->win_id));
#else
    wrefresh((WINDOW *) (G2BW(g)->win_id));
#endif
}

/* #]scr_update:	*/ 
/* #[scr_getch:		*/

static int
scr_getch()
{
    return ((int) wgetch(stdscr));
}

/* #]scr_getch:		*/ 
/* #[scr_empty:		*/

static void
scr_empty(g, p)
GOGAME *g;
PLACE	p;
{
    wput(g, p, font_decode(scr_decode(g, p)));
}

/* #]scr_empty:		*/ 
/* #[scr_move:		*/

static void
scr_move(g, p, c, n)
/*
 * Emit a numbered stone
 */
GOGAME *g;
PLACE	p;
int	c;
int	n;
{
    char s[3];

    sprintf(s, "%-2d", n % 100);
    if (c == BLACK) wstandout((WINDOW *) (G2BW(g)->win_id));
    wput(g, p, s);
    if (c == BLACK) wstandend((WINDOW *) (G2BW(g)->win_id));
}

/* #]scr_move:		*/ 
/* #[scr_stone:		*/

static void
scr_stone(g, p, c)
GOGAME *g;
PLACE	p;
int	c;
{
    wput(g, p, font_decode((c == BLACK) ? def_black : def_white));
}

/* #]scr_stone:		*/ 
/* #[scr_letter:	*/

static void
scr_letter(g, p, l)
GOGAME *g;
PLACE	p;
LETTER	l;
{
    char s[3];

    sprintf(s, "%c ", l);
    wput(g, p, s);
}

/* #]scr_letter:	*/ 
/* #[scr_mark:		*/

static void
scr_mark(g, p, c, m)
GOGAME *g;
PLACE	p;
int	c;
MARK	m;
{
    WINDOW *w = G2BW(g)->win_id;

    if (m && LETTER_MIN <= *m && *m <= LETTER_MAX) {
	wput(g, p, (char *) m);
	return;
    }

    wstandout(w);
    switch(c) {
	case FREE : wput(g, p, (char *) m); break;
	case BLACK:
	case WHITE: wput(g, p, font_decode((c == BLACK) ? def_black : def_white));
    }
    wstandend(w);
}

/* #]scr_mark:		*/ 
/* #[scr_text:		*/

static void
scr_text(g, s)
GOGAME *g;
char   *s;
{
    scr_scroll	(G2TW(g));
    if (scr_bop	(G2TW(g)) && *s == '\n') s++;
    waddstr	((WINDOW *) (G2TW(g)->win_id), s);
}

/* #]scr_text:		*/ 
/* #[scr_header:	*/

static void
scr_header(g, s)
GOGAME *g;
char   *s;
{
    WINDOW *w = G2HW(g)->win_id;
    char title[256];
    int  indent;

    strncpy(title, s, G2HW(g)->win_w);
    title[G2HW(g)->win_w-1] = 0;
    indent = (G2HW(g)->win_w - (int) strlen(title))/2;

    wmove    (w, 0, 0);
    wclrtobot(w);
    wmove    (w, 0, indent);
    waddstr  (w, title);
    wrefresh (w);
}

/* #]scr_header:	*/ 
/* #[scr_footer:	*/

static void
scr_footer(g, s)
GOGAME *g;
char   *s;
{
    WINDOW *w = G2IW(g)->win_id;

    wclrtobot(w);
    waddstr  (w, s);
    wrefresh (w);
}

/* #]scr_footer:	*/ 
/* #[scr_clear:		*/

static void
scr_clear(g)
GOGAME *g;
{
    PLACE p;

    for (p = 0; p < g->gm_sz * g->gm_sz; p++) scr_empty(g, p);
}

/* #]scr_clear:		*/ 
/* #[scr_decode:	*/

static int
scr_decode(g, p)
GOGAME *g;
PLACE	p;
{
    int i = P2I(g,p);
    int j = P2J(g,p);
    int s = g->gm_sz;

    /*
     * Edge or corner?!
     */
    if (j ==   0) {	if (i ==   0)	return def_ul;
			if (i == s-1)	return def_ur;
					return def_upper; }
    if (j == s-1) {	if (i ==   0)	return def_ll;
			if (i == s-1)	return def_lr;
					return def_lower; }
    if (i ==   0)			return def_left;
    if (i == s-1)			return def_right;

    /*
     * Hoshi or regular?!
     */
    return

	(((s == 19 &&  i == 9) ||
	  (s >= 10 && (i == 3  ||  i == s-4)) ||
	  (s >   6 &&  s < 10  && (i == 2 || i == s-3))) &&
	 ((s == 19 &&  j == 9) ||
	  (s >= 10 && (j == 3  ||  j == s-4)) ||
	  (s >   6 &&  s < 10  && (j == 2 || j == s-3))))

		? def_hoshi
		: def_empty;
}

/* #]scr_decode:	*/ 
/* #[scr_getcmd:	*/

static int
scr_getcmd(g, explain)
GOGAME *g;
char *	explain;
{
    WINDOW *	w = G2TW(g)->win_id;

    if (go) { wrefresh(w); return 1; }

    wclrtobot	(w);
    wstandout	(w);
    scr_text	(g, explain);
    wstandend	(w);
    wrefresh	(w);
    switch (go = scr_getch()) {
	case 'q': go = 0; return 0;
	case 'g': go = 1; break;
	default : go = 0;
    }
    wclear	(w);

    return 1;
}

/* #]scr_getcmd:	*/ 
/* #[scr_scroll:	*/

static void
scr_scroll(h)
WIN *h;
{
    WINDOW *w = h->win_id;

    if (go) { wrefresh (w); return; }

    /*
     * Check whether the output window is full
     */
    if (w->_cury +1 >= h->win_h - 1 && h->win_h > 1) {
	wstandout(w);
	waddstr  (w, "[more]");
	wstandend(w);
	wrefresh (w);
	switch ((int) wgetch(w)) {
	    case 'q':
	    case 'g':
	    default : break;
	}
	wclear	(w);
	wmove	(w, 0, 0);
    }
}

/* #]scr_scroll:	*/ 
/* #[scr_bold:		*/

static void
scr_bold(g, s)
GOGAME *g;
char   *s;
{
    WINDOW *w = G2TW(g)->win_id;

    wstandout	(w);
    scr_text	(g, s);
    wstandend	(w);
}

/* #]scr_bold:		*/ 
/* #[scr_bop:		*/

static int
scr_bop(h)
WIN *h;
{
    return (((WINDOW *)(h->win_id))->_cury == 0);
}

/* #]scr_bop:		*/ 

