/* #[info:		*/
/************************************************************************
*									*
*			  |\		     /\				*
*			   \\		    ||				*
*			  / \\	      _-_, =||=				*
*			 || ||	     ||_.   ||				*
*			 || ||	      ~ ||  ||				*
*			  \\/	     ,-_-   \\,				*
*			       _____					*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : d_sf.c							*
* Purpose : Write an SF file from the internal Go game structure	*
* Version : 1.5								*
* Modified: 22 Mar 1995 09:42:51					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <string.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "d_sf.h"

/* #]include:		*/ 
/* #[define:		*/

#define EMIT_SF_POINT(g,p)	fprintf(fw, "%c%d",		\
					(P2a(g,p) >= 'i')	\
					? (P2a(g,p)+1)		\
					: P2a(g,p),		\
					g->gm_sz-P2J(g,p))

#define EMIT_SF_NL		fputc('\n', fw)
#define EMIT_SF_SPACE		fputc(' ' , fw)

#define SF_N(t,f)		fprintf(fw, "%s %d\n", t, (g->f) ? (g->f) : 0)
#define SF_S(t,f) \
	if (g->f) { \
	    char tmp[80]; \
	    if (strlen(g->f) >= 66) { \
		strncpy(tmp, g->f, 66); tmp[66] = 0;	\
		strcat (tmp, "...\n");			\
	    } else {					\
		strcpy(tmp, g->f);			\
	    }						\
	    fprintf(fw, "%s %s\n", t, tmp);		\
	}

/* #]define:		*/ 
/* #[extern:		*/

extern	FILE *	fw;		/* Output stream			*/

/* #]extern:		*/ 

/* #[game2sf:		*/

void
game2sf(g)
/*
 * Write a Go game in SF format
 */
GOGAME *g;
{
    GOMOVE *m;
    CHAIN  *c;
    char   *indent = (g->gm_parent) ? "\t" : "";

    for (m = g->gm_move; m; m = m->mv_next) {

	switch (m->mv_place) {
/*
		 #[pass:
*/
	    case MV_PASS:
		fprintf(fw, "%s%c %d pass\n", indent,
			(m->mv_side == BLACK) ? 'B' : 'W', m->mv_nr);
		break;
/*
		 #]pass: 
		 #[tenuki:
*/
	    case MV_TENUKI:
		fprintf(fw, "%s%c %d tenuki\n", indent,
			(m->mv_side == BLACK) ? 'B' : 'W', m->mv_nr);
		break;
/*
		 #]tenuki: 
		 #[Header:
*/
	    case MV_HEADER:
	    /*
	     * Yack! The following test is necessary since
	     * there can be empty GOMOVE records in the data.
	     * They came there for example like this:
	     *		(
	     *		...
	     *		;	<<< Empty record
	     *		)
	     */
	    if (m->mv_prev) break;	/* We already wrote it */

	    if (g->gm_parent == NULL) {
		/*
		 * Write the EVENT header
		 */
		SF_S("EVENT"	, gm_gn);
		fprintf(fw, "BLACK %s (%s)\n",
			(g->gm_pb) ? g->gm_pb : "",
			(g->gm_br) ? g->gm_br : "");
		fprintf(fw, "WHITE %s (%s)\n",
			(g->gm_pw) ? g->gm_pw : "",
			(g->gm_wr) ? g->gm_wr : "");
		SF_S("HANDICAP"	, gm_ha);
		SF_S("KOMI"	, gm_km);
		SF_S("DATE"	, gm_dt);
		SF_S("PLACE"	, gm_pc);
		SF_S("RESULT"	, gm_re);
		SF_S("TIMELIMIT", gm_tm);
		SF_S("ANALYSIS"	, gm_analysis);
		SF_S("SOURCE"	, gm_so);
		SF_S("RECORDER"	, gm_recorder);
		SF_S("RULES"	, gm_rules);
	    } else {
		/*
		 * Write the VAR/DIAGRAM header
		 */
		fprintf(fw, "%sVAR\n", indent);
		fprintf(fw, "%sDIAGRAM %s\n", indent,
			(g->gm_n) ? g->gm_n : "");
	    }
	    break;

/*
		 #]Header: 
		 #[Move:
*/
	    default:
	{
	    PLACE p = m->mv_place;

	    fprintf(fw, "%s%c %d ", indent,
		    (m->mv_side == BLACK) ? 'B' : 'W', m->mv_nr);
	    EMIT_SF_POINT(g,p);
	    EMIT_SF_NL;

	    /*
	     * The captured stones
	     */
	    if (m->mv_captured) {
		fprintf(fw, "%sPRISONER", indent);
		for (c = m->mv_captured; c; c = c->ch_next) {
		    EMIT_SF_SPACE;
		    EMIT_SF_POINT(g,c->ch_oldstone->mv_place);
		}
		EMIT_SF_NL;
	    }
	}
 
/*
		 #]Move: 
*/
	}
/*	#[Addstone:		*/

	if (m->mv_addstone) {
	    int bs = 0;
	    int ws = 0;

	    /*
	     * Black/white stones?!
	     */
	    for (c = m->mv_addstone; c; c = c->ch_next)
		if (c->ch_newstone->mv_side == BLACK) bs++; else ws++;
		
	    if (bs) {
		fprintf(fw, "%sSETUP B", indent);
		for (c = m->mv_addstone; c; c = c->ch_next) {
		    if (c->ch_newstone->mv_side == BLACK) {
			EMIT_SF_SPACE;
			EMIT_SF_POINT(g,c->ch_newstone->mv_place);
		    }
		}
		EMIT_SF_NL;
	    }
	    if (ws) {
		fprintf(fw, "%sSETUP W", indent);
		for (c = m->mv_addstone; c; c = c->ch_next) {
		    if (c->ch_newstone->mv_side == WHITE) {
			EMIT_SF_SPACE;
			EMIT_SF_POINT(g,c->ch_newstone->mv_place);
		    }
		}
		EMIT_SF_NL;
	    }
	}

/*	#]Addstone:		*/ 
/*	#[Delstone:		*/

	if (m->mv_delstone) {
	    fprintf(fw, "%sHIDE", indent);
	    for (c = m->mv_delstone; c; c = c->ch_next) {
		EMIT_SF_SPACE;
		EMIT_SF_POINT(g,c->ch_oldstone->mv_place);
	    }
	    EMIT_SF_NL;
	}

/*	#]Delstone:		*/ 
/*	#[Marks:		*/

	/*
	 * Humm, if we don't have text, what's the point of marks?!
	 */
	if (m->mv_mark) {
	    fprintf(fw, "%sMARK", indent);
	    for (c = m->mv_mark; c; c = c->ch_next) {
		EMIT_SF_SPACE;
		fprintf(fw, "%s@", c->ch_mark);
		EMIT_SF_POINT(g,c->ch_place);
	    }
	    EMIT_SF_NL;
	}

/*	#]Marks:		*/ 
/*	#[Letters:		*/

	/*
	 * Humm, if we don't have text, what's the point of letters?!
	 */
	if (m->mv_letter) {
	    fprintf(fw, "%sMARK", indent);
	    for (c = m->mv_letter; c; c = c->ch_next) {
		EMIT_SF_SPACE;
		fprintf(fw, "%c@", c->ch_letter);
		EMIT_SF_POINT(g,c->ch_place);
	    }
	    EMIT_SF_NL;
	}

/*	#]Letters:		*/ 
/*	#[Text:			*/

	/*
	 * Comment block
	 */
	if (m->mv_text) {
	    TEXT *t;

	    fprintf(fw, "%sCOM\n", indent);
	    for (t = m->mv_text; t; t = t->cc_next) {
		fprintf(fw, "%s%s", indent, t->cc_text);
		if (*(t->cc_text) &&
		    *(t->cc_text + t->cc_spec -1) == ' ') EMIT_SF_NL;
	    }
	    fprintf(fw, "%sENDCOM\n", indent);
	}

/*	#]Text:			*/ 
/*	#[Variations:		*/

	/*
	 * Finally, show all variations referred to in the shown position...
	 */
	if (m->mv_variation) {
	    GOGAME *v;

	    for (v = m->mv_variation; v; v = v->gm_next) game2sf(v);
	}

/*	#]Variations:		*/ 
    }
    if (g->gm_parent != NULL) fprintf(fw, "%sENDVAR\n", indent);
}

/* #]game2sf:		*/ 

