/* #[info:		*/
/************************************************************************
*									*
*		       |\			/\			*
*			\\		  _    ||			*
*		       / \\	   _-_,	 / \\ =||=			*
*		      || ||	  ||_.	|| ||  ||			*
*		      || ||	   ~ || || ||  ||			*
*		       \\/	  ,-_-	\\_-|  \\,			*
*			    _____	 /  \				*
*					'----`				*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : d_sgf.c							*
* Purpose : Write an SGF file from the internal Go game structure	*
* Version : 1.12							*
* Modified: 22 Mar 1995 09:42:52					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <string.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "d_sgf.h"
#include "d_db.h"

/* #]include:		*/ 
/* #[define:		*/

#define EMIT_POINT(g,p)	{ fprintf(fw, "[%c%c]", P2a(g,p), P2b(g,p)); mpl++; }
#define EMIT_PASS	{ fprintf(fw, "[tt]"); mpl++; }
#define EMIT_NL		if (mpl >= 10) { fputc('\n', fw); mpl = 0; }
#define EMIT_ESC_NL	fputc('\\', fw); fputc('\n', fw);

/* #]define:		*/ 
/* #[extern:		*/

extern	FILE *	fw;		/* Output stream			*/
extern	int	longkey;	/* Long or short SGF keys while writing	*/
extern	int	textout;	/* Emit text comments?			*/

/* #]extern:		*/ 

/* #[game2sgf:		*/

void
game2sgf(g)
/*
 * Write a Go game in SGF format
 */
GOGAME *g;
{
    GOMOVE *m;
    CHAIN  *c;
    int     mpl  = 0;	/* Moves/line */
    int     done = 0;

    fprintf(fw, "(\n");

    for (m = g->gm_move; m; m = m->mv_next) {

/*	#[Header:		*/

	if (IS_HEADER(m)) {
	    fprintf(fw, ";\n");
	    /*
	     * Write game info
	     */
	    if (!g->gm_parent) game2db(g, NULL);
	    else {
		if (g->gm_n && !done) {
		    fprintf(fw, "%s[%s]\n", N__KEY, g->gm_n);
		    done = 1;
		}
	    }
	    mpl = 0;
	}

/*	#]Header:		*/ 
/*	#[Move:			*/

	if (!IS_HEADER(m)) {
	    PLACE p = m->mv_place;

	    fputc(';', fw);
	    fprintf(fw, "%s", (m->mv_side == BLACK) ? B__KEY : W__KEY);
	    if (p == MV_PASS) EMIT_PASS else EMIT_POINT(g,p);
	    EMIT_NL;

	    /*
	     * The captured stones
	     */
#if INCOMPATIBLE
	    if (m->mv_captured) {
		fprintf(fw, "%s", CP_KEY);
		for (c = m->mv_captured; c; c = c->ch_next) {
		    EMIT_POINT(g,c->ch_oldstone->mv_place);
		}
		EMIT_NL;
	    }
#endif
	}
 
/*	#]Move:			*/ 
/*	#[Addstone:		*/

	if (m->mv_addstone) {
	    int bs = 0;
	    int ws = 0;

	    /*
	     * Black/white stones?!
	     */
	    for (c = m->mv_addstone; c; c = c->ch_next)
		if (c->ch_newstone->mv_side == BLACK) bs++; else ws++;
		
	    if (bs) {
		fprintf(fw, "%s", AB_KEY);
		for (c = m->mv_addstone; c; c = c->ch_next) {
		    if (c->ch_newstone->mv_side == BLACK)
			EMIT_POINT(g,c->ch_newstone->mv_place);
		}
		EMIT_NL;
	    }
	    if (ws) {
		fprintf(fw, "%s", AW_KEY);
		for (c = m->mv_addstone; c; c = c->ch_next) {
		    if (c->ch_newstone->mv_side == WHITE)
			EMIT_POINT(g,c->ch_newstone->mv_place);
		}
		EMIT_NL;
	    }
	}

/*	#]Addstone:		*/ 
/*	#[Delstone:		*/

	if (m->mv_delstone) {
	    fprintf(fw, "%s", AE_KEY);
	    for (c = m->mv_delstone; c; c = c->ch_next) {
		EMIT_POINT(g,c->ch_oldstone->mv_place);
	    }
	    EMIT_NL;
	}

/*	#]Delstone:		*/ 
/*	#[Marks:		*/

	/*
	 * Humm, if we don't have text, what's the point of marks?!
	 */
	if (m->mv_mark) {
	    fprintf(fw, "%s", M__KEY);
	    for (c = m->mv_mark; c; c = c->ch_next) {
		EMIT_POINT(g,c->ch_place);
	    }
	    EMIT_NL;
	}

/*	#]Marks:		*/ 
/*	#[Letters:		*/

	/*
	 * Humm, if we don't have text, what's the point of letters?!
	 */
	if (m->mv_letter) {
	    fprintf(fw, "%s", L__KEY);
	    for (c = m->mv_letter; c; c = c->ch_next) {
		EMIT_POINT(g,c->ch_place);
	    }
	    EMIT_NL;
	}

/*	#]Letters:		*/ 
/*	#[Time:			*/

	if (m->mv_btime) {
	    fprintf(fw, "%s[", BL_KEY);
	    text2sgf(m->mv_btime);
	    fputc(']', fw);
	    EMIT_NL;
	}
	if (m->mv_wtime) {
	    fprintf(fw, "%s[", WL_KEY);
	    text2sgf(m->mv_wtime);
	    fputc(']', fw);
	    EMIT_NL;
	}

/*	#]Time:			*/ 
/*	#[Text:			*/

	/*
	 * Comment block
	 */
	if (textout && m->mv_text) {
	    TEXT *t;

	    fprintf(fw, "%s[\n", C__KEY);
	    for (t = m->mv_text; t; t = t->cc_next) {
		text2sgf(t->cc_text);
		if (*(t->cc_text) &&
		    *(t->cc_text + strlen(t->cc_text)-1) == ' ') {
		    EMIT_ESC_NL;
		}
	    }
	    fprintf(fw, "]\n");
	}

/*	#]Text:			*/ 
/*	#[Variations:		*/

	/*
	 * Finally, show all variations referred to in the shown position...
	 */
	if (m->mv_variation) {
	    GOGAME *v;

	    for (v = m->mv_variation; v; v = v->gm_next) game2sgf(v);
	}

/*	#]Variations:		*/ 
    }
    fprintf(fw, ")\n");
}

/* #]game2sgf:		*/ 

