/* #[info:		*/
/************************************************************************
*									*
*		       |\	    ,					*
*			\\	   ||	     ,				*
*		       / \\	  =||=	_-_  \\ /`			*
*		      || ||	   ||  || \\  \\			*
*		      || ||	   ||  ||/    /\\			*
*		       \\/	   \\, \\,/  /	\;			*
*			    _____					*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : d_tex.c							*
* Purpose : Write a TeX file from the internal Go game structure	*
* Version : 1.13							*
* Modified: 22 Mar 1995 09:42:53					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "textools.h"
#include "tools.h"
#include "d_tex.h"

/* #]include:		*/ 
/* #[define:		*/

#define P2HANNA_I(g,p)	(P2b(g,p) + (P2J(g,p) > 8 ? 1 : 0))
#define P2HANNA_J(g,p)	(P2I(g,p) + 1)

/* #]define:		*/ 
/* #[extern:		*/

extern	FILE *		fw;		/* Output stream		*/

/* #]extern:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

static char *	n2hanna		PROTO((int n, int c));

#undef PROTO

/* #]prototype:		*/ 
 
/* #[n2hanna:		*/

static char *
n2hanna(n, color)
/*
 * The Hanna fonts have numbered stones up to 252.
 * We want to use them in captions if possibly.
 */
int n;
int color;
{
    static char texnum[128];

    if (n <= 252)
	sprintf(texnum,
		"\\mbox{\\text%s{%d}}",
		(color == BLACK) ? "black" : "white",
		n);
    else
	sprintf(texnum, "%d", n);

    return texnum;
}

/* #]n2hanna:		*/ 

/* #[TeX_init:		*/

int
TeX_init()
{
    return 1;
}

/* #]TeX_init:		*/ 
/* #[TeX_exit:		*/

void
TeX_exit()
{
    tex_trailer();
    if (fw != stdout) fclose(fw);
}

/* #]TeX_exit:		*/ 
/* #[TeX_alloc:		*/

int
TeX_alloc(g)
GOGAME *g;
{
    return 1;
}

/* #]TeX_alloc:		*/ 
/* #[TeX_free:		*/

void
TeX_free(g)
GOGAME *g;
{
}

/* #]TeX_free:		*/ 
/* #[TeX_onmove:	*/

void
TeX_onmove(g, n1, c1, n2, c2)
GOGAME *g;
int	n1;
int	c1;
int	n2;
int	c2;
{
    char onmove[128];

    strcpy(onmove, n2hanna(n1, c1));
    strcat(onmove, " at ");
    strcat(onmove, n2hanna(n2, c2));
    con_printf(onmove);
}

/* #]TeX_onmove:	*/ 
/* #[TeX_onstone:	*/

void
TeX_onstone(g, n, c, p)
GOGAME *g;
int	n;
int	c;
PLACE	p;
{
    con_printf("%s at (%c,%c)", n2hanna(n, c), P2a(g,p), P2b(g,p));
}

/* #]TeX_onstone:	*/ 
/* #[TeX_pass:		*/

void
TeX_pass(g, n, c)
GOGAME *g;
int	n;
int	c;
{
    char pass[128];

    strcpy(pass, n2hanna(n, c));
    strcat(pass, " pass");
    con_printf(pass);
}

/* #]TeX_pass:		*/ 
/* #[TeX_time:		*/

void
TeX_time(g, btime, wtime)
/*
 * For the moment we ignore the time on the clock
 */
GOGAME *g;
char *btime;
char *wtime;
{
}

/* #]TeX_time:		*/ 
/* #[TeX_emit:		*/

int
TeX_emit(g, m1, m2, t1, t2)
GOGAME *g;
GOMOVE *m1, *m2;	/* Diagram from m1 to m2	*/
GOMOVE *t1, *t2;	/* Text    from t1 to t2	*/
{
    PLACE	p;
    char *	name = board_caption(g, m1, m2);
    static int	dianr	   = 0;
    int		n1 = 999;			/* The 1st move number	*/
    int		n2 = 0;				/* The lst move number	*/

    tex_header(g);

/*	#[position:		*/
    /*
     * Print the position
     */
    dianr++;
    fprintf(fw, "\\inifulldiagram\n");
    for (p = 0; p < g->gm_sz*g->gm_sz; p++) {
	GOMOVE *m = g->gm_board + p;

	if (m->mv_letter)
		message("[L[%c,%c) ignored]", P2a(g,p), P2b(g,p));
	else
	if (m->mv_mark)
		fprintf(fw, "\\gomark{%c}{%d}{%s}\n",
			P2HANNA_I(g,p),
			P2HANNA_J(g,p),
			"\\triangle");
	else
	if (m->mv_side) {
	    if (m->mv_nr) {
		if (m->mv_nr < n1) n1 = m->mv_nr;
		if (m->mv_nr > n2) n2 = m->mv_nr;
		fprintf(fw, "\\p:%c-%d=\\%c{%d}\n",
			P2HANNA_I(g,p),
			P2HANNA_J(g,p),
			(m->mv_side == BLACK) ? 'b' : 'w',
			((m->mv_nr - 1) % 100) + 1);
	    } else {
		fprintf(fw, "\\p:%c-%d=\\%c.\n",
			P2HANNA_I(g,p),
			P2HANNA_J(g,p),
			(m->mv_side == BLACK) ? 'b' : 'w');
	    }
	}
    }

/*	#]position:		*/ 
/*	#[connections/caption:	*/

    /*
     * Flush the ko/connect messages and give it a title
     */
    fprintf(fw, "\\connect{%s}\n", con_cat((char *) 0));
    fprintf(fw, "\\texmove{%d}{", dianr);
	    text2tex(name);
    fprintf(fw, "}\n");
    message("[%d (%s)]", dianr, name);

/*	#]connections/caption:	*/ 
/*	#[text:			*/

    /*
     * Print the text associated with the position
     */
    tex_text(t1, t2, n1, n2);

/*	#]text:			*/ 

    return 1;
}

/* #]TeX_emit:		*/ 

