/* #[info:		*/
/************************************************************************
*									*
*									*
*		   _	       _     _					*
*		  / \\	/'\\  / \\  < \, \\/\\/\\  _-_			*
*		 || || || || || ||  /-|| || || || || \\			*
*		 || || || || || || (( || || || || ||/			*
*		 \\_-| \\,/  \\_-|  \/\\ \\ \\ \\ \\,/			*
*		  /  \	      /	 \					*
*		 '----`	     '----`					*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : gogame.h							*
* Purpose : The Go game data structure					*
* Version : 1.18							*
* Modified: 03 May 1995 10:33:31					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 

#ifndef __GOGAMEH
#define __GOGAMEH

/* #[define:		*/

/* #[moves:		*/
/*	#[mv_place:	*/

#define MV_HEADER	-1
#define MV_PASS		-2
#define MV_TENUKI	-3
#define IS_MOVE(m)	(m->mv_place >=  0)
#define IS_HEADER(m)	(m->mv_place == MV_HEADER)
#define IS_PASS(m)	(m->mv_place == MV_PASS)
#define IS_TENUKI(m)	(m->mv_place == MV_TENUKI)

/*	#]mv_place:	*/ 
/*	#[mv_side:	*/

#ifdef	FREE
#undef	FREE
#endif
#define FREE		0
#ifdef	BLACK
#undef	BLACK
#endif
#define BLACK		1
#ifdef	WHITE
#undef	WHITE
#endif
#define WHITE		2
#ifdef	PASS
#undef	PASS
#endif
#define PASS		3
#ifdef	DUMMY
#undef	DUMMY
#endif
#define DUMMY		4

/*	#]mv_side:	*/ 
/*	#[mv_nr:	*/

#define AUTO_NR		-1

/*	#]mv_nr:	*/ 
/* #]moves:		*/ 
/* #[marks:		*/

/*
 * Defines for special marks (see MARK)
 */
#define	  SQUARE_EMPTY	1
#define	  SQUARE_FILLED	2
#define	  CIRCLE_EMPTY	3
#define	  CIRCLE_FILLED	4
#define	TRIANGLE_EMPTY	5
#define	TRIANGLE_FILLED	6
#define	SHADE_00	7
#define	SHADE_10	8
#define	SHADE_20	9
#define	SHADE_30	10
#define	SHADE_40	11
#define	SHADE_50	12
#define	SHADE_60	13
#define	SHADE_70	14
#define	SHADE_80	15
#define	SHADE_90	16
#define	SHADE_100	17
#define	TERR_B		18
#define	TERR_W		19
#define	REGCOLOR	20
#define AUTO_MARK	""

/* #]marks:		*/ 
/* #[letters:		*/

/*
 * Limits for letters (see LETTER)
 */
#define	LETTER_MIN	'!'
#define	LETTER_MAX	'z'
#define AUTO_LETTER	-1

/* #]letters:		*/ 

#define MAXSIZE 	19

#define NODE(g,p)	(((g)->gm_goban)->gb_node + (p))
#define SIDE(g,p)	(NODE(g,p)->gn_stone->mv_side)
#define N2P(g,n)	((int)((n) - NODE(g,0)))
#define N2I(g,n)	(N2P(g,n) % (g)->gm_sz)
#define N2J(g,n)	(N2P(g,n) / (g)->gm_sz)
#define P2I(g,p)	((p) % (g)->gm_sz)
#define P2J(g,p)	((p) / (g)->gm_sz)
#define P2a(g,p)	(P2I(g,p)+'a')
#define P2b(g,p)	(P2J(g,p)+'a')

/* #]define:		*/ 
/* #[typedef:		*/

typedef int	PLACE;			/* Place on the Goban	*/
typedef char	MARK[3];		/* On Goban or a stone	*/
typedef int	LETTER;			/* On Goban or a stone	*/

/* #[text  info:	*/

typedef struct text_info {
    char	     *	cc_name;	/* Name of kibitzer	*/
    long		cc_spec;	/* Rank of kibitzer	*/
    char	     *	cc_text;	/* Text of kibitzer	*/
    struct text_info *	cc_next;	/* Next     kibitz	*/
    struct text_info *	cc_prev;	/* Previous kibitz	*/
} TEXT;

/* #]text  info:	*/ 
/* #[goban info:	*/

typedef struct gn_info {
    struct gn_info *	gn_n;		/* Northern neighour	*/
    struct gn_info *	gn_e;		/* Eastern  neighour	*/
    struct gn_info *	gn_s;		/* Southern neighour	*/
    struct gn_info *	gn_w;		/* Western  neighour	*/
    struct mv_info *	gn_stone;	/* Actually (GOMOVE *)	*/
    int 		gn_loop;	/* No endless recursion */
} GONODE;

typedef struct gb_info {
    GONODE *		gb_node;	/* All grid points	*/
    int 		gb_loop;	/* No endless recursion */
} GOBAN;

/* #]goban info:	*/ 
/* #[chain info:	*/

typedef struct chain_info {
    struct mv_info    * ch_newstone;	/* (GOMOVE *)		*/
    struct mv_info    * ch_oldstone;	/* (GOMOVE *)		*/
    LETTER		ch_letter;	/* (see defines)	*/
    MARK		ch_mark;	/* (see defines)	*/
    PLACE		ch_place;	/* Place on the Goban	*/
    struct chain_info * ch_next;	/* Next stone in chain	*/
} CHAIN;

/* #]chain info:	*/ 
/* #[move  info:	*/

typedef struct mv_info {
    PLACE		mv_place;	/* Place on the Goban	*/
    short		mv_side;	/* Color (BLACK/WHITE)	*/
    short		mv_nr;		/* When it was played	*/
    TEXT	     *	mv_text;	/* Kibitz list on move	*/
    TEXT	     *	mv_textlast;	/* Last kibitz on move	*/
    char	     *	mv_btime;	/* Black time left	*/
    char	     *	mv_wtime;	/* White time left	*/
    CHAIN	     *	mv_captured;	/* Captured stones	*/
    CHAIN	     *	mv_delstone;	/* Deleted  stones	*/
    CHAIN	     *	mv_addstone;	/* Extra    stones	*/
    CHAIN	     *	mv_letter;	/* Letters on the board	*/
    CHAIN	     *	mv_mark;	/* Marks on the board	*/
    struct   mv_info *	mv_prev;	/* Previous move	*/
    struct   mv_info *	mv_next;	/* Next     move	*/
    struct game_info *	mv_variation;	/* A variation		*/
} GOMOVE;

/* #]move  info:	*/ 
/* #[game  info:	*/

typedef struct game_info {
    GOBAN  *		gm_goban;	/* The Go board 	*/
    GOMOVE *		gm_move;	/* All the moves	*/
    GOMOVE *		gm_movelast;	/* Current move 	*/
    GOMOVE *		gm_status;	/* The board   status	*/
    GOMOVE *		gm_board;	/* The visible status	*/
    GOMOVE *		gm_initial;	/* The initial status	*/
    int 		gm_gid;		/* Game    number 	*/
    int 		gm_fid;		/* Figure  number 	*/
    int 		gm_did;		/* Diagram number 	*/
    int 		gm_movenr;	/* Current move number	*/
    int 		gm_sz;		/* Board size		*/
    char   *		gm_ev;		/* Event specification	*/
    char   *		gm_id;		/* Game  id		*/
    char   *		gm_ff;		/* File format		*/
    char   *		gm_us;		/* Message for user	*/
    char   *		gm_gamenr;	/* Game number		*/
    char   *		gm_gn;		/* Game name		*/
    char   *		gm_gc;		/* Game specific notice	*/
    char   *		gm_n;		/* Game name		*/
    char   *		gm_pb;		/* Black player 	*/
    char   *		gm_br;		/* Black rank		*/
    char   *		gm_bl;		/* Black alloted time	*/
    char   *		gm_pw;		/* White player 	*/
    char   *		gm_wr;		/* White rank		*/
    char   *		gm_wl;		/* White alloted time	*/
    char   *		gm_ha;		/* Handicap 		*/
    char   *		gm_km;		/* Komi 		*/
    char   *		gm_tm;		/* Allotted time/player	*/
    char   *		gm_dt;		/* Date 		*/
    char   *		gm_pc;		/* Place		*/
    char   *		gm_re;		/* Result		*/
    char   *		gm_so;		/* Source   of the game	*/
    /*
     * View and region bounding boxes
     */
    PLACE		gm_vw[4];	/* Clipping region	*/
    PLACE		gm_rg[4];	/* Region BoundingBox	*/
    int			gm_ro;		/* Round in the tourn.	*/
    /*
     * The next three lines are there for the Standard format
     */
    char   *		gm_recorder;	/* Recorder of the game	*/
    char   *		gm_analysis;	/* Analist  of the game	*/
    char   *		gm_rules;	/* Rules    of the game	*/
    /*
     * The next three lines allows variations in a game
     */
    struct game_info *	gm_parent;	/* Parent   game/var	*/
    struct game_info *	gm_next;	/* Next     variation	*/
    struct game_info *	gm_prev;	/* Previous variation	*/
    /*
     * Allow an arbitraire instance of a game position
     */
    void   *		gm_instance;	/* Some representation	*/
} GOGAME;

/* #]game  info:	*/ 

/* #]typedef:		*/ 
/* #[prototype: 	*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

/*
 * Memory managers
 */
GOGAME *	game_alloc	PROTO((GOGAME *g));
void		game_free	PROTO((GOGAME *g));
GOBAN * 	goban_new	PROTO((GOGAME *g, int size));
int		board_alloc	PROTO((GOGAME *g));
void		board_free	PROTO((GOGAME *g));
GOMOVE *	move_alloc	PROTO((void));
GOMOVE *	move_new	PROTO((GOGAME *g, GOMOVE *parent));
GOMOVE *	move_free	PROTO((GOMOVE *m));

/*
 * Data managers
 */
void		node_store	PROTO((GOGAME *g));
void		kibitz_store	PROTO((GOGAME *g, char *text));
void		text_store	PROTO((GOGAME *g, char *text));
void		move_store	PROTO((GOGAME *g, int nr, int side, PLACE p));
void		pass_store	PROTO((GOGAME *g, int nr, int side));
void		addstone_store	PROTO((GOGAME *g, int side, PLACE p));
void		delstone_store	PROTO((GOGAME *g, PLACE p));
void		letter_store	PROTO((GOGAME *g, PLACE p, LETTER letter));
void		mark_store	PROTO((GOGAME *g, PLACE place, MARK mark));
void		view_store	PROTO((GOGAME *g, PLACE i, PLACE j));
void		reg_store	PROTO((GOGAME *g, PLACE i, PLACE j));
void		rnd_store	PROTO((GOGAME *g, int rnd));
void		capture_store	PROTO((GOGAME *g, GONODE *gonode));
void		time_store	PROTO((GOGAME *g, int side, char *t));
void		gamename_store	PROTO((GOGAME *g, char *name));
void		name_store	PROTO((GOGAME *g, char *name));
void		user_store	PROTO((GOGAME *g, char *text));
void		size_store	PROTO((GOGAME *g, int size));

GOGAME *	push_var	PROTO((GOGAME *g));
GOGAME *	pop_var		PROTO((GOGAME *g));

/*
 * Moving through the game
 */
GOMOVE *	move_current	PROTO((GOGAME *g));
GOMOVE *	move_mv_back	PROTO((GOGAME *g));
GOMOVE *	move_mv_forw	PROTO((GOGAME *g));
GOMOVE *	move_mv_allback	PROTO((GOGAME *g));
GOMOVE *	move_mv_allforw	PROTO((GOGAME *g));

/*
 * Tools
 */
char *		game_name	PROTO((GOGAME *g));

#undef PROTO

/* #]prototype: 	*/ 

#endif
