/* #[info:		*/
/************************************************************************
*									*
*		  ,,	,,		    ,				*
*		  ||  ' ||		   ||				*
*		  || \\ ||/|,  _-_  ,._-_ =||= '\\/\\			*
*		  || || || || || \\  ||	   ||	|| ;'			*
*		  || || || |' ||/    ||	   ||	||/			*
*		  \\ \\ \\/   \\,/   \\,   \\,	|/			*
*					       (			*
*						-_-			*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : liberty.h							*
* Purpose : Read a Liberty Go game					*
* Version : 1.3								*
* Modified: 7/24/93 17:08:40						*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[define:		*/

#define LIB_FIG		0
#define LIB_DIA		1

#define LIB_FREE	0
#define LIB_BLACK	1
#define LIB_WHITE	2

#define	LIB_PLUS	1
#define LIB_MINUS	2
#define	LIB_TRIANGLE	3

#define	LIB_CONTINUE	0
#define	LIB_DIAGRAM	1
#define	LIB_BOARDSIZE	2
#define LIB_STONE	3
#define LIB_NUMSTONE	4
#define LIB_MARKSTONE	5
#define LIB_CHARACTER	6
#define LIB_RESULT	7
#define LIB_HIT		8
#define LIB_PASS	9
#define LIB_TENUKI	10
#define LIB_HITSTONE	11
#define LIB_HITNUM	12
 
/* #]define:		*/
/* #[datastructure:	*/

/*
 *	Action bitmask:
 *
 *	21098765432109876543210987654321
 *				    ^^^^ status
 *				  ^^---- color
 *				^^------ mark
 *			  ^^^^^^^^------ character
 *			^^^^^^^^^^------ number
 *		   ^^^^^---------------- x coordinate
 *	      ^^^^^--------------------- y coordinate
 *	 ^^^---------------------------- window #
 *
 */

#define STATEMASK	0x000f	/*  4 bits */
#define STATSHIFT	4
#define STATMASK	0x0003	/*  2 bits */
#define MARKSHIFT	6
#define MARKMASK	0x0003	/*  2 bits */
#define CHARSHIFT	6
#define CHARMASK	0x00ff	/*  8 bits */
#define NUMSHIFT	6
#define NUMMASK		0x03ff	/* 10 bits */
#define XSHIFT		16
#define XMASK		0x001f	/*  5 bits */
#define YSHIFT		21
#define YMASK		0x001f	/*  5 bits */
#define NSHIFT		28
#define NMASK		0x0007	/*  3 bits */

#define LIB_STATE(p)	( (int)((*p)		   & STATEMASK))
#define LIB_N(p)	(((int)((*p) >>	  NSHIFT)  &	 NMASK))
#define LIB_X(p)	(((int)((*p) >>	  XSHIFT)  &	 XMASK))
#define LIB_Y(p)	(((int)((*p) >>	  YSHIFT)  &	 YMASK))
#define LIB_STAT(p)	(((int)((*p) >> STATSHIFT) &  STATMASK))
#define LIB_MARK(p)	(((int)((*p) >> MARKSHIFT) &  MARKMASK))
#define LIB_CHAR(p)	(((int)((*p) >> CHARSHIFT) &  CHARMASK))
#define LIB_NUM(p)	(((int)((*p) >>	 NUMSHIFT) &   NUMMASK))

/* #]datastructure:	*/ 
/* #[prototype:		*/

#ifdef __STDC__
#       define  PROTO(s) s
#else
#       define  PROTO(s) ()
#endif

GOGAME *		read_liberty	PROTO((char *filename));

#undef PROTO

/* #]prototype:		*/ 

