/* #[info:		*/
/************************************************************************
*									*
*	                                                ,,		*
*	                                                ||		*
*	      \\/\\/\\  _-_  \\/\\/\\ -_-_   /'\\  /'\\ ||		*
*	      || || || || \\ || || || || \\ || || || || ||		*
*	      || || || ||/   || || || || || || || || || ||		*
*	      \\ \\ \\ \\,/  \\ \\ \\ ||-'  \\,/  \\,/  \\		*
*	                              |/                  		*
*	                              '                   		*
*									*
*		 Copyright (c) 1993  Jan van der Steen			*
*	      Centre for Mathematics and Computer Science		*
*		      Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File    : mempool.h							*
* Purpose : ___________________________________________________________	*
* Version : 1.5								*
* Modified: 19 Oct 1994 14:04:25					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 

#ifndef __MEMPOOLH
#define __MEMPOOLH

/* #[define:		*/

/*
 * Values for mp_flags
 */
#define MP_MSG	0x01
#define MP_CHK	0x02

/*
 * Default pool dimension
 */
#define MP_N	1000		/* Initial    # of blocks */
#define MP_DN	100		/* Additional # of blocks */

/* #]define:		*/ 
/* #[typedef:		*/

typedef struct mem_info {
    struct mem_info *	mem_next;	/* Next free block		*/
} MEM;

typedef struct mp_info {
    long		mp_n;		/* # allocated blocks		*/
    long		mp_dn;		/* block increment		*/
    long		mp_sz;		/* sizeof(block)		*/
    char *		mp_mem;		/* mp_n * (sizeof(MEM)+mp_sz)	*/
    MEM  *		mp_free;	/* 1st free block		*/
    struct mp_info *	mp_next;	/* Next allocation		*/
    struct mp_info *	mp_last;	/* Last allocation		*/
    short		mp_flags;	/* Behaviour flags		*/
} MP;

/* #]typedef:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

MP   *	mp_init		PROTO((long n, long dn, long sz, int flags));
void *	mp_alloc	PROTO((MP *mp));
void *	mp_calloc	PROTO((MP *mp));
void	mp_free		PROTO((MP *mp, void *p));
MP *	mp_stat		PROTO((MP *mp));
void	mp_exit		PROTO((MP *mp));

#undef PROTO

/* #]prototype:		*/ 

#endif
