/* #[info:		*/
/************************************************************************
*									*
*			       ,					*
*			      ||   '					*
*		  /'\\ -_-_  =||= \\  /'\\ \\/\\  _-_,			*
*		 || || || \\  ||  || || || || || ||_.			*
*		 || || || ||  ||  || || || || ||  ~ ||			*
*		 \\,/  ||-'   \\, \\ \\,/  \\ \\ ,-_-			*
*		       |/						*
*		       '						*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : options.c							*
* Purpose : Get and process the command line options			*
* Version : 1.30							*
* Modified: 03 May 1995 13:15:43					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#ifdef __STDC__
#  include <stdlib.h>
#endif

#include <string.h>
#include "sgf2misc.h"
#include "textools.h"
#include "d_gif.h"
#include "tools.h"
#include "patch.h"

/* #]include:		*/ 
/* #[define:		*/

/*
 * Defines for the option lookup table
 * The numbers don't need to be in sequence, just unique.
 * But the compiler will complain if they aren't, since they
 * are the labels of the switch() in getoptions.
 */
#define OPT_DONE		-1	/* We're finished */
#define OPT_ERROR		-2	/* To pass errors */
#define OPT_CACHING		 0
#define OPT_COORDINATES		 1
#define OPT_DIAGRAMS		 2
#define OPT_FIGURES		 3
#define OPT_FLATTEN		 4
#define OPT_FROM		 5
#define OPT_HELP		 6
#define OPT_KIBITZ		 7
#define OPT_LABELS		 8
#define OPT_OUTPUTFILE		 9
#define OPT_SCALE		10
#define OPT_SHORT_KEYS		11
#define OPT_STATISTICS		12
#define OPT_TEXT_OUTPUT		13
#define OPT_TMP			14
#define OPT_TO			15
#define OPT_VERBOSE		16
#define OPT_VERSION		17
#define OPT_ZHUGE_MODE		18
#define OPT_PAPER   	    	19
#define OPT_MAX   	    	20
#define OPT_PPM   	    	21
#define OPT_RGBB   	    	22
#define OPT_RGBW   	    	23
#define OPT_RGBG   	    	24
#define OPT_RGBO   	    	25
#define OPT_RGBR   	    	26
#define OPT_FNTSZ   	    	27
#define OPT_PBMBIN   	    	28
#define OPT_SEQUENT   	    	29
#define OPT_HOLD   	    	30

/* #]define:		*/ 
/* #[typedef:		*/

typedef struct option_info {
    char *	opt_name;	/* Long option specifier */
    int		opt_n;		/* Return value		 */
    int		opt_arg;	/* Needs an argument ?!	 */
} OPTION;

/* #]typedef:		*/ 
/* #[static:		*/

static	char *opt_arg;		/* option argument */

static OPTION	options[] = {
	{ "caching"		, OPT_CACHING		, 0 },
	{ "coordinates"		, OPT_COORDINATES	, 0 },
	{ "diagrams"		, OPT_DIAGRAMS		, 1 },
	{ "figures"		, OPT_FIGURES		, 1 },
	{ "flatten"		, OPT_FLATTEN		, 0 },
	{ "fontsize"		, OPT_FNTSZ		, 1 },
	{ "from"		, OPT_FROM		, 1 },
	{ "help"		, OPT_HELP		, 0 },
	{ "hold"		, OPT_HOLD		, 0 },
	{ "kibitz"		, OPT_KIBITZ		, 0 },
	{ "labels"		, OPT_LABELS		, 0 },
	{ "max"			, OPT_MAX		, 1 },
	{ "output"		, OPT_OUTPUTFILE	, 1 },
	{ "pbmbin"		, OPT_PBMBIN		, 1 },
	{ "ppm"			, OPT_PPM		, 1 },
	{ "rgbb"		, OPT_RGBB		, 1 },
	{ "rgbg"		, OPT_RGBG		, 1 },
	{ "rgbo"		, OPT_RGBO		, 1 },
	{ "rgbr"		, OPT_RGBR		, 1 },
	{ "rgbw"		, OPT_RGBW		, 1 },
	{ "scale"		, OPT_SCALE		, 1 },
	{ "short_keys"		, OPT_SHORT_KEYS	, 0 },
	{ "sequent"		, OPT_SEQUENT		, 0 },
	{ "statistics"		, OPT_STATISTICS	, 0 },
	{ "text_output"		, OPT_TEXT_OUTPUT	, 0 },
	{ "tmp"			, OPT_TMP		, 1 },
	{ "to"			, OPT_TO		, 1 },
	{ "verbose"		, OPT_VERBOSE		, 0 },
	{ "version"		, OPT_VERSION		, 0 },
	{ "zhuge_mode"		, OPT_ZHUGE_MODE	, 0 },
    	{ "paper"   	    	, OPT_PAPER 	    	, 1 },
	{ ""			, OPT_DONE		, 0 }
};

/* #]static:		*/ 
/* #[extern:		*/

extern	FILE *	 fw;		/* Output channel			*/
extern	EPSFINFO epsf;		/* The eps filename/directory		*/
extern	char *	 program;	/* Program name				*/
extern	char *	 ppm;		/* PPM bitmap image tree		*/
extern	char *	 pbmbin;	/* PBMBIN bitmap image tree		*/
extern	char *	 rgb_b;		/* Color of black			*/
extern	char *	 rgb_w;		/* Color of white			*/
extern	char *	 rgb_g;		/* Color of board			*/
extern	char *	 rgb_o;		/* Color of outside			*/
extern	char *	 rgb_r;		/* Color of region			*/
extern	int	 file_fmt;	/* Input format (SGF, Liberty or std)	*/
extern	int	 prn_dev;	/* DEVICE_{PSTEX,TEX,DB,ASCII,SGF,NULL}	*/
extern	int	 paper_size;	/* PAPER_{DEFAULT,LETTER,A4WIDE,A4} 	*/
extern	int	 verbose;	/* Verbose mode				*/
extern	int	 caching;	/* Caching mode				*/
extern	int	 textout;	/* Emit the text too			*/
extern	int	 statout;	/* Emit the kibitz stats		*/
extern	int	 coords;	/* Print coordinates			*/
extern	int	 labels;	/* Label numbered moves			*/
extern	int	 kibitz;	/* Interpret comments as IGS kibitz	*/
extern	int	 zhuge;		/* Apply the "zhuge" conversions	*/
extern	int	 flatten;	/* Apply the "flatten" conversion	*/
extern	int	 sequent;	/* Apply the "sequent" conversion	*/
extern	int	 longkey;	/* Long or short SGF keys while writing	*/
extern	int	 figlist[];	/* Figure  list				*/
extern	int	 dialist[];	/* Diagram list				*/
extern	int	 fntsz;		/* Fontsize in GIF mode			*/
extern  unsigned maxmoves;  	/* Maximum moves per figure 	    	*/
extern	int	 optind;	/* index into argv vector		*/
extern	int	 hold;		/* Hold GIF script			*/

/* #]global:		*/

/* #]extern:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#	define  PROTO(s) s
#else
#	define  PROTO(s) ()
#endif

static	void	usage		PROTO((int exitcode));
static   int	mygetopt	PROTO((int nargc, char *nargv[]));
static	void	getlist		PROTO((char *userlist, int mylist[]));

#undef PROTO

/* #]prototype:		*/ 

/* #[mygetopt:		*/

static int
mygetopt(argc, argv)
int   argc;
char *argv[];
{
    OPTION *o;
    char   *s = argv[optind];

    if (optind >= argc || *s++ != '-' || *s == 0) return OPT_DONE;

    for (o = options; o->opt_name && *(o->opt_name); o++) {
	if (strncmp(o->opt_name, s, strlen(s))) continue;

	if (!strncmp((o+1)->opt_name, s, strlen(s))) {
	    fprintf(stderr,
		    "%s: Option \"%s\" is ambiguous (-%s and -%s)\n",
		    program, s, o->opt_name, (o+1)->opt_name);
	    return OPT_ERROR;
	}

	/*
	 * We found an unique option
	 */
	optind++; opt_arg = argv[optind];
	if (o->opt_arg) {
	    if (opt_arg == (char *) 0 || *(opt_arg) == '-') {
		fprintf(stderr,
			"%s: Option \"%s\" needs an argument\n",
			program, o->opt_name);
		return OPT_ERROR;
	    }
	    optind++;	/* skip argument */
	}
	return o->opt_n;
    }
    fprintf(stderr, "%s: \"%s\": no such option\n", program, s);
    return OPT_ERROR;
}

/* #]mygetopt:		*/ 
/* #[getoptions:	*/

#define MATCH(device,arg)	(!strncmp(device, arg, strlen(arg)))

void
getoptions(argc, argv)
int	argc;
char  **argv;
{
    int	  c;
    int	  i;
    char *s;
#ifndef __STDC__
    extern char *getenv();
#endif

    epsf.eps_dir  = TMPDIR;
    epsf.eps_size = MAXWIDTH_A4;
    if ((s = getenv("EPSFTMP" )) != (char *) 0) epsf.eps_dir  = s;
    if ((s = getenv("EPSFSIZE")) != (char *) 0) epsf.eps_size = atoi(s);

    /*
     * Initialise the figure and diagram arrays
     */
    for (i = 1; i < MAXNUM; i++) figlist[i] = 0;   /* No forced figures	*/
    for (i = 1; i < MAXNUM; i++) dialist[i] = i;   /* All diagrams...	*/

    while ((c = mygetopt(argc, argv)) != OPT_DONE)
    switch (c) {
	case OPT_FROM:
	    if MATCH("sgf"    , opt_arg) { file_fmt = FMT_SGF; break; }
	    if MATCH("liberty", opt_arg) { file_fmt = FMT_LIB; break; }
	    if MATCH("sf"     , opt_arg) { file_fmt = FMT_STD; break; }
	    fprintf(stderr, "%s: \"%s\": unknown input format\n",
		    program, opt_arg);
	    exit(1);

	case OPT_TO:
	    if MATCH("ascii"  , opt_arg) { prn_dev = DEVICE_ASCII ; break; }
	    if MATCH("graphic", opt_arg) { prn_dev = DEVICE_SCREEN; break; }
	    if MATCH("tex"    , opt_arg) { prn_dev = DEVICE_TEX   ; break; }
	    if MATCH("pstex"  , opt_arg) { prn_dev = DEVICE_PSTEX ; break; }
	    if MATCH("sgf"    , opt_arg) { prn_dev = DEVICE_SGF   ; break; }
	    if MATCH("sf"     , opt_arg) { prn_dev = DEVICE_SF    ; break; }
	    if MATCH("db"     , opt_arg) { prn_dev = DEVICE_DB    ; break; }
	    if MATCH("gif"    , opt_arg) { prn_dev = DEVICE_GIF   ; break; }
	    if MATCH("html"   , opt_arg) { prn_dev = DEVICE_HTML  ; break; }
	    if MATCH("null"   , opt_arg) { prn_dev = DEVICE_NULL  ; break; }
	    fprintf(stderr, "%s: \"%s\": unknown output format\n",
		    program, opt_arg);
	    exit(1);

	case OPT_PAPER:
	    if MATCH("default", opt_arg) { paper_size = PAPER_DEFAULT; break; }
	    if MATCH("letter" , opt_arg) { paper_size = PAPER_LETTER ; break; }
	    if MATCH("a4wide" , opt_arg) { paper_size = PAPER_A4WIDE ; break; }
	    if MATCH("a4"     , opt_arg) { paper_size = PAPER_A4     ; break; }
	    fprintf(stderr, "%s: \"%s\": unknown paper size\n",
		    program, opt_arg);
	    exit(1);

	/*
	 * GIF mode handling
	 */
	case OPT_PPM:		ppm	      = opt_arg;		break;
	case OPT_PBMBIN:	pbmbin	      = opt_arg;		break;
	case OPT_RGBB:		rgb_b	      = opt_arg;		break;
	case OPT_RGBW:		rgb_w	      = opt_arg;		break;
	case OPT_RGBG:		rgb_g	      = opt_arg;		break;
	case OPT_RGBO:		rgb_o	      = opt_arg;		break;
	case OPT_RGBR:		rgb_r	      = opt_arg;		break;
	case OPT_FNTSZ:		fntsz	      = atoi(opt_arg);		break;

	case OPT_TMP:		epsf.eps_dir  = opt_arg;		break;
	case OPT_SCALE:		epsf.eps_size = atoi(opt_arg);		break;

	case OPT_FIGURES:	getlist(opt_arg, figlist);		break;
	case OPT_DIAGRAMS:	getlist(opt_arg, dialist);		break;
	case OPT_MAX:	    	maxmoves = (unsigned) atoi(opt_arg);	break;

	case OPT_OUTPUTFILE:	if ((fw = fopen(opt_arg, "w")) == NULL) {
				    fprintf(stderr, "%s Can't write \"%s\"\n",
					    program, opt_arg);
				    exit(1);
				}
				break;

	case OPT_CACHING:	caching = 1;				break;
	case OPT_SHORT_KEYS:	longkey = 0;				break;
	case OPT_COORDINATES:	coords  = 0;				break;
	case OPT_STATISTICS:	statout = 1;				break;
	case OPT_TEXT_OUTPUT:	textout = 0;				break;
	case OPT_LABELS:	labels  = 0;				break;
	case OPT_VERBOSE:	verbose = 1;				break;
	case OPT_KIBITZ:	kibitz  = 1;				break;
	case OPT_ZHUGE_MODE:	zhuge   = 1;				break;
	case OPT_FLATTEN:	flatten = 1;				break;
	case OPT_SEQUENT:	sequent = 1;				break;
	case OPT_HOLD:		hold    = 1;				break;

				/* show program version and exit(0) */
	case OPT_VERSION:	fprintf(stdout, "%s-%d.%d.%d\n", program,
					RELEASE, PATCHLEVEL/10, PATCHLEVEL%10);
				exit(0);

	case OPT_HELP:		usage(0);	/* Online help	*/
	case OPT_ERROR:		fprintf(stderr,
					"%s -help gives online help\n",
					program);
				exit(1);
    }
    if (epsf.eps_size == 0)
    	epsf.eps_size = tex_figuresize(paper_size);
}

/* #]getoptions:	*/ 
/* #[usage:		*/

#define FMTS2(s1,s2)	fprintf(stderr, "%-20s%-44s\n", s1, s2)
#define FMTS3(s1,s2,s3)	fprintf(stderr, "%-20s%-44s%s\n", s1, s2, s3)
#define FMTN3(s1,s2,n1)	fprintf(stderr, "%-20s%-44s%d\n", s1, s2, n1)
#define DOTTEDLINE	{\
			    int i;\
			    for (i = 0; i < 72; i++) fputc('-', stderr);\
			    fputc('\n', stderr);\
			}

static void
usage(code)
int code;
{
    fprintf(stderr, "Usage: %s [options] [file]\n", program);
    DOTTEDLINE
    FMTS3("Option"		, "Description"		   , "Default");
    DOTTEDLINE
    FMTS3("[-from fmt_spec]"	, "Input Go game format"   , "sgf file");
    FMTS3("", "[sgf,liberty,sf]"			   , "");
    FMTS3("[-to dev_spec]"	, "Output format"	   , "pstex");
    FMTS3("", "[pstex,tex,ascii,graphic,sgf,sf,db,gif,html,null]", "");
    FMTS3("[-paper paper_spec]"	, "Paper Size"		   , "default");
    FMTS3("", "[default,letter,a4wide,a4]"  	    	   , "");
    FMTS3("[-tmp path]"		, "EPS directory"	   , epsf.eps_dir);
    FMTN3("[-scale size]"	, "Diagram size [72 = 1in]", epsf.eps_size);
    FMTS3("[-max moves]"	, "Limit moves per figure" , "unlimited");
    FMTS3("[-figures  list]"	, "Emit figures  in list"  , "automatic");
    FMTS3("[-diagrams list]"	, "Emit diagrams in list"  , "all");
    FMTS3("[-output file]"	, "Emit output to file"	   , "stdout");
    FMTS3("[-coordinates]"	, "Don't print coordinates", "on");
    FMTS3("[-text_output]"	, "Don't print text output", "on");
    FMTS3("[-labels]"		, "Don't label moves"	   , "on");
    FMTS3("[-caching]"		, "Caching   mode"	   , "off");
    FMTS3("[-verbose]"		, "Verbose   mode"	   , "off");
    FMTS3("[-sequent]"		, "Variation(1) mode"	   , "off");
    FMTS3("[-flatten]"		, "Variation(2) mode"	   , "off");
    FMTS3("[-kibitz]"		, "Kibitz    mode"	   , "off");
    FMTS3("[-zhuge_mode]"	, "Zhuge     mode"	   , "off");
    FMTS3("[-statistics]"	, "Emit kibitz statistics" , "off");
    FMTS3("[-short_keys]"	, "Don't use long SGF keys", "off");
    FMTS3("[-ppm path]"		, "Path to ppm directory"  , PBMFNT);
    FMTS3("[-pbmbin path]"	, "Path to pbmplus tools"  , PBMBIN);
    FMTS3("[-rgbb color]"	, "GIF Color for Black"    , "rgb\\:00/00/00");
    FMTS3("[-rgbw color]"	, "GIF Color for White"    , "rgb\\:ff/ff/ff");
    FMTS3("[-rgbg color]"	, "GIF Color for Board"    , "rgb\\:ff/e1/4b");
    FMTS3("[-rgbo color]"	, "GIF Color for Outside"  , "rgb\\:ff/e1/4b");
    FMTS3("[-rgbr color]"	, "GIF Color for Region"   , "rgb\\:ff/ff/7f");
    FMTN3("[-fontsize bits]"	, "GIF image unit size"    , PBMSZ);
    FMTS2("[-version]"		, "Print program version");
    FMTS2("[-help]"		, "Print usage");

    exit(code);
}

/* #]usage:		*/ 

/* #[getlist:		*/

static void
getlist(userlist, mylist)
/*
 * Get the list of figures to print
 *
 * List can be any of:
 *
 *	%n		Create figure after each n moves
 *	k+l+...+n	Create figure at move k, l, ..., n
 *
 * We fill the mylist array with the move/diagram numbers where we
 * will print a figure/diagram
 */
char *userlist;
int   mylist[];
{
    int interval;
    int number;
    int n = 1;

    if (*userlist == '%') {
	if (mylist == dialist) {
	    fprintf(stderr, "%s: This format is only useful for figures\n",
		    program);
	    exit(1);
	}
	/*
	 * Interval request
	 */
	interval = atoi(userlist+1);
	if (interval < 1 || interval >= MAXNUM) {
	    fprintf(stderr, "%s: Figure interval out of range (1 - %d)\n",
		    program, MAXNUM);
	    exit(1);
	}
	while ((number = n*interval) < MAXNUM) figlist[n++] = number;
    }
    else {
	/*
	 * User list request
	 */
	char *	delim = "+,-", *t;
	int	lastn = 0;

	t = strtok(userlist, delim);
	while (t != NULL) {
	    number = atoi(t);
	    if (number >= MAXNUM) {
		fprintf(stderr, "%s: %s number %d out of range (< %d)\n",
			program, (mylist == figlist) ? "Move" : "Diagram",
			number, MAXNUM);
		exit(1);
	    }
	    if (mylist == figlist) figlist[n++] = number;
	    else {
		if (!number) {
		    int j;

		    for (j = lastn+1; j < MAXNUM; j++) dialist[j] = 0;
		} else {
		    dialist[number] = number;
		    lastn = number;
		}
	    }
	    t = strtok(NULL, delim);
	}
    }
}

/* #]getlist:		*/ 

