/* #[info:				*/
/************************************************************************
*									*
*			   /\  |\	    /\				*
*			  ||	\\	   ||				*
*		    _-_, =||=  / \\  _-_  =||=	_-_,			*
*		   ||_.	  ||  || || || \\  ||  ||_.			*
*		    ~ ||  ||  || || ||/	   ||	~ ||			*
*		   ,-_-	  \\,  \\/  \\,/   \\, ,-_-			*
*									*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : sfdefs.h							*
* Purpose : Read ISHI format Go games					*
* Version : 1.7								*
* Modified: 01 Aug 1994 18:33:27					*
* Authors : Wayne A. Lobb						*
* Mods.	  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:				*/ 
/* #[copyright:				*/
/********  Copyright (C) 1990	Oxbow Research Inc.  ************************

    Filename: stdform.h

    Header for parsing Standard Format Go files

    Development begun:	8 March 1990	    Wayne A. Lobb

****************************************************************************/

/* #]copyright:				*/ 

#ifndef SFDEFS_H
#define SFDEFS_H

/* #[include:				*/

#include "gogame.h"

/* #]include:				*/ 
/* #[define -- general:			*/
/*------------------------------------------------------------------------*\
		       General constants
\*------------------------------------------------------------------------*/
#define SF_TRUE	    1
#define SF_FALSE    0

#define S_OK	    0		/* everything's A-OK */
#define S_ERR	    1		/* error(s) found */
#define S_QUIT	    2		/* terminate process */

/* #]define -- general:			*/ 
/* #[define -- Go related:		*/
/*------------------------------------------------------------------------*\
		      Go-related constants
\*------------------------------------------------------------------------*/
#define NODATA	    (-1)

#define MINBOARD    2
#define MAXBOARD    19

#define BLACKPC	    0
#define WHITEPC	    1

/*
 *  move types
 */
#define EVENTMOVE	0
#define SETUPMOVE	1
#define GAMEMOVE	2
#define VARIATION	3
#define DIAGRAM		4

/*------------------------------------------------------------------------*\
		      File-i/o-related constants
\*------------------------------------------------------------------------*/
#define MAXLINELEN	256	/* 1 + max acceptable input line */
#define MAXKEYWORDLEN	15	/* max length of a file-line keyword */
#define MAXREMLEN	300	/* max length of one event/move's remarks */
#define MAXCOMLEN	5000	/* max length of one move's commentary */
#define MAXUSERLEN	5000	/* max length of one move's user data */

/*------------------------------------------------------------------------*\
		     Standard Format keywords
\*------------------------------------------------------------------------*/
/*
 *	keyword indexes
 */
#define MAXFILEKEY	29

#define K_ANALYSIS	0
#define K_BLACKMOVE	1
#define K_BLACKPLAYER	2
#define K_BOARDSIZE	3
#define K_COM		4
#define K_DATE		5
#define K_DIAGRAM	6
#define K_ENDCOM	7
#define K_ENDUSER	8
#define K_ENDVAR	9
#define K_EVENT		10
#define K_HANDICAP	11
#define K_HIDE		12
#define K_KOMI		13
#define K_MARK		14
#define K_PLACE		15
#define K_PRISONER	16
#define K_RECORDER	17
#define K_REMARK	18
#define K_RESULT	19
#define K_RULES		20
#define K_SETUP		21
#define K_SOURCE	22
#define K_TIMELIMIT	23
#define K_UNMARK	24
#define K_USER		25
#define K_VAR		26
#define K_WHITEMOVE	27
#define K_WHITEPLAYER	28

/* #]define -- Go related:		*/ 
/* #[typedef -- key handler:		*/
/*
 *	keyword-handler data structure
 */
#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

typedef struct file_key_data {
	char	keyword[MAXKEYWORDLEN];
	int	(*handler) PROTO((int index, char *buf));
} FILE_KEY;

#undef PROTO

/* #]typedef -- key handler:		*/ 
/* #[define -- read errors:		*/
/*------------------------------------------------------------------------*\
	   Parsing-related error strings ("RE" = Read Error)
\*------------------------------------------------------------------------*/
#define MAXREADERRS	    23

#define RE_NOMEMORY	    0
#define RE_UNPARSED	    1
#define RE_BOARDSIZE	    2
#define RE_SIZE_TWICE	    3
#define RE_LOCATION	    4
#define RE_SEQUENCE	    5
#define RE_COM_UNOPENED	    6
#define RE_COM_UNCLOSED	    7
#define RE_COM_TOO_LONG	    8
#define RE_USER_UNOPENED    9
#define RE_USER_UNCLOSED    10
#define RE_USER_TOO_LONG    11
#define RE_VAR_UNOPENED	    12
#define RE_VAR_UNCLOSED	    13
#define RE_PARTIALMOVE	    14
#define RE_MOVENUMBER	    15
#define RE_MOVERANGE	    16
#define RE_PRISONER	    17
#define RE_VAR_NOMOVE	    18
#define RE_MARK		    19
#define RE_SETUP_LATE	    20
#define RE_SETUP_COM	    21
#define RE_SETUP_MARK	    22

/* #]define -- read errors:		*/ 
/* #[prototype:				*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

/* sfread.c */
GOGAME *read_standard		PROTO((char *filename));
int	decode_location		PROTO((int size, char *buf,
					int *col, int *row));
void	encode_location		PROTO((int size, int col, int row ,
					char str[]));
int	decode_color		PROTO((char *buf, int *col));
int	decode_move_range	PROTO((char *buf, int *from, int *to));
int	echo_read_error		PROTO((int index, char *str));

/* sfstore.c */
void	init_recording		PROTO((void));
int	record_boardsize	PROTO((int size));
int	record_commentary	PROTO((char *buf));
int	record_endvar		PROTO((void));
int	record_event		PROTO((char *name));
int	record_hide		PROTO((int col, int row));
int	record_mark		PROTO((int all, int from, int to,
					char *letters, int col, int row));
int	record_move		PROTO((int indiagram, int color,
					int index, int col, int row));
int	record_prisoner		PROTO((int number, int col, int row));
int	record_remark		PROTO((char *buf));
int	record_unmark		PROTO((int all, int from, int to,
					int col, int row));
int	record_userdata		PROTO((char *buf));
int	record_var		PROTO((char *buf));
int	record_pass		PROTO((int side, int n));

#undef PROTO

/* #]prototype:				*/ 

#endif
