/* #[info:		*/
/************************************************************************
*									*
*			   /\			 |\			*
*			  ||		    _	  \\			*
*		    _-_, =||= ,._-_  _-_   < \,	 / \\			*
*		   ||_.	  ||   ||   || \\  /-|| || ||			*
*		    ~ ||  ||   ||   ||/	  (( || || ||			*
*		   ,-_-	  \\,  \\,  \\,/   \/\\	 \\/			*
*									*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : sfread.c							*
* Purpose : Read Standard format Go game				*
* Version : 1.11							*
* Modified: 22 Mar 1995 09:43:13					*
* Author  : Wayne A. Lobb						*
* Mods.	  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[copyright:		*/
/********  Copyright (C) 1990	Oxbow Research Inc.  ************************

    Filename: sfread.c

    Reads Standard Format Go files (to the extent that GoScribe 2.0 does)

    Development begun:	8 March 1990	    Wayne A. Lobb

	Builds under Turbo C 2.0 and runs under DOS 3.0+ without graphics.

	No warranties are made or implied. No claims are made about
	beauty, rigor or portability of this code. It does work, however.

	Motivated by and based on work done in late 1989 by Bob High of AGA.

	May be used for any and all purposes whatever EXCEPT sale.

****************************************************************************/

/* #]copyright:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "gogame.h"
#include "tools.h"
#include "sfdefs.h"

#ifdef __STRICT_ANSI__
char * strdup           (const char *);
int    strncasecmp      (const char *s1, const char *s2, size_t n);
int    isascii          (int c);
#endif

/* #]include:		*/ 
/* #[extern:		*/

extern char *	program;		/* program name		*/

/* #]extern:		*/ 
/* #[global:		*/

GOGAME *	sf_g;			/* Exported for sfstore routines */

/* #]global:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#       define  PROTO(s) s
#else
#       define  PROTO(s) ()
#endif

/*------------------------------------------------------------------------*\
	    Standard Format keywords and their handlers
\*------------------------------------------------------------------------*/
static int	read_remark		PROTO((int key, char *buf));
static int	read_event		PROTO((int key, char *buf));
static int	read_boardsize		PROTO((int key, char *buf));
static int	record_head_data	PROTO((int key, char *buf));
static int	read_setup		PROTO((int key, char *buf));
static int	read_bmove		PROTO((int key, char *buf));
static int	read_wmove		PROTO((int key, char *buf));
static int	read_prisoners		PROTO((int key, char *buf));
static int	read_mark		PROTO((int key, char *buf));
static int	read_unmark		PROTO((int key, char *buf));
static int	read_com		PROTO((int key, char *buf));
static int	read_endcom		PROTO((int key, char *buf));
static int	read_user		PROTO((int key, char *buf));
static int	read_enduser		PROTO((int key, char *buf));
static int	read_var		PROTO((int key, char *buf));
static int	read_endvar		PROTO((int key, char *buf));
static int	read_diagram		PROTO((int key, char *buf));
static int	read_hide		PROTO((int key, char *buf));

static int	parse_file_line		PROTO((char *buf));
static void	init_reading		PROTO((void));
static void	clear_buffers		PROTO((void));
static int	record_reset_movetext	PROTO((void));
static int	add_comment_line	PROTO((char *buf));
static int	add_user_line		PROTO((char *buf));

/*
 * general Go related
 */
static int	column_from_index	PROTO((int size, int index, int *cptr));
static int	index_from_column	PROTO((int ch, int *index));

/*
 * general non-Go related
 */
static void	shave_string		PROTO((char str[]));
static int	next_word		PROTO((char *buf, int *offset,
						char wordbuf[], int maxword));

#if !defined (ATARI)
#define NOSTRCMPI
#endif
#if defined (NOSTRCMPI)
static int	strcmpi			PROTO((char *s1, char *s2));
#endif

#undef PROTO

/* #]prototype:		*/ 
/* #[static -- keys:	*/

static FILE_KEY keys[MAXFILEKEY] = {
	{ "Analysis",	record_head_data    },	/*  0 */
	{ "B",		read_bmove	    },	/*  1 */
	{ "Black",	record_head_data    },	/*  2 */
	{ "BOARDSIZE",	read_boardsize	    },	/*  3 */
	{ "COM",	read_com	    },	/*  4 */
	{ "Date",	record_head_data    },	/*  5 */
	{ "DIAGRAM",	read_diagram	    },	/*  6 */
	{ "ENDCOM",	read_endcom	    },	/*  7 */
	{ "ENDUSER",	read_enduser	    },	/*  8 */
	{ "ENDVAR",	read_endvar	    },	/*  9 */
	{ "EVENT",	read_event	    },	/* 10 */
	{ "Handicap",	record_head_data    },	/* 11 */
	{ "HIDE",	read_hide	    },	/* 12 */
	{ "Komi",	record_head_data    },	/* 13 */
	{ "MARK",	read_mark	    },	/* 14 */
	{ "Place",	record_head_data    },	/* 15 */
	{ "PRISONER",	read_prisoners	    },	/* 16 */
	{ "Recorder",	record_head_data    },	/* 17 */
	{ "REMARK",	read_remark	    },	/* 18 */
	{ "Result",	record_head_data    },	/* 19 */
	{ "Rules",	record_head_data    },	/* 20 */
	{ "SETUP",	read_setup	    },	/* 21 */
	{ "Source",	record_head_data    },	/* 22 */
	{ "Timelimit",	record_head_data    },	/* 23 */
	{ "UNMARK",	read_unmark	    },	/* 24 */
	{ "USER",	read_user	    },	/* 25 */
	{ "VAR",	read_var	    },	/* 26 */
	{ "W",		read_wmove	    },	/* 27 */
	{ "White",	record_head_data    } 	/* 28 */
};

/* #]static -- keys:	*/ 
/* #[static:		*/
/*------------------------------------------------------------------------*\
		  Local constants and structures
\*------------------------------------------------------------------------*/

static char *empty_string = "";
static long int fileline;
static int movenum;			/* # in current event (error msg) */

/*
 *	buffers
 */
static char remarkbuf[MAXREMLEN];  /* remark(s) prefixed to current move   */
static char cbuf     [MAXCOMLEN];  /* commentary postfixed to current move */
static char userbuf  [MAXUSERLEN]; /* user-data postfixed to current move  */

/*
 *	events
 */
static int eventcount;	    /* number of distinct EVENTs so far in the file */
static int eventsize;	    /* current event's board size, default MAXBOARD */
static int movecount;	    /* moves in current event NOT counting setups   */
static int setupcount;	    /* setup stones in current event		    */

/*
 *	moves etc. within events
 */
static int indiagram;	    /* SF_TRUE if current data applies to a diagram */
static int incomment;	    /* SF_TRUE if currently within commentary block */
static int comment_status;  /* S_OK or S_ERR, latter if overflow found   */
static int inuser;	    /* SF_TRUE if currently within user block	 */
static int user_status;	    /* S_OK or S_ERR, latter if overflow found	 */
static int read_vardepth;   /* stacked depth of variations		 */

/* #]static:		*/ 

/* #[read_standard:	*/

GOGAME *
read_standard(filename)
/*
 *	Read Standard-Format game file (or stdin)
 */
char *filename;
{
    GOGAME *g = sf_g = game_alloc((GOGAME *) 0);
    FILE   *fp = stdin;
    char    linebuf[MAXLINELEN];

    memset(linebuf, 0, sizeof(linebuf));
    init_reading();

    if (filename && (fp = fopen(filename, "r")) == (FILE *) 0) {
	message("%s: Can't open \"%s\"\n", program, filename);
	exit(1);
    }
    while (fgets(linebuf, MAXLINELEN, fp) != (char *) NULL) {
	++fileline;
	if (parse_file_line(linebuf) == S_QUIT) return ((GOGAME *) 0);
	memset(linebuf, 0, sizeof(linebuf));
    }
    fclose(fp);

    if (incomment)	   echo_read_error(RE_COM_UNCLOSED , empty_string);
    if (inuser)		   echo_read_error(RE_USER_UNCLOSED, empty_string);
    if (read_vardepth > 0) echo_read_error(RE_VAR_UNCLOSED , empty_string);
    (void) record_reset_movetext();

    return g;
}

/* #]read_standard:	*/ 
/* #[parse_file:	*/

static int
parse_file_line (buf)
/*
 *	parse_file_line
 */
char *buf;
{
    int i, offset, parsed, status;
    char nextword[MAXLINELEN];

    shave_string(buf);

    /*
     *	get keyword
     */
    offset = 0;
    next_word(buf, &offset, nextword, MAXLINELEN-1);

    /*
     *	add a line to current move's commentary?
     */
    if (incomment && strcmpi(nextword, keys[K_ENDCOM].keyword) != 0)
	return (add_comment_line(buf));

    /*
     *	add a line to current move's user-defined data?
     */
    if (inuser	 &&  strcmpi(nextword, keys[K_ENDUSER].keyword) != 0)
	return (add_user_line(buf));

    /*
     *	parse some other nonempty line
     */
    status = S_OK;
    parsed = SF_FALSE;

    if ((int) strlen(nextword) > 0) {
	for (i = 0; !parsed  &&  i < MAXFILEKEY; ++i) {
	    if (keys[i].handler != NULL  &&
		strcmpi(nextword, keys[i].keyword) == 0) {
		parsed = SF_TRUE;
		status = (*keys[i].handler)(i, buf+offset);
	    }
	}

	if (parsed)	return (status);
	else		return (echo_read_error(RE_UNPARSED, buf));
    }
    return (S_OK);
}

/* #]parse_file:	*/ 
/* #[init_reading:	*/

static void
init_reading ()
/*
 *	init_reading
 */
{
    movenum	  = 0;
    movecount	  = 0;
    fileline	  = 0L;
    eventcount	  = 0;
    setupcount	  = 0;

    inuser	  = SF_FALSE;
    incomment	  = SF_FALSE;
    indiagram	  = SF_FALSE;
    read_vardepth = 0;
    eventsize	  = MAXBOARD;

    clear_buffers();
}

/* #]init_reading:	*/ 
/* #[clear_buffers:	*/

static void
/*
 *	clear_buffers
 */
clear_buffers ()
{
    memset(cbuf     , 0, sizeof(cbuf));
    memset(userbuf  , 0, sizeof(userbuf));
    memset(remarkbuf, 0, sizeof(remarkbuf));
}

/* #]clear_buffers:	*/ 

/* #[key handlers:	*/
/*------------------------------------------------------------------------*\
	    Keyword handler routines in alphabetical order
\*------------------------------------------------------------------------*/
/* #[read_bmove:	*/

static int
read_bmove (keyindex, buf)
/*
 *	{ "B",		read_bmove	    },
 */
int keyindex;
char *buf;
{
    int col, row, index, offset;
    char nextword[MAXLINELEN];

    indiagram = SF_FALSE;

    if (record_reset_movetext() == S_QUIT) return (S_QUIT);

    /*
     *	get move number
     */
    offset = 0;
    if (next_word(buf, &offset, nextword, MAXLINELEN-1) == 0)
	return (echo_read_error(RE_PARTIALMOVE, empty_string));

    if ((index = atoi(nextword)) < 1)
	return (echo_read_error(RE_MOVENUMBER, nextword));

    movenum = index;

    /*
     *	get board location
     */
    if (next_word(buf, &offset, nextword, MAXLINELEN-1) == 0)
	return (echo_read_error(RE_PARTIALMOVE, empty_string));

    col = row = 0;

    /*
     *	check for "pass" and "tenuki"
     */
    if (!strcmpi(nextword, "Pass"  ) ||
	!strcmpi(nextword, "Tenuki")) {
	record_pass(BLACKPC, index);
	return (S_OK);
    }

    /*
     *	record move
     */
    if (!decode_location(eventsize, nextword, &col, &row))
	return (echo_read_error(RE_LOCATION, nextword));
    ++movecount;
    return (record_move(indiagram, BLACKPC, index, col, row));
}

/* #]read_bmove:	*/ 
/* #[read_boardsize:	*/

static int
read_boardsize (keyindex, buf)
/*
 *	{ "BOARDSIZE",	read_boardsize	    },
 */
int keyindex;
char *buf;
{
    int size;

    indiagram = SF_FALSE;

    if ((size = atoi(buf)) > MAXBOARD  ||  size < MINBOARD)
	return (echo_read_error(RE_BOARDSIZE, buf));

    if (size != eventsize) {
	if (setupcount > 0  ||  movecount > 0)
	    return (echo_read_error(RE_SIZE_TWICE, buf));

	eventsize = size;
    }
    return (record_boardsize(eventsize));
}

/* #]read_boardsize:	*/ 
/* #[read_com:		*/

static int
read_com (keyindex, buf)
/*
 *	{ "COM",	read_com	    },
 */
int keyindex;
char *buf;
{
    if (incomment)
	return (echo_read_error(RE_COM_UNCLOSED, empty_string));

    incomment = SF_TRUE;
    comment_status = S_OK;

    shave_string(buf);

    if ((int) strlen(buf) > 0) {
	strcat(buf, "\n\n");
	return (add_comment_line(buf));
    }
    return (S_OK);
}

/* #]read_com:		*/ 
/* #[read_diagram:	*/

static int
read_diagram (keyindex, buf)
/*
 *	{ "DIAGRAM",	read_diagram	    },
 */
int keyindex;
char *buf;
{
    indiagram = SF_TRUE;

    if (record_reset_movetext() == S_QUIT) return (S_QUIT);

    ++movecount;

    return (record_move(indiagram, BLACKPC, (-1), 0, 0));
}

/* #]read_diagram:	*/ 
/* #[read_endcom:	*/

static int
read_endcom (keyindex, buf)
/*
 *	{ "ENDCOM",	read_endcom	    },
 */
int keyindex;
char *buf;
{
    if (!incomment)
	return (echo_read_error(RE_COM_UNOPENED, empty_string));

    incomment = SF_FALSE;
    comment_status = S_OK;

    return (S_OK);
}

/* #]read_endcom:	*/ 
/* #[read_enduser:	*/

static int
read_enduser (keyindex, buf)
/*
 *	{ "ENDUSER",	read_enduser	    },
 */
int keyindex;
char *buf;
{
    if (inuser == SF_FALSE)
	return (echo_read_error(RE_USER_UNOPENED, empty_string));

    inuser = SF_FALSE;
    user_status = S_OK;

    return (S_OK);
}

/* #]read_enduser:	*/ 
/* #[read_endvar:	*/

static int
read_endvar (keyindex, buf)
/*
 *	{ "ENDVAR",	read_endvar	    },
 */
int keyindex;
char *buf;
{
    indiagram = SF_FALSE;

    if (record_reset_movetext() == S_QUIT) return (S_QUIT);

    if (movecount < 1) {
	if (echo_read_error(RE_VAR_NOMOVE, empty_string) == S_QUIT)
	    return (S_QUIT);
    }

    if (read_vardepth < 1)
	return (echo_read_error(RE_VAR_UNOPENED, empty_string));

    --read_vardepth;
    return (record_endvar());
}

/* #]read_endvar:	*/ 
/* #[read_event:	*/

static int
read_event (keyindex, buf)
/*
 *	{ "EVENT",	read_event	    },
 */
int keyindex;
char *buf;
{
    indiagram = SF_FALSE;

    if (record_reset_movetext() == S_QUIT) return (S_QUIT);

    if (movecount > 0 && eventcount == 0)
	eventcount = 2;
    else
	++eventcount;

    movenum    = 0;
    movecount  = 0;
    setupcount = 0;
    eventsize  = MAXBOARD;

    shave_string(buf);

    return (record_event(buf));
}

/* #]read_event:	*/ 
/* #[read_hide:		*/

static int
read_hide (keyindex, buf)
/*
 *	{ "HIDE",	read_hide	    },
 */
int keyindex;
char *buf;
{
    int col, row, offset;
    char nextword[MAXLINELEN];

    if (!indiagram)
	return (echo_read_error(RE_MARK, keys[keyindex].keyword));

    offset = 0;

    while (next_word(buf, &offset, nextword, MAXLINELEN-1) > 0) {
	if (decode_location(eventsize, nextword, &col, &row)) {
	    if (record_hide(col, row) == S_QUIT) return (S_QUIT);
	}
    }
    return (S_OK);
}

/* #]read_hide:		*/ 
/* #[read_mark:		*/

static int
read_mark (keyindex, buf)
/*
 *	{ "MARK",	read_mark	    },
 */
int keyindex;
char *buf;
{
    char *strrchr();

    int i, j, from, to, col, row, offset, atoffset;
    char nextword[MAXLINELEN], parseword[MAXLINELEN], letters[MAXLINELEN];

    /*
     *	formats for marks:
     *	    ALL			show all move numbers
     *	    <location>		move number at location:    q8 A5
     *	    #<number>		move number		    #211
     *	    #<num>-<num>	range of move numbers	    #31-42
     *	    <cc>@<loc>		1 or 2 letters at loc	    a@r14 10@d17
     *				(special chars: \t \c \d \s)
     *	    <cc>@#<num>		1 or 2 chars on move	    zz@#59
     *	    <cc>@#<num>-<num>	1 or 2 chars on move range  x@#3-7 21@#9-12
     */

    offset = 0;

    while (next_word(buf, &offset, nextword, MAXLINELEN-1) > 0) {
	memset(letters, 0, sizeof(letters));

	if (strcmpi(nextword, "all") == 0) {
	    if (record_mark(SF_TRUE, 0, 0, letters, 0, 0) == S_QUIT)
		return (S_QUIT);
	}
	else {
	    atoffset = (-1);
	    row = col = from = to = NODATA;

	    j = (int) strlen(nextword);

	    for (i = j-1; i >= 0; --i) {
		if (nextword[i] == '@') {
		    atoffset = i;
		    strncpy(letters, nextword, i);
		    break;
		}
	    }

	    if (atoffset >= 0)
		strcpy(parseword, nextword + atoffset + 1);
	    else
		strcpy(parseword, nextword);

	    if ((int) strlen(letters) > 2) {
		if (echo_read_error(RE_MARK, nextword) == S_QUIT)
		    return (S_QUIT);
	    }
	    else if (decode_location(eventsize, parseword, &col, &row)) {
		if (record_mark(SF_FALSE, 0, 0, letters, col, row) == S_QUIT)
		    return (S_QUIT);
	    }
	    else if (decode_move_range(parseword, &from, &to)) {
		if (record_mark(SF_FALSE, from, to, letters, 0, 0) == S_QUIT)
		    return (S_QUIT);
	    }
	    else if (echo_read_error(RE_MARK, nextword) == S_QUIT)
		return (S_QUIT);
	}
    }
    return (S_OK);
}

/* #]read_mark:		*/ 
/* #[read_prisoners:	*/

static int
read_prisoners (keyindex, buf)
/*
 *	{ "PRISONER",	read_prisoners	    },
 */
int keyindex;
char *buf;
{
    int col, row, from, to, offset;
    char nextword[MAXLINELEN];

    if (movecount < 1 ||  indiagram)
	return (echo_read_error(RE_PRISONER, buf));

    /*
     *	formats for prisoner records:
     *	    <location>	    capture stone at location:	    q8 A5
     *	    #<number>	    capture numbered move	    #211 #31
     */

    offset = 0;

    while (next_word(buf, &offset, nextword, MAXLINELEN-1) > 0) {
	from = to = row = col = 0;

	if (decode_location(eventsize, nextword, &col, &row)) {
	    if (record_prisoner(0, col, row) == S_QUIT)
		return (S_QUIT);
	}
	else if (decode_move_range(nextword, &from, &to)) {
	    if (from != to) {
		if (echo_read_error(RE_MOVERANGE, nextword) == S_QUIT)
			return (S_QUIT);
	    }
	    else {
		if (record_prisoner(from, 0, 0) == S_QUIT)
		    return (S_QUIT);
	    }
	}

	else if (echo_read_error(RE_PRISONER, nextword) == S_QUIT)
	    return (S_QUIT);
    }

    return (S_OK);
}

/* #]read_prisoners:	*/ 
/* #[read_remark:	*/

static int
read_remark (keyindex, buf)
/*
 *	{ "REMARK",	read_remark	    },
 */
int keyindex;
char *buf;
{
    shave_string(buf);

    if (!buf) return (S_OK);
    if ((int) (strlen(remarkbuf) + strlen(buf)) > MAXREMLEN-1) return (S_OK);

    /*
     * Seperate remarks by newlines
     */
    if (*remarkbuf) strcat(remarkbuf, "\n");

    strcat(remarkbuf, buf);

    return (S_OK);
}

/* #]read_remark:	*/ 
/* #[read_setup:	*/

static int
read_setup (keyindex, buf)
/*
 *	{ "SETUP",	read_setup	    },
 */
int keyindex;
char *buf;
{
    int col, row, color, newcolor, offset;
    char nextword[MAXLINELEN];

    if (record_reset_movetext() == S_QUIT) return (S_QUIT);

    if (movecount > 0  &&  !indiagram)
	return (echo_read_error(RE_SETUP_LATE, buf));

    color    = NODATA;
    newcolor = NODATA;

    /*
     *	each data field must be one of <blackmove>, <whitemove> or <location>
     */
    offset = 0;

    while (next_word(buf, &offset, nextword, MAXLINELEN-1) > 0) {
	if (decode_color(nextword, &newcolor)) {
	    color = newcolor;
	}
	else {
	    if (!decode_location(eventsize, nextword, &col, &row)) {
		if (echo_read_error(RE_LOCATION, nextword) == S_QUIT)
		    return (S_QUIT);
	    }
	    else if (color == NODATA) {
		if (echo_read_error(RE_PARTIALMOVE, empty_string) == S_QUIT)
		    return (S_QUIT);
	    }
	    else {
		if (record_move(indiagram, color, 0, col, row) == S_QUIT)
		    return (S_QUIT);

		++setupcount;
	    }
	}
    }
    return (S_OK);
}

/* #]read_setup:	*/ 
/* #[read_unmark:	*/

static int
read_unmark (keyindex, buf)
/*
 *	{ "UNMARK",	read_unmark	    },
 */
int keyindex;
char *buf;
{
    int from, to, col, row, offset;
    char nextword[MAXLINELEN];

    /*
     *	formats for unmarks:
     *	    ALL			hide all marks
     *	    <location>		hide mark at location:	    q8 A5
     *	    #<number>		hide mark on move	    #211
     *	    #<num>-<num>	hide on range of moves	    #31-42
     */

    offset = 0;

    while (next_word(buf, &offset, nextword, MAXLINELEN-1) > 0) {
	if (strcmpi(nextword, "all") == 0) {
	    if (record_unmark(SF_TRUE, 0, 0, 0, 0) == S_QUIT)
		return (S_QUIT);
	}
	else {
	    row = col = from = to = 0;

	    if (decode_location(eventsize, nextword, &col, &row)) {
		if (record_unmark(SF_FALSE, 0, 0, col, row) == S_QUIT)
		    return (S_QUIT);
	    }
	    else if (decode_move_range(nextword, &from, &to)) {
		if (record_unmark(SF_FALSE, from, to, 0, 0) == S_QUIT)
		    return (S_QUIT);
	    }
	    else if (echo_read_error(RE_MARK, nextword) == S_QUIT)
		return (S_QUIT);
	}
    }
    return (S_OK);
}

/* #]read_unmark:	*/ 
/* #[read_user:		*/

static int
read_user (keyindex, buf)
/*
 *	{ "USER",	read_user	    },
 */
int keyindex;
char *buf;
{
    if (inuser == SF_TRUE)
	return (echo_read_error(RE_USER_UNCLOSED, empty_string));

    inuser = SF_TRUE;
    user_status = S_OK;

    shave_string(buf);
    return (add_user_line(buf));
}

/* #]read_user:		*/ 
/* #[read_var:		*/

static int
read_var (keyindex, buf)
/*
 *	{ "VAR",	read_var	    },
 */
int keyindex;
char *buf;		/* optional variation 1- or 2-letter mark */
{
    indiagram = SF_FALSE;

    if (record_reset_movetext() == S_QUIT) return (S_QUIT);

    if (movecount < 1)
	return (echo_read_error(RE_VAR_NOMOVE, empty_string));

    ++read_vardepth;
    return (record_var(buf));
}

/* #]read_var:		*/ 
/* #[read_wmove:	*/

static int
read_wmove (keyindex, buf)
/*
 *	{ "W",		read_wmove	    },
 */
int keyindex;
char *buf;
{
    int col, row, index, offset;
    char nextword[MAXLINELEN];

    indiagram = SF_FALSE;

    if (record_reset_movetext() == S_QUIT) return (S_QUIT);

    /*
     *	get move number
     */
    offset = 0;
    if (next_word(buf, &offset, nextword, MAXLINELEN-1) == 0)
	return (echo_read_error(RE_PARTIALMOVE, empty_string));

    if ((index = atoi(nextword)) < 1)
	return (echo_read_error(RE_MOVENUMBER, nextword));

    movenum = index;

    /*
     *	get board location
     */
    if (next_word(buf, &offset, nextword, MAXLINELEN-1) == 0)
	return (echo_read_error(RE_PARTIALMOVE, empty_string));

    col = row = 0;

    /*
     *	check for "pass" and "tenuki"
     */
    if (!strcmpi(nextword, "Pass"  ) ||
	!strcmpi(nextword, "Tenuki")) {
	record_pass(WHITEPC, index);
	return (S_OK);
    }

    /*
     *	record move
     */
    if (!decode_location(eventsize, nextword, &col, &row))
	return (echo_read_error(RE_LOCATION, nextword));
    ++movecount;
    return (record_move(indiagram, WHITEPC, index, col, row));
}

/* #]read_wmove:	*/ 
/* #[record_head_data:	*/

static int
record_head_data (keyindex, buf)
/*
 *	{ "Analysis",	record_head_data    },
 *	{ "Black",	record_head_data    },
 *	{ "Date",	record_head_data    },
 *	{ "Handicap",	record_head_data    },
 *	{ "Komi",	record_head_data    },
 *	{ "Place",	record_head_data    },
 *	{ "Recorder",	record_head_data    },
 *	{ "Result",	record_head_data    },
 *	{ "Rules",	record_head_data    },
 *	{ "Source",	record_head_data    },
 *	{ "Timelimit",	record_head_data    },
 *	{ "White",	record_head_data    } 
 */
int keyindex;
char *buf;
{
    shave_string(buf);
    if ((int) strlen(buf) == 0) return (S_OK);

    switch (keyindex) {
      case  0: sf_g->gm_analysis = strdup(buf); break; /* "Analysis"  */
      case  2: sf_g->gm_pb    = strdup(buf); break; /* "Black"     */
      case  5: sf_g->gm_dt     = strdup(buf); break; /* "Date"      */
      case 11: sf_g->gm_ha = strdup(buf); break; /* "Handicap"  */
      case 13: sf_g->gm_km     = strdup(buf); break; /* "Komi"      */
      case 15: sf_g->gm_pc    = strdup(buf); break; /* "Place"     */
      case 17: sf_g->gm_recorder = strdup(buf); break; /* "Recorder"  */
      case 19: sf_g->gm_re   = strdup(buf); break; /* "Result"    */
      case 20: sf_g->gm_rules    = strdup(buf); break; /* "Rules"     */
      case 22: sf_g->gm_so   = strdup(buf); break; /* "Source"    */
      case 23: sf_g->gm_tm     = strdup(buf); break; /* "Timelimit" */
      case 28: sf_g->gm_pw    = strdup(buf); break; /* "White"     */
      default: message("Funny index (%d) in record_head_data\n", keyindex);
    }
    return (S_OK);
}

/* #]record_head_data:	*/ 
/* #[record_reset_movetext:*/

static int
/*
 *	record_reset_movetext
 */
record_reset_movetext ()
{
    if ((int) strlen(remarkbuf) > 0) {
	if (record_remark(remarkbuf) == S_QUIT) return (S_QUIT);
    }

    if ((int) strlen(cbuf) > 0) {
	add_comment_line(" ");		/* pad end of commentary a tad */
	if (record_commentary(cbuf) == S_QUIT) return (S_QUIT);
    }

    if ((int) strlen(userbuf) > 0) {
	if (record_userdata(userbuf) == S_QUIT) return (S_QUIT);
    }

    clear_buffers();
    return (S_OK);
}

/* #]record_reset_movetext:*/ 

/* #]key handlers:	*/ 

/*------------------------------------------------------------------------*\
		    Parsing-specific utilities
\*------------------------------------------------------------------------*/
/* #[add_comment_line:	*/

static int
add_comment_line (buf)
/*
 *	add_comment_line
 */
char *buf;
{
    int buflen, cbuflen;

    if (comment_status != S_OK) return (S_OK);

    /*
     *	if nonempty comment, install paragraph or pad end with a space
     */
    if ((cbuflen = (int) strlen(cbuf)) > 0  &&  cbuflen < MAXCOMLEN-2) {
	if ((buflen = (int) strlen(buf)) == 0) {
	    strcpy(buf, "\n\n");
	}
	else if (!isspace(buf[buflen-1])  &&  !isspace(cbuf[cbuflen-1])) {
	    strcat(cbuf, " ");
	    cbuflen += 1;
	}
    }

    /*
     *	append new text only if it's nonempty and won't cause overflow;
     *	allow an extra character for next pad-space
     */
    if ((buflen = (int) strlen(buf)) == 0) return (S_OK);

    if (buflen + cbuflen + 1 > MAXCOMLEN)
	return (echo_read_error(RE_COM_TOO_LONG, buf));

    strcat(cbuf, buf);
    return (S_OK);
}

/* #]add_comment_line:	*/ 
/* #[add_user_line:	*/

static int
add_user_line (buf)
/*
 *	add_user_line
 */
char *buf;
{
    int ubuflen;

    if (user_status != S_OK) return (S_OK);

    /*
     *	add to user buffer if room; allow 1 char for CRLF
     */
    if ((ubuflen = (int) strlen(userbuf)) + (int) strlen(buf) + 1
						> MAXUSERLEN-1) {
	user_status = S_ERR;
	return (echo_read_error(RE_USER_TOO_LONG, buf));
    }

    /*
     *	prefix this line with CRLF. then append it
     */
    if (ubuflen > 0) strcat(userbuf, "\n");

    strcat(userbuf, buf);

    return (S_OK);
}

/* #]add_user_line:	*/ 
/* #[decode_location:	*/

int
decode_location (boardsize, buf, colptr, rowptr)
/*
 *	DECODE_LOCATION
 *
 *	Returns SF_TRUE if passed buf consists exactly of a valid location;
 *	if so, fills column and row indexes in range 1...boardsize.
 *	Otherwise, returns SF_FALSE.
 */
int boardsize;
char *buf;
int *colptr, *rowptr;
{
    int buflen;

    *colptr = *rowptr = NODATA;

    if ((buflen = (int) strlen(buf)) < 2 || buflen > 3) return (SF_FALSE);

    if (!index_from_column((int)(buf[0]), colptr) || *colptr > boardsize)
	return (SF_FALSE);

    if ((*rowptr = atoi(buf+1)) < 1 || *rowptr > boardsize) {
	*rowptr = NODATA;
	return (SF_FALSE);
    }

    return (SF_TRUE);
}

/* #]decode_location:	*/ 
/* #[encode_location:	*/

void
encode_location (board_size, col, row, str)
/*
 *	ENCODE_LOCATION
 */
int board_size;
int col, row;		/* 1...board_size */
char str[];		/* place 2- or 3-letter composed string here */
{
    int colname;

    strcpy(str, "???");

    if (!column_from_index(board_size, col, &colname) ||
	row < 1 || row > board_size) {
	return;
    }
    sprintf(str, "%c%d", (char)(colname), row);
}

/* #]encode_location:	*/ 
/* #[decode_color:	*/

int
decode_color (buf, colorptr)
/*
 *	DECODE_COLOR
 *
 *	Returns SF_TRUE if passed buf consists exactly of a
 *	valid stone-color keyword; if so, fills in the color index.
 *	Otherwise, returns SF_FALSE.
 */
char *buf;
int *colorptr;
{
    *colorptr = NODATA;

	 if (strcmpi(buf, keys[K_BLACKMOVE].keyword) == 0) *colorptr = BLACKPC;
    else if (strcmpi(buf, keys[K_WHITEMOVE].keyword) == 0) *colorptr = WHITEPC;
    else return (SF_FALSE);

    return (SF_TRUE);
}

/* #]decode_color:	*/ 
/* #[decode_move_range:	*/

int
decode_move_range (buf, fromptr, toptr)
/*
 *	DECODE_MOVE_RANGE
 *
 *	Returns SF_TRUE if passed buffer consists exactly of a move number or
 *	a range of move numbers; if so, fills start and end numbers.
 *	Otherwise, returns SF_FALSE.
 */
char *buf;
int *fromptr, *toptr;
{
    int i, j;
    char numstr[5];

    *fromptr = *toptr = 0;

    if (buf[0] != '#') return (SF_FALSE);

    /*
     *	get starting number
     */
    i = 1;
    j = 0;
    memset(numstr, 0, sizeof(numstr));
    while (j < 4  &&  isdigit(buf[i])) numstr[j++] = buf[i++];
    if ((*fromptr = atoi(numstr)) < 1) {
	*fromptr = 0;
	return (SF_FALSE);
    }

    /*
     *	get ending number if any
     */
    *toptr = *fromptr;
    if (buf[i++] != '-') return (SF_TRUE);

    j = 0;
    memset(numstr, 0, sizeof(numstr));
    while (j < 4  &&  isdigit(buf[i])) numstr[j++] = buf[i++];
    if ((*toptr = atoi(numstr)) < 1) {
	*fromptr = *toptr = 0;
	return (SF_FALSE);
    }

    return (SF_TRUE);
}

/* #]decode_move_range:	*/ 
/* #[echo_read_error:	*/

/* #[error messages:	*/

static char *
read_err_strings[] = {
	"Not enough memory",		/* RE_NOMEMORY		*/
	"Unable to use data",		/* RE_UNPARSED		*/
	"Unusable board size",		/* RE_BOARDSIZE		*/
	"Size changed after move",	/* RE_SIZE_TWICE	*/
	"Unusable board location",	/* RE_LOCATION		*/
	"Out of sequence?",		/* RE_SEQUENCE		*/
	"Unopened comment",		/* RE_COM_UNOPENED	*/
	"Unclosed comment",		/* RE_COM_UNCLOSED	*/
	"Comment too long",		/* RE_COM_TOO_LONG	*/
	"Unopened user data",		/* RE_USER_UNOPENED	*/
	"Unclosed user data",		/* RE_USER_UNCLOSED	*/
	"User data too long",		/* RE_USER_TOO_LONG	*/
	"Unopened variation",		/* RE_VAR_UNOPENED	*/
	"Unclosed variation",		/* RE_VAR_UNCLOSED	*/
	"Incomplete move data",		/* RE_PARTIALMOVE	*/
	"Unusable move number",		/* RE_MOVENUMBER	*/
	"Unusable move range",		/* RE_MOVERANGE		*/
	"Unusable prisoner",		/* RE_PRISONER		*/
	"Unusable variation",		/* RE_VAR_NOMOVE	*/
	"Unusable board mark",		/* RE_MARK		*/
	"Setup after B/W move",		/* RE_SETUP_LATE	*/
	"Setup comment deleted",	/* RE_SETUP_COM		*/
	"Setup un/mark deleted",	/* RE_SETUP_MARK	*/
};

/* #]error messages:	*/ 

int
echo_read_error (index, str)
/*
 *	ECHO_READ_ERROR
 */
int index;			/* in read_err_strings[] */
char *str;			/* supplementary string */
{
    printf("\n*** Line %-6ld  move %-3d ", fileline, movenum);

    if (index < 0  ||  index >= MAXREADERRS)
	printf(" Unknown read error %s\n", str);
    else
	printf("%s %s\n", read_err_strings[index], str);

    return (S_OK);	/* Continue parsing */
}

/* #]echo_read_error:	*/ 

/*--------------------------------------------------------------------------*\
		      General Go-related utilities
\*--------------------------------------------------------------------------*/

static char *column_letters = "abcdefghjklmnopqrst";

/* #[column_from_index:	*/

static int
column_from_index (boardsize, index, cptr)
/*
 *	COLUMN_FROM_INDEX
 */
int boardsize;
int index;			/* 1...boardsize */
int *cptr;			/* column letter */
{
    *cptr = NODATA;

    if (index < 1 || index > boardsize) return (SF_FALSE);

    *cptr = (int)(column_letters[index-1]);
    return (SF_TRUE);
}

/* #]column_from_index:	*/ 
/* #[index_from_column:	*/

static int
index_from_column (ch, index)
/*
 *	INDEX_FROM_COLUMN
 */
int ch;				/* column letter */
int *index;			/* 1...eventsize */
{
    int i, count, lch;

    *index = NODATA;
    lch = tolower(ch);
    count = (int) strlen(column_letters);

    for (i = 0; i < count; ++i) {
	if (lch == column_letters[i]) {
	    *index = i+1;
	    return (SF_TRUE);
	}
    }
    return (SF_FALSE);
}

/* #]index_from_column:	*/ 

/*--------------------------------------------------------------------------*\
		      General non-Go-related utilities
\*--------------------------------------------------------------------------*/

/* #[shave_string:	*/

static void
shave_string (str)
/*
 *	SHAVE_STRING
 */
char str[];
{
    int i   = 0;
    int j   = 0;
    int len = (int) strlen(str);

    while (i < len  &&  isspace(str[i])) ++i;
    while (i < len) str[j++] = str[i++];
    while (j < len) str[j++] = '\0';

    i = (int) strlen(str) - 1;
    while (i >= 0  &&  isspace(str[i])) str[i--] = '\0';
}

/* #]shave_string:	*/ 
/* #[next_word:		*/

static int
next_word (buf, offset, wordbuf, maxword)
/*
 *	NEXT_WORD
 *
 *	Returns number of characters placed in 'wordbuf'.
 *	Passed buffer is assumed to be null-terminated.
 *	Passed 'wordbuf' is assumed to have room for null terminator at end.
 */
char *buf;
int *offset;		/* starting offset in buf */
char wordbuf[];		/* filled by this routine */
int maxword;		/* max chars allowable in worbuf[] */
{
    int i;

    memset(wordbuf, 0, maxword+1);

    /*
     *	skip white space
     */
    while (isspace(buf[*offset])) ++(*offset);

    /*
     *	advance to end or to next white char, copying up to allowable chars
     */
    i = 0;
    while (buf[*offset] != '\0'  &&  !isspace(buf[*offset])) {
	if (i < maxword) wordbuf[i++] = buf[*offset];
	++(*offset);
    }
    return (i);
}

/* #]next_word:		*/ 

#if defined (NOSTRCMPI)
/* #[strcmpi:		*/

static int
strcmpi(s1, s2)
char *s1;
char *s2;
{
    while (*s1) {
	if (tolower(*s1) != tolower(*s2)) return (*s1 - *s2);
	else { s1++; s2++; }
    }
    return 0;
}

/* #]strcmpi:		*/ 
#endif
