/* #[info:		*/
/************************************************************************
*									*
*			/\	   ,					*
*		       ||	  ||					*
*		 _-_, =||=  _-_, =||=  /'\\ ,._-_  _-_			*
*		||_.   ||  ||_.	  ||  || ||  ||	  || \\			*
*		 ~ ||  ||   ~ ||  ||  || ||  ||	  ||/			*
*		,-_-   \\, ,-_-	  \\, \\,/   \\,  \\,/			*
*									*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : sfstore.c							*
* Purpose : Store Standard format Go game				*
* Version : 1.14							*
* Modified: 22 Mar 1995 09:43:14					*
* Author  : Wayne A. Lobb						*
* Mods.	  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[copyright:		*/
/********  Copyright (C) 1990	Oxbow Research Inc.  ************************

    Filename: sfrecord.c

    Echoes data read from Standard Format Go files

    Development begun:	8 March 1990	    Wayne A. Lobb

****************************************************************************/

/* #]copyright:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "gogame.h"
#include "tools.h"
#include "sfdefs.h"

/* #]include:		*/ 
/* #[define:		*/

/* #define SF2P(x,y)	((sf_g->gm_sz-x) + (y-1) * sf_g->gm_sz) */
#define	SF2P(x,y)	((x-1) + (sf_g->gm_sz-y) * sf_g->gm_sz)
#define SF2C(c)		((c == BLACKPC) ? BLACK : WHITE)

/* #]define:		*/ 
/* #[extern:		*/

extern GOGAME *	sf_g;		/* Set by sfread module */

/* #]extern:		*/ 
/* #[static:		*/

static int	boardsize       = MAXBOARD;
static int	at_variation    = SF_FALSE;

/* #]static:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#       define  PROTO(s) s
#else
#       define  PROTO(s) ()
#endif

#undef PROTO

/* #]prototype:		*/ 

/* #[init_recording:	*/

void
init_recording ()
/*
 *	INIT_RECORDING
 */
{
}

/* #]init_recording:	*/ 
/* #[record_event:	*/

int
/*
 *	EVENT
 */
record_event (name)
char *name;
{
    node_store    (sf_g);
    gamename_store(sf_g, name);

    at_variation = SF_FALSE;

    return (S_OK);
}

/* #]record_event:	*/ 
/* #[record_boardsize:	*/

int
record_boardsize (size)
/*
 *	BOARDSIZE
 */
int size;
{
    size_store(sf_g, size);

    at_variation = SF_FALSE;
    boardsize = size;

    return (S_OK);
}

/* #]record_boardsize:	*/ 
/* #[record_var:	*/

int
record_var (buf)
/*
 *	VAR
 */
char *buf;		    /* optional 1- or 2-letter variation mark */
{
    GOMOVE *m;

    sf_g = push_var(sf_g);

    /*
     * Store node and name for the variation
     */
			node_store(sf_g);
    if (buf && *buf)	name_store(sf_g, buf);

    /*
     * Remove the last move
     */
    if ((m = move_current(sf_g->gm_parent)) != NULL)
	delstone_store(sf_g, m->mv_place);
    else
	fprintf(stderr, "Couldn't hide last move!\n");

    at_variation = SF_TRUE;

    return (S_OK);
}

/* #]record_var:	*/ 
/* #[record_endvar:	*/

int
/*
 *	ENDVAR
 */
record_endvar ()
{
    GOGAME *parent = pop_var(sf_g);

    /*
     * The next can't happen in a Standard game
     */
    if (parent == (GOGAME *) 0) return (S_QUIT);
    sf_g = parent;

    at_variation = SF_FALSE;

    return (S_OK);
}

/* #]record_endvar:	*/ 
/* #[record_remark:	*/

int
record_remark (buf)
/*
 *	REMARK
 */
char *buf;
{
    /*
     * First, we allocate a node to hold the following information
     */
    node_store(sf_g);

    /*
     * The remark buffer holds:
     *
     *	<rem1>\n
     *	<rem2>\n
     *	<...>		# Note the missing newline
     */
    text_store(sf_g, "(Remark: ");
    text_store(sf_g, buf);
    text_store(sf_g, ")\n");
    at_variation = SF_FALSE;

    return (S_OK);
}

/* #]record_remark:	*/ 
/* #[record_move:	*/

int
record_move (indiagram, color, index, col, row)
/*
 *	MOVE
 */
int indiagram;		/* SF_TRUE or SF_FALSE		*/
int color;		/* BLACKPC or WHITEPC		*/
int index;		/*  -1 = restore hidden move	*/
			/*   0 = setup move		*/
			/* > 0 = moves			*/
int col, row;
{
    if (indiagram) {
	if (col > 0  &&  row > 0) {
	    addstone_store(sf_g, SF2C(color), SF2P(col,row));
	}
    }
    else if (at_variation) {
	node_store(sf_g);
	move_store(sf_g, index, SF2C(color), SF2P(col,row));
    }
    else {
	if (index != 0) {
	    node_store(sf_g);
	    move_store(sf_g, index, SF2C(color), SF2P(col,row));
	}
	else {
	    addstone_store(sf_g, SF2C(color), SF2P(col,row));
	}
    }
    at_variation = SF_FALSE;
    return (S_OK);
}

/* #]record_move:	*/ 
/* #[record_pass:	*/

int
record_pass(side, n)
int	side;
int	n;
{
    pass_store(sf_g, n, side);
    return (S_OK);
}

/* #]record_pass:	*/ 
/* #[record_prisoner:	*/

int
/*
 *	PRISONER
 */
record_prisoner (number, col, row)
int number;		    /* move number if appropriate */
int col, row;		    /* board location if appropriate */
{
#if 0
    if (number > 0) message("Prisoner (%d)\n", number);
    else	    message("Prisoner at (%d,%d)", col, row);
#endif

    return (S_OK);
}

/* #]record_prisoner:	*/ 
/* #[record_hide:	*/

int
record_hide (col, row)
/*
 *	HIDE
 */
int col, row;		    /* board location */
{
    delstone_store(sf_g, SF2P(col,row));
    at_variation = SF_FALSE;

    return (S_OK);
}

/* #]record_hide:	*/ 
/* #[record_mark:	*/

int
record_mark (all, from, to, letters, col, row)
/*
 *	MARK
 *
 */
int	all;		    /* SF_TRUE = show all move numbers		*/
int	from, to;	    /* range of move numbers if applicable	*/
char *	letters;	    /* 1 or 2 letters at location or on stone	*/
int	col, row;	    /* board location if applicable		*/
{
    at_variation = SF_FALSE;

    if (all) {
	message("Mark all move numbers\n");
	return (S_OK);
    }

    if (from > 0  &&  to > 0) {
	if (from != to) {
	    message("Mark on moves %d to %d\n", from, to);
	    return (S_OK);
	}
	message("Mark on move %d\n", from);
	return (S_OK);
    }

    if (strlen(letters) == 1  &&
	(LETTER_MIN <= *letters && *letters <= LETTER_MAX))
	/*
	 * Store these as letters
	 */
	letter_store(sf_g, SF2P(col,row), *letters);
    else
	/*
	 * Store these as marks
	 */
	mark_store(sf_g, SF2P(col,row), *letters ? letters : AUTO_MARK);

    return (S_OK);
}

/* #]record_mark:	*/ 
/* #[record_unmark:	*/

int
record_unmark (all, from, to, col, row)
/*
 *	UNMARK
 */
int all;		    /* SF_TRUE = hide all marks */
int from, to;		    /* range of move numbers if applicable */
int col, row;		    /* board location if applicable */
{
    char mark_loc[5];

    message("Unmark");

    if (all) {
	message(" all marks\n");
    }
    else if (from > 0  &&  to > 0) {
	if (from != to)
	    message(" numbers on moves %d to %d\n", from, to);
	else
	    message(" number on move %d\n", from);
    }
    else {
	encode_location(boardsize, col, row, mark_loc);
	message(" mark on point or stone at %s\n", mark_loc);
    }
    at_variation = SF_FALSE;

    return (S_OK);
}

/* #]record_unmark:	*/ 
/* #[record_userdata:	*/

int
record_userdata (buf)
/*
 *	USER
 */
char *buf;
{
    user_store(sf_g, buf);
    at_variation = SF_FALSE;

    return (S_OK);
}

/* #]record_userdata:	*/ 
/* #[record_commentary:	*/

#define MAXLINE		1024
#define LINELENGTH	64		/* Limit linelength in comments	*/
#define SPACE		' '
#define TAB		'\t'
#define NEWLINE		'\n'
#define WHITESPACE(c)	((c) == SPACE || (c) == TAB || (c) == NEWLINE)

int
record_commentary (buf)
/*
 *	COM/ENDCOM
 */
char *buf;
{
    char line[MAXLINE];
    int  i;
    char *s;

    memset(line, 0, MAXLINE); i = 0;

    for (s = buf; *s; s++) {
	line[i++] = *s;
	if (i >= LINELENGTH && WHITESPACE(*s)) {
	    text_store(sf_g, line);
	    memset(line, 0, MAXLINE); i = 0;
	    /*
	     * Skip leading whitespace
	     */
	    while (*(s+1) && WHITESPACE(*(s+1))) s++;
	}
    }
    text_store(sf_g, line);

    at_variation = SF_FALSE;

    return (S_OK);
}

/* #]record_commentary:	*/ 

