/* #[info:		*/
/************************************************************************
*									*
*			    /\	/\					*
*		      _	   ||  (  )	      '				*
*	       _-_,  / \\ =||=	 // \\/\\/\\ \\	 _-_,  _-_		*
*	      ||_.  || ||  ||	//  || || || || ||_.  ||		*
*	       ~ || || ||  ||  /(   || || || ||	 ~ || ||		*
*	      ,-_-  \\_-|  \\, {___ \\ \\ \\ \\ ,-_-  \\,/		*
*		     /	\						*
*		    '----`						*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : sgf2misc.h							*
* Purpose : Read/write Go games in various formats			*
* Version : 1.14							*
* Modified: 03 May 1995 12:06:52					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 

#ifndef SGF2MISC_H
#define SGF2MISC_H

/* #[include:		*/

#include "gogame.h"

/* #]include:		*/ 
/* #[define:		*/

#if defined (ATARI)
#define TMPDIR		"k:"
#define TMPFILE		"k:\\xxx"
#define	DIRSEP		'\\'
#else
#define TMPDIR		"/tmp"
#define TMPFILE		"./xxx"
#define	DIRSEP		'/'
#endif

/*
 * Supported input formats
 */
#define	FMT_SGF		0
#define	FMT_LIB		1
#define	FMT_STD		2

/*
 * Supported output devices
 */
#define	DEVICE_PSTEX	0
#define	DEVICE_TEX	1
#define	DEVICE_ASCII	2
#define	DEVICE_SCREEN	3
#define	DEVICE_GIF	4
#define	DEVICE_HTML	5
#define	DEVICE_NULL	6
#define	DEVICE_DB	7
#define	DEVICE_SGF	8
#define	DEVICE_SF	9

/*
 * Supported paper sizes
 */
#define PAPER_DEFAULT  	0
#define PAPER_LETTER  	1
#define PAPER_A4WIDE	2
#define PAPER_A4    	3

#define MAXWIDTH_DEFAULT 180	/* Critical diagram size for twocolumn	*/
#define MAXWIDTH_A4	 MAXWIDTH_DEFAULT
#define MAXWIDTH_A4WIDE	 200	/* A4 wide allows for bigger columns	*/
#define MAXWIDTH_LETTER	 MAXWIDTH_DEFAULT

#define	MAXNUM		1023	/* Maximum for fig/dia lists		*/
#define MAXUSER		1024	/* Maximum length of username		*/
#define MAXKIBITZER	256	/* Maximum number of kibitzers		*/

/* #]define:		*/ 
/* #[typedef:		*/

typedef struct epsf_info {
    char *	eps_dir;	/* Where the eps files are created	*/
    char *	eps_name;	/* Constructed from input filename	*/
    int		eps_size;	/* The size of a diagram when printed	*/
    int		eps_width;	/* The size of diagram + coordinates	*/
} EPSFINFO;

/* #]typedef:		*/ 
/* #[prototype:		*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

char   *	board_caption	PROTO((GOGAME *g, GOMOVE *m1, GOMOVE *m2));
char   *	con_cat		PROTO((char *s));
void		con_printf	PROTO((char *fmt, ...));

#undef PROTO

/* #]prototype:		*/ 

#endif
