SGF2MISC(L)					      SGF2MISC(L)



NAME
       sgf2misc - Read and write Go games in various formats


SYNOPSIS
       sgf2misc [options] [file]


DESCRIPTION
       The program sgf2misc takes a Go game in SGF, SF or Liberty
       notation as input and produces one of various  formats  as
       output.	Currently, the supported output formats are:

	 * LaTeX text with PostScript diagrams
	 * LaTeX text with LaTeX font diagrams
	 * Screen output (using the curses library)
	 * ASCII (both text and diagrams)
	 * Smart Go Format (SGF)
	 * Standard Format (SF)
	 * db dump (just the game header information)
	 * Plain PostScript (with script n-up/mkrawps)
	 * Null device (no output whatsoever)

       The output is tunable by setting command line options (see
       the options below) and (in case you  are  producing  LaTeX
       output)	by changing settings and macros as defined in the
       style file sgf.sty.


OPTIONS
       While specifying options on  the  command  line,  you  can
       abbreviate  both the option, and its parameter to the sub
       string which is unique amongst the other options or param
       eters in that context.
       For  example,  instead of typing: "-from liberty -diagrams
       0" you can type: "-fr l -di 0" since the strings "fr"  and
       "di"  are  unique  amongst the other options and the input
       format "li" is unique  amongst  the  other  input  formats
       ("sgf" and "sf").
       Here are the options of sgf2misc and a discussion of their
       effect on the  output.  The  options  which  need  special
       attention  will	have  a  special section devoted to them.
       You will be pointed to these sections  while  the  options
       are discussed.

       -fromformat
	   Specify  the input format of the Go game. The sgf2misc
	   program knows about:

	     format   input

	     sgf      SGF output (Smart Go Format)
	     sf       SF output (Standard Format)
	     Liberty  Liberty game format

	   A further discussion on these formats can be found  in
	   the section ON FORMATS.

       -toformat
	   Specify  the  output  format  of  the Go game. You can
	   select any of:

	     format   output

	     pstex    LaTeX text -- PostScript diagrams
	     tex      LaTeX text -- TeX diagrams
	     sgf      SGF output (Smart Go Format)
	     sf       SF output (Standard Format)
	     ascii    ASCII text and diagrams
	     graphic  Screen output (using the curses library)
	     db       Database output (general game info)
	     null     No output whatsoever

	   A further discussion on these formats can be found  in
	   the section ON FORMATS.

       -tmpdirectory
	   During the conversion of the Go game to LaTeX sgf2misc
	   will create EPSF files for each  diagram  it  creates.
	   During  the	consecutive LaTeX run these files will be
	   read and during the dvips  (or  similar  program)  run
	   these files will be imported into the PostScript file.
	   By default sgf2misc creates these files in  /tmp.  You
	   can	overwrite this by using this command line option.
	   You might want to do this if the file system /tmp hap
	   pens to be small on your particular system, and change
	   it to /usr/tmp for example.
	   The disk requirements for these  EPSF  files  are  not
	   dramatic.  While converting a Go Seigen game of Jim Z.
	   Yu, a total of 106 figure and diagram  positions  were
	   written which took an average of 1 kb each, making the
	   total disk space consumption  130 kb.
	   This option is only useful in pstex mode.

       -scalesize
	   This option will change the size of the  Go	diagrams.
	   The	default  size (160 points = 160/72 inch) is quite
	   suitable for the default double column output mode for
	   the	text.	If  you  choose  a size beyond 180 points
	   sgf2misc will automatically switch the text to  single
	   column mode to accommodate for the larger diagrams.
	   This option is only useful in pstex mode.

       -figureslist
	   By default pstex will try to find suitable breakpoints
	   in the submitted Go game to print diagrams.	See  sec
	   tion FIGURE BREAKS for details on this.
	   With the figures option you can overrule this build-in
	   selection scheme. The list argument takes two forms:

	     n1,n2,...,nk

	     %n

	   The first form will create diagrams	of  the  game  on
	   move n1, n2, ..., nk.
	   The	latter option will create figures at move n, 2*n,
	   3*n, ...

       -diagramlist
	   This option is comparable to the  figures  option.  In
	   this  case the subject is variation diagrams in the Go
	   game. By default sgf2misc will print all the variation
	   diagrams contained in the submitted game.
	   With  the  diagrams	option	you  can overrule this by
	   specifying a list of diagrams to be printed.  The list
	   argument takes two forms:

	     n1,n2,...,nk

	     0

	   The	first  form will create the diagrams n1, n2, ...,
	   nk.
	   The latter option will create no diagrams at all.

       -outputfile
	   If sgf2misc needs to create a file as output  it  will
	   by  default	write  to  standard output (normally your
	   screen).  Of course, on  UNIX  like	systems  you  can
	   redirect  this output using the shell redirection fea
	   ture: "> filename".	However, sometimes it is conveni
	   ant	to be able to specify an implicit filename, espe
	   cially in automatic scripts.

       -coordinates
	   Turn off the printing of  coordinates  around  the  Go
	   diagrams.
	   By  default	sgf2misc  will	create EPSF diagrams with
	   coordinates around them.  Although typographically not
	   nice  to  see, it is essential in most of the cases to
	   understand the text comments associated with the  dia
	   grams.
	   This option is only useful in pstex mode.

       -text_output
	   Suppress all text comments in the output.
	   Since  MARK and LETTER indicators in diagrams are only
	   useful when the explaining text is printed these  will
	   also be automatically suppressed.

       -labels
	   Suppress  the generation of move labels in the output.
	   Move labels are generated when there are multiple com
	   ment  blocks referring on the same diagram. The labels
	   will refer to the move number:

	     Black 37: An excellent move!

       -caching
	   Suppress caching of	Go  diagrams  while  printing  in
	   plain  PostScript  mode.   The effect of caching these
	   position states is that  the  output  is  dramatically
	   reduced. However, the printing speed will be decreased
	   as well. Since the best  performance  depends  on  the
	   specific  circumstances which you work in (previewing,
	   printing on a low speed connected  printer,	transfer
	   ring  a  game)  you	are  enabled to do this trade-off
	   decision yourself.
	   This option is without effect for the moment.  Caching
	   is always off.

       -verbose
	   Suppress  all  kind	of messages generated by sgf2misc
	   during operation. Since this release  of  is  kind  of
	   beta,  I decided to emit all messages despite the ver
	   bose flag.

       -flatten
	   This option is only meaningful when supplying  an  SGF
	   file as input.
	   There are two ways to organise game and variation data
	   in an SGF file:

		(			   (
		game... 		   game...
		  (			     (
		  game (continued...)	     variation...
		  )			     )
		  (			   game (continued...)
		  variation...		   )
		  )
		)

		round			    flattened

	   The idea of the first structure is that while  replay
	   ing	the  game  (e.g. with mgt), you will visit varia
	   tions only when making some	effort.  By  default  you
	   will  walk  through the game.  In the latter structure
	   you will visit  games  and  variations  as  the  occur
	   (exactly what you want when printing the game).
	   The flatten option will explain sgf2misc that the game
	   is organised according to the first structure and  the
	   program  will  then	convert  the  data  to the second
	   structure before printing it.

       -kibitz
	   This option assumes as input an  SGF  game  record  as
	   produced by IGS (the Internet Go Server).
	   The	text  will  be	parsed on username, rank and com
	   ments. When consequtive comments  are  from	the  same
	   person  then  these data will be collected and emitted
	   as one unit.
	   In LaTeX mode the usernames will  be  printed  in  the
	   margin leaving more space for the text itself and hav
	   ing a pleasant overall effect on the output.
	   The	kibitz	option	also  affects  the  way  sgf2misc
	   decides  when  to  print  a	diagram  of  the game. By
	   default it will do so for each  move  for  which  some
	   kibitz  is  in  the	record.  Since this might produce
	   quite a lot of diagrams you	might  want  to  overrule
	   this using the figures option.

       -statistics
	   This  option  assumes  as  input an SGF game record as
	   produced by IGS (the Internet Go Server).
	   The statistics option  will	emit  statistics  on  the
	   amount, and frequency of the kibitz and produce sorted
	   lists of the users by name and rank.

       -zhuge
	   In zhuge mode sgf2misc will automatically replace  mgt
	   replay  directives  by  empty strings or text which is
	   meaningful when printing the game (instead of  replay
	   ing	it  on	the screen).  The lines which are deleted
	   contain the text strings:

		"... ...")
		"[*** end of ..."
		"'[' to go back"
		"'[' to get back"
		"...' to continue.]"

	   The following text strings are replaced by  (see  dia_
	   gram):

		 "[*** '?' to see ..."(c = ['B','C',...])
		 "[*** '?' and '?'..."(c = ['B','C',...])

	   For people who wonder why it is called zhuge mode: Jim
	   Z. Yu is the person who  enjoys  the  readers  of  the
	   newsgroup  rec.games.go  with  instructive and indepth
	   analysis of Go Seigen games.  These magnificant pieces
	   of work and Jim's encouragement were an important rea
	   son why this program ever was  written  in  the  first
	   place.

       -short_keys
	   The	SGF  output format supports both a long and short
	   notation:

		      Black[fk]        B[fk]
		      White[dn]        W[dn]
		      Comment[	       C[
		      text...	       text...
		      ] 	       ]

		      long	       short

	   The short_keys  option  chooses  the  second  notation
	   while  by  default sgf2misc will select the long nota
	   tion.

       -version
	   Print the current version of sgf2misc and quit. Please
	   issue  this	command  when  sending in bug reports and
	   tell the author which version you run.

       -help
	   Print online help on the  usage  of	the  program  and
	   quit.


ENVIRONMENT
       GOMACROS   Directory for Go specific TeX macros
       GOPROLOG   Directory for Go specific PostScript prolog
       EPSFTMP	  Directory for creation of epsf files
       EPSFSIZE   Default size for Go diagrams


FILES
       @GOMACROS@/sgf.sty
		   Go specific TeX macros
       @GOMACROS@/epsf.sty
		   PostScript import macro file
		   (Sourced by @GOMACROS@/sgf.sty)
       @GOMACROS@/secstyle.sty
		   Section style file
		   (Sourced by @GOMACROS@/sgf.sty)
       @GOMACROS@/hyphenlist.tex
		   Go jargon hyphenation list
		   (Sourced by @GOMACROS@/sgf.sty)
       @GOPROLOG@/gopro.eps
		   Go specific PostScript prolog


EXAMPLES
       (Being worked on)




INTERNALS
AUTHOR
       Jan van der Steen, CWI, Amsterdam (jansteen@cwi.nl).


ACKNOWLEDGEMENT
       I want to express my gratitude to the following people for
       helping sgf2misc to become reality:
       First of all, I would like to thank Jim Yu for  his  never
       ending  encouragements, enthusiasm and beta-testing of the
       program. Being able to print his fascinating  articles  on
       Go  Seigen  (posted to rec.games.go in 1992/1993) were the
       main reason to write this program in the first place.
       Next, I would like to thank Jim Mattson	for  various  bug
       reports	and extensions to the program. Amongst others, he
       added the handicap and komi header info fields, pass  han
       dling, and fixed a bug in the PostScript prolog.


SEE ALSO
       mgt(1), xmgt(1)

       Furthermore,  details  on  Go game formats can be found in
       the documents smart-go.def, SF.doc and  The  Liberty  User
       Manual.
       The   first   two   files  are  available  from	ftp  site
       ftp.u.washington.edu.




						      SGF2MISC(L)
