/* #[info:			*/
/************************************************************************
*									*
*			      /\		    |\			*
*			_    ||		       _     \\			*
*		 _-_,  / \\ =||= ,._-_	_-_   < \,  / \\		*
*		||_.  || ||  ||	  ||   || \\  /-|| || ||		*
*		 ~ || || ||  ||	  ||   ||/   (( || || ||		*
*		,-_-  \\_-|  \\,  \\,  \\,/   \/\\  \\/			*
*		       /  \						*
*		      '----`						*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : sgfread.c							*
* Purpose : Read a Smart Go Format Go game				*
* Version : 1.27							*
* Modified: 03 May 1995 10:40:07					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:			*/ 
/* #[include:			*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tools.h"
#include "gogame.h"
#include "sgfread.h"

#ifdef __STRICT_ANSI__
char * strdup           (const char *);
int    strncasecmp      (const char *s1, const char *s2, size_t n);
int    isascii          (int c);
#endif

/* #]include:			*/ 
/* #[typedef:			*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

typedef struct key_info {
    char *		ky_key;		/* Key identifier	*/
    /*
     * Key handler (function handling the request)
     */
    void		(*ky_handle)PROTO((GOGAME *g, char *line));
} KEY;

#undef PROTO

/* #]typedef:			*/ 
/* #[extern:			*/

extern	char *	program;	/* program name				*/
extern	int	kibitz;		/* Interpret comments as IGS kibitz	*/
extern	int	zhuge;		/* We are reading a Zhuge game analysis	*/

/* #]extern:			*/ 
/* #[prototype:			*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

/*
 * Keyword lookup
 */
static void	sgf_handle	PROTO((GOGAME *g, char *key, char *arg));

/*
 * Keyword handlers
 */
static void	do_ab		PROTO((GOGAME *g, char *line));
static void	do_ae		PROTO((GOGAME *g, char *line));
static void	do_aw		PROTO((GOGAME *g, char *line));
static void	do_b		PROTO((GOGAME *g, char *line));
static void	do_bl		PROTO((GOGAME *g, char *line));
static void	do_br		PROTO((GOGAME *g, char *line));
static void	do_c		PROTO((GOGAME *g, char *line));
static void	do_dt		PROTO((GOGAME *g, char *line));
static void	do_ev		PROTO((GOGAME *g, char *line));
static void	do_ff		PROTO((GOGAME *g, char *line));
static void	do_gc		PROTO((GOGAME *g, char *line));
static void	do_gm		PROTO((GOGAME *g, char *line));
static void	do_gn		PROTO((GOGAME *g, char *line));
static void	do_ha		PROTO((GOGAME *g, char *line));
static void	do_id		PROTO((GOGAME *g, char *line));
static void	do_kb		PROTO((GOGAME *g, char *line));
static void	do_km		PROTO((GOGAME *g, char *line));
static void	do_l		PROTO((GOGAME *g, char *line));
static void	do_m		PROTO((GOGAME *g, char *line));
static void	do_n		PROTO((GOGAME *g, char *line));
static void	do_pb		PROTO((GOGAME *g, char *line));
static void	do_pc		PROTO((GOGAME *g, char *line));
static void	do_pw		PROTO((GOGAME *g, char *line));
static void	do_re		PROTO((GOGAME *g, char *line));
static void	do_rg		PROTO((GOGAME *g, char *line));
static void	do_ro		PROTO((GOGAME *g, char *line));
static void	do_so		PROTO((GOGAME *g, char *line));
static void	do_sz		PROTO((GOGAME *g, char *line));
static void	do_tb		PROTO((GOGAME *g, char *line));
static void	do_tm		PROTO((GOGAME *g, char *line));
static void	do_tw		PROTO((GOGAME *g, char *line));
static void	do_us		PROTO((GOGAME *g, char *line));
static void	do_vw		PROTO((GOGAME *g, char *line));
static void	do_w		PROTO((GOGAME *g, char *line));
static void	do_wl		PROTO((GOGAME *g, char *line));
static void	do_wr		PROTO((GOGAME *g, char *line));

#undef PROTO

/* #]prototype:			*/ 
/* #[static:			*/

static KEY skey[] = {
    {	"AB"		, do_ab		},
    {	"AE"		, do_ae		},
    {	"AW"		, do_aw		},
    {	"B" 		, do_b		},
    {	"BL"		, do_bl		},
    {	"BR"		, do_br		},
    {	"C" 		, do_c		},
    {	"DT"		, do_dt		},
    {	"EV"		, do_ev		},
    {	"FF"		, do_ff		},
    {	"GC"		, do_gc		},
    {	"GM"		, do_gm		},
    {	"GN"		, do_gn		},
    {	"HA"		, do_ha		},
    {	"ID"		, do_id		},
    {	"KB"		, do_kb		},
    {	"KM"		, do_km		},
    {	"L"		, do_l		},
    {	"M"		, do_m		},
    {	"N"		, do_n		},
    {	"NB"		, NULL		},
    {	"NW"		, NULL		},
    {	"PB"		, do_pb		},
    {	"PC"		, do_pc		},
    {	"PW"		, do_pw		},
    {	"RE"		, do_re		},
    {	"RG"		, do_rg		},
    {	"RO"		, do_ro		},
    {	"SO"		, do_so		},
    {	"SZ"		, do_sz		},
    {	"TB"		, do_tb		},
    {	"TM"		, do_tm		},
    {	"TW"		, do_tw		},
    {	"US"		, do_us		},
    {	"VW"		, do_vw		},
    {	"W" 		, do_w		},
    {	"WL"		, do_wl		},
    {	"WR"		, do_wr		},
};

/* #]static:			*/ 

/* #[do_b:			*/

static void
do_b(g, line)
GOGAME *g;
char   *line;
{
    int		i	= *(line  ) - 'a';
    int		j	= *(line+1) - 'a';
    PLACE	place	= i + (j * g->gm_sz);

    if (*line == 0 || (i == 't' - 'a' && j == 't' - 'a'))
        pass_store(g, AUTO_NR, BLACK);
    else
        move_store(g, AUTO_NR, BLACK, place);
}

/* #]do_b:			*/ 
/* #[do_w:			*/

static void
do_w(g, line)
GOGAME *g;
char   *line;
{
    int		i	= *(line  ) - 'a';
    int		j	= *(line+1) - 'a';
    PLACE	place	= i + (j * g->gm_sz);

    if (*line == 0 || (i == 't' - 'a' && j == 't' - 'a'))
        pass_store(g, AUTO_NR, WHITE);
    else
        move_store(g, AUTO_NR, WHITE, place);
}

/* #]do_w:			*/ 
/* #[do_ev:			*/

static void
do_ev(g, line)
GOGAME *g;
char   *line;
{
    g->gm_ev = strdup(line);
}

/* #]do_ev:			*/ 
/* #[do_id:			*/

static void
do_id(g, line)
GOGAME *g;
char   *line;
{
    g->gm_id = strdup(line);
}

/* #]do_id:			*/ 
/* #[do_us:			*/

static void
do_us(g, line)
GOGAME *g;
char   *line;
{
    g->gm_us = strdup(line);
}

/* #]do_us:			*/ 
/* #[do_pb:			*/

static void
do_pb(g, line)
GOGAME *g;
char   *line;
{
    g->gm_pb = strdup(line);
}

/* #]do_pb:			*/ 
/* #[do_br:			*/

static void
do_br(g, line)
GOGAME *g;
char   *line;
{
    g->gm_br = strdup(line);
}

/* #]do_br:			*/ 
/* #[do_tb:			*/

static void
do_tb(g, line)
GOGAME *g;
char   *line;
{
    int	  i	= *(line  ) - 'a';
    int	  j	= *(line+1) - 'a';
    PLACE place	= i + (j * g->gm_sz);
    MARK  m;

    sprintf(m, "%c", TERR_B /* SHADE_30 */);
    mark_store(g, place, m);
}

/* #]do_tb:			*/ 
/* #[do_pw:			*/

static void
do_pw(g, line)
GOGAME *g;
char   *line;
{
    g->gm_pw = strdup(line);
}

/* #]do_pw:			*/ 
/* #[do_wr:			*/

static void
do_wr(g, line)
GOGAME *g;
char   *line;
{
    g->gm_wr = strdup(line);
}

/* #]do_wr:			*/ 
/* #[do_tw:			*/

static void
do_tw(g, line)
GOGAME *g;
char   *line;
{
    int	  i	= *(line  ) - 'a';
    int	  j	= *(line+1) - 'a';
    PLACE place	= i + (j * g->gm_sz);
    MARK  m;

    sprintf(m, "%c", TERR_W /* SHADE_70 */);
    mark_store(g, place, m);
}

/* #]do_tw:			*/ 
/* #[do_so:			*/

static void
do_so(g, line)
GOGAME *g;
char   *line;
{
    g->gm_so = strdup(line);
}

/* #]do_so:			*/ 
/* #[do_gc:			*/

static void
do_gc(g, line)
GOGAME *g;
char   *line;
{
    g->gm_gc = strdup(line);
}

/* #]do_gc:			*/ 
/* #[do_tm:			*/

static void
do_tm(g, line)
GOGAME *g;
char   *line;
{
    g->gm_tm = strdup(line);
}

/* #]do_tm:			*/ 
/* #[do_dt:			*/

static void
do_dt(g, line)
GOGAME *g;
char   *line;
{
    g->gm_dt = strdup(line);
}

/* #]do_dt:			*/ 
/* #[do_gm:			*/

static void
do_gm(g, line)
GOGAME *g;
char   *line;
{
    g->gm_gamenr = strdup(line);
}

/* #]do_gm:			*/ 
/* #[do_ro:			*/

static void
do_ro(g, line)
GOGAME *g;
char   *line;
{
    rnd_store(g, atoi(line));
}

/* #]do_ro:			*/ 
/* #[do_rg:			*/

static void
do_rg(g, line)
GOGAME *g;
char   *line;
{
    if (*line) {
	int i = *(line  ) - 'a';
	int j = *(line+1) - 'a';

	reg_store(g, i, j);
    }
}

/* #]do_rg:			*/ 
/* #[do_vw:			*/

static void
do_vw(g, line)
GOGAME *g;
char   *line;
{
    if (*line) {
	int i = *(line  ) - 'a';
	int j = *(line+1) - 'a';

	view_store(g, i, j);
    }
}

/* #]do_vw:			*/ 
/* #[do_sz:			*/

static void
do_sz(g, line)
GOGAME *g;
char   *line;
{
    size_store(g, atoi(line));
}

/* #]do_sz:			*/ 
/* #[do_gn:			*/

static void
do_gn(g, line)
GOGAME *g;
char   *line;
{
    g->gm_gn = strdup(line);
}

/* #]do_gn:			*/ 
/* #[do_n:			*/

static void
do_n(g, line)
GOGAME *g;
char   *line;
{
    g->gm_n = strdup(line);
}

/* #]do_n:			*/ 
/* #[do_pc:			*/

static void
do_pc(g, line)
GOGAME *g;
char   *line;
{
    g->gm_pc = strdup(line);
}

/* #]do_pc:			*/ 
/* #[do_re:			*/

static void
do_re(g, line)
GOGAME *g;
char   *line;
{
    g->gm_re = strdup(line);
}

/* #]do_re:			*/ 
/* #[do_ff:			*/

static void
do_ff(g, line)
GOGAME *g;
char   *line;
{
    g->gm_ff = strdup(line);
}

/* #]do_ff:			*/ 
/* #[do_ha:			*/

static void
do_ha(g, line)
GOGAME *g;
char   *line;
{
    g->gm_ha = strdup(line);
}

/* #]do_ha:			*/ 
/* #[do_km:			*/

static void
do_km(g, line)
GOGAME *g;
char   *line;
{
    char *p = line + strlen(line) - 1;

    /* Trim excess precision */
    while(*p == '0' && p > line)
      *p-- = '\0';

    g->gm_km = strdup(line);
}

/* #]do_km:			*/ 

/* #[do_ab:			*/

static void
do_ab(g, line)
/*
 * Add a black stone to the game
 */
GOGAME *g;
char   *line;
{
    int	  i	= *(line  ) - 'a';
    int	  j	= *(line+1) - 'a';
    PLACE place	= i + (j * g->gm_sz);

    addstone_store(g, BLACK, place);
}

/* #]do_ab:			*/ 
/* #[do_aw:			*/

static void
do_aw(g, line)
/*
 * Add a white stone to the game
 */
GOGAME *g;
char   *line;
{
    int	  i	= *(line  ) - 'a';
    int	  j	= *(line+1) - 'a';
    PLACE place	= i + (j * g->gm_sz);

    addstone_store(g, WHITE, place);
}

/* #]do_aw:			*/ 
/* #[do_l:			*/

static void
do_l(g, line)
/*
 * Add a letter indicator to the game
 */
GOGAME *g;
char   *line;
{
    int	  i	= *(line  ) - 'a';
    int	  j	= *(line+1) - 'a';
    PLACE place	= i + (j * g->gm_sz);

    letter_store(g, place, AUTO_LETTER);
}

/* #]do_l:			*/ 
/* #[do_m:			*/

static void
do_m(g, line)
/*
 * Add a mark indicator to the game
 */
GOGAME *g;
char   *line;
{
    int	  i	= *(line  ) - 'a';
    int	  j	= *(line+1) - 'a';
    PLACE place	= i + (j * g->gm_sz);

    mark_store(g, place, AUTO_MARK);
}

/* #]do_m:			*/ 
/* #[do_ae:			*/

static void
do_ae(g, line)
/*
 * Delete a white/black stone
 */
GOGAME *g;
char   *line;
{
    int	  i	= *(line  ) - 'a';
    int	  j	= *(line+1) - 'a';
    PLACE place	= i + (j * g->gm_sz);

    delstone_store(g, place);
}

/* #]do_ae:			*/ 
/* #[do_kb:			*/

static void
do_kb(g, line)
GOGAME *g;
char   *line;
{
    if (*line) kibitz_store(g, line);
}

/* #]do_kb:			*/ 
/* #[do_c:			*/

static void
do_c(g, line)
GOGAME *g;
char   *line;
{
    if (zhuge) {
	/*
	 * [*** end of s ...	(s = ["reference","variation",...])
	 */
	if (strstr(line,         "... ...")	||
	    strstr(line,     "[*** end of")	||
	    strstr(line,  "'[' to go back")	||
	    strstr(line, "'[' to get back")	||
	    strstr(line, "' to continue.]")) return;

	/*
	 * [*** 'c' to see ...	(c = ['B','C',...])
	 * [*** 'c' and ...	(c = ['B','C',...])
	 */
	if (strstr(line, "[*** '")) {
	    text_store(g, "(See diagram)\n");
	    return;
	}
    }
    text_store(g, line);
}

/* #]do_c:			*/ 
/* #[do_bl:			*/

static void
do_bl(g, line)
GOGAME *g;
char   *line;
{
    time_store(g, BLACK, line);
}

/* #]do_bl:			*/ 
/* #[do_wl:			*/

static void
do_wl(g, line)
GOGAME *g;
char   *line;
{
    time_store(g, WHITE, line);
}

/* #]do_wl:			*/ 
 
/* #[cmp_key:			*/

static int
cmp_key(d1, d2)
const void *d1;
const void *d2;
{
    KEY *k1 = (KEY *) d1;
    KEY *k2 = (KEY *) d2;

    return strcmp(k1->ky_key, k2->ky_key);
}

/* #]cmp_key:			*/ 
/* #[sgf_handle:		*/

static void
sgf_handle(g, keyword, argument)
GOGAME *g;
char   *keyword;
char   *argument;
{
    KEY	*h;
    KEY	 k;

    k.ky_key = keyword;
#if 0
    if (*(keyword+2))
	h = bsearch(&k, lkey, sizeof(lkey)/sizeof(KEY), sizeof(KEY), cmp_key);
    else
#endif
    h = bsearch(&k, skey, sizeof(skey)/sizeof(KEY), sizeof(KEY), cmp_key);

    if (h) {
	if (h->ky_handle)
	    (h->ky_handle)(g, argument);
	else
	    message("Info ignored: \"%s[%s]\"\n", keyword, argument);
    } else {
	message("Not supported: \"%s[%s]\"\n", keyword, argument);
    }
}

/* #]sgf_handle:		*/ 
/* #[sgf_read:			*/

/* #[defines:		*/

#define MAXLINE		1024
#define MAXKEY		32
#define MAXARG		MAXLINE
#define LINELENGTH	64		/* Limit linelength in comments	*/

#define IN_NIL		0		/* Are we parsing garbage	*/
#define	IN_KEY		1		/* Are we parsing a keyword	*/
#define	IN_ARG		2		/* Are we parsing an argument	*/

#define	ESCAPED		(c == '\\')
#define	GAME_B		'('
#define	GAME_E		')'
#define	LIST_B		'['
#define	LIST_E		']'
#define	NODE_B		';'
#define	SPACE		' '
#define	TAB		'\t'
#define	FF		'\f'
#define	NEWLINE		'\n'
#define	RETURN		'\r'


/* #]defines:		*/ 

GOGAME *
sgf_read(filename)
/* #[comments:		*/
/*
 * Read a Go game from filename (or stdin). We parse in two steps:
 *
 *	1. In this function we keep track of the state
 *
 *		State	Description		Example
 *		---------------------------------------------------
 *		IN_NIL	game didn't start yet	(header  garbage)
 *			or has ended		(trailer garbage)
 *		IN_KEY	parsing keyword		(Black, White, ...)
 *		IN_ARG	parsing an argument	([..][..]...)
 *
 *	2. In sgf_handle we handle the keyword events
 *
 * At the moment we collect text blocks by splitting them up in chunks
 * of LINELENGTH bytes. They are stored in linked lists.
 */
/* #]comments:		*/ 
char *filename;
{
    FILE   *	fp = stdin;
    GOGAME *	g  = (GOGAME *) 0;	/* Current Go game		*/
    char	keyword [MAXKEY+1];	/* Current keyword  (+nullbyte)	*/
    char	argument[MAXARG+1];	/* Current argument (+nullbyte)	*/
    int		c;			/* Currently read input char	*/
    int		mode   = IN_NIL;	/* IN_NIL, IN_KEY, IN_ARG	*/
    int		linenr = 0;		/* Currently processed line	*/
    int		k_len  = 0;		/* length of keyword		*/
    int		a_len  = 0;		/* length of argument		*/
    int		cr     = 0;		/* Characters read		*/
    int		in_par = 0;		/* Are we parsing a paragraph	*/
    int		overflow = 0;		/* overflow detected		*/

    if (filename && (fp = fopen(filename, "r")) == (FILE *) 0) {
	fprintf(stderr, "%s: Can't open \"%s\"\n", program, filename);
	exit(1);
    }
    memset(keyword , 0, MAXKEY+1);
    memset(argument, 0, MAXARG+1);

next_char:
    while ((c = fgetc(fp)) != EOF) {
	linenr++;
	switch (mode) {
/* #[IN_NIL:		*/

	case IN_NIL:
	    if (c == GAME_B) {
		g    = game_alloc((GOGAME *) 0);
		mode = IN_KEY;
	    }
	    goto next_char;

/* #]IN_NIL:		*/ 
/* #[IN_KEY:		*/

	case IN_KEY:
	    switch (c) {
/*
		 #[GAME_B:
*/

		case GAME_B:
		    g = push_var(g);
		    goto next_char;

/*
		 #]GAME_B: 
		 #[GAME_E:
*/

		case GAME_E:
		    {
			GOGAME *parent = pop_var(g);

			if (parent == (GOGAME *) 0) {
			    /*
			     * We could return to IN_NIL state when we'd
			     * accept multiple games in one file.
			     * The main loop would then be something like:
			     *
			     *    while (g = sgfread(fp)) game_print(g);
			     *
			     * In this case we shouldn't close the file
			     * of course before returning below...
			     */
			    goto we_re_done;
			}
			g = parent;
		    }
		    goto next_char;

/*
		 #]GAME_E: 
		 #[LIST_B:
*/

		case LIST_B:
		    if (!strncmp("C"      , keyword, strlen(keyword)) ||
			!strncmp("Comment", keyword, strlen(keyword))) {
			/*
			 * We are in a text block
			 */
			in_par = 1;
			if (kibitz) {
			    strcpy(keyword, "KB");
			    cr = MAXARG;
			} else
			    cr = LINELENGTH;
		    }
		    mode = IN_ARG;

		    if (in_par) {
			/*
			 * The following husstle is necessary to ignore
			 * the first newline following "Comment["/"C["
			 */
			c = fgetc(fp);
			if (c == RETURN) c = fgetc(fp); /* ignore '\r' */
			if (c == EOF    ) goto error_parse;
			if (c != NEWLINE) ungetc(c, fp);
		    }
		    goto next_char;

/*
		 #]LIST_B: 
		 #[LIST_E:
*/

		case LIST_E:
		    message("List ended in keyword!\n");
		    goto error_parse;

/*
		 #]LIST_E: 
		 #[NODE_B:
*/

		case NODE_B:
		    node_store(g);
		    goto next_char;

/*
		 #]NODE_B: 
		 #[Whitespace:
*/

		case SPACE   :
		case TAB     :
		case FF	     :
		case RETURN  :
		case NEWLINE :
		    /*
		     * For the moment we ignore whitespace.
		     * Actually, we should eat it away till
		     * we know in which state we end up.
		     * Now it could happen that we concatenate
		     * something (which is wrong).
		     */
		    goto next_char;

/*
		 #]Whitespace: 
		 #[default:
*/

		default	     :
		    if (k_len == MAXKEY) {
			if (!overflow) {
			    message("Out of keyword space! (\"%s\" >= %d)\n",
				    keyword, MAXKEY);
			    message("Further characters skipped\n");
			    overflow = 1;
			}
			goto next_char;
		    }
		    if ('A' <= c && c <= 'Z') {
			keyword[k_len] = c; k_len++;
		    } else if ('a' <= c && c <= 'z') {
			;	/* Ignore character */
		    } else
			fprintf(stderr,
				"Suspicuous character encountered in key: \"%c\" (ignored)\n", c);
		    goto next_char;
	    }

/*
		 #]default: 
*/
/* #]IN_KEY:		*/ 
/* #[IN_ARG:		*/

	case IN_ARG:
	    if (!keyword[0]) {
		message("List without keyword encountered!\n");
		goto error_parse;
	    }
	    switch (c) {
/*
		 #[LIST_E:
*/
		case LIST_E:
		    if (in_par	    &&
			*(argument) &&
			*(argument + strlen(argument)-1) != NEWLINE)
			*(argument + strlen(argument)) = NEWLINE;
		    sgf_handle(g, keyword, argument);
		    memset(argument, 0, MAXARG); a_len = 0;
		    overflow = 0;
		    while ((c = fgetc(fp)) != EOF) {
			if (c == SPACE  ||
			    c == FF     ||
			    c == TAB    ||
			    c == RETURN ||
			    c == NEWLINE ) continue;
			if (c == LIST_B) {
			    /*
			     * Continuation of current list
			     */
			    mode = IN_ARG;
			    goto next_char;
			}
			break;
		    }
		    if (c != EOF) {
			memset(keyword, 0, MAXKEY); k_len = 0; in_par = 0;
			overflow = 0;
			mode = IN_KEY;
		    }
		    ungetc(c, fp);
		    goto next_char;
/*
		 #]LIST_E: 
		 #[default:
*/
		default:
		    /*
		     * First of all check for available space
		     * to store the incomming character...
		     */
		    if (a_len == MAXARG) {
			if (!overflow) {
			    message("Out of string space! (\"%s\" >= %d)\n",
				    argument, MAXARG);
			    message("Further characters skipped\n");
			    overflow = 1;
			}
			goto next_char;
		    }

		    if (ESCAPED) {
			c = fgetc(fp);
			if (c == RETURN ) c = fgetc(fp);    /* Ignore \r */
			if (c == EOF    ) goto error_parse;
			if (c == NEWLINE) c = SPACE;        /* Escaped \n */
		    }

		    if (c == RETURN) goto next_char;        /* Ignore \r */

		    /*
		     * Don't let the comments overflow the input buffer
		     */
		    if (in_par && (c == NEWLINE || (--cr <= 0 && c == SPACE))) {
			argument[a_len] = c;
			if (!kibitz && c == SPACE) {
			    /*
			     * Skip multiple whitespace
			     */
			    while (c == SPACE) c = fgetc(fp);
			    if (c == EOF  ) goto error_parse;
			    ungetc(c, fp);
			}

			/*
			 * Execute the collected command
			 */
			sgf_handle(g, keyword, argument);
			memset(argument, 0, MAXARG); a_len = 0;
			overflow = 0;
			cr = (!strcmp(keyword, "KB")) ? MAXARG : LINELENGTH;
			goto next_char;
		    }

		    /*
		     * We don't store leading spaces for kibitz
		     */
		    if (kibitz && c == SPACE && !a_len) goto next_char;

		    /*
		     * Store the character
		     */
		    argument[a_len] = c; a_len++;
		    goto next_char;
/*
		 #]default: 
*/
	    }
/* #]IN_ARG:		*/ 
	}
    }
/* #[Exiting:		*/

    if (g != (GOGAME *) 0) {
        GOGAME *parent = NULL;

	message("sgf_read(): EOF encountered before End Of Game\n");
#if 0
	game_free(g);
	g = (GOGAME *) 0;
#else
        while ((parent = g->gm_parent) != NULL) g = pop_var(g);
#endif
    }
    goto we_re_done;

error_parse:
    g = (GOGAME *) 0;
    message("%d: %s[%s]\n", linenr, keyword, argument);

we_re_done:
    if (fp != stdin) fclose(fp);
    return g;

/* #]Exiting:		*/ 
}

/* #]sgf_read:			*/ 

