/* #[info:			*/
/************************************************************************
*									*
*		       ,	  ,		,			*
*		      ||    _	 ||   '	       ||			*
*		_-_, =||=  < \, =||= \\	 _-_, =||=  _-_,		*
*	       ||_.   ||   /-||	 ||  || ||_.   ||  ||_.			*
*		~ ||  ||  (( ||	 ||  ||	 ~ ||  ||   ~ ||		*
*	       ,-_-   \\,  \/\\	 \\, \\ ,-_-   \\, ,-_-			*
*									*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : statists.c							*
* Purpose : Gather statistics from IGS kibitz				*
* Version : 1.8								*
* Modified: 7/24/93 17:08:51						*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:			*/ 
/* #[include:			*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "tools.h"

/* #]include:			*/ 
/* #[typedef:			*/

typedef struct kib_info {
    char *	kib_name;
    long	kib_rank;
    int		kib_n;
    long	kib_tot;
    int		kib_avrg;
} KIBITZER;

/* #]typedef:			*/ 
/* #[extern:			*/

extern	int	 statout;	/* Emit the kibitz stats		*/

/* #]extern:			*/ 
/* #[prototype:			*/

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

static int	namesort	PROTO((KIBITZER *a, KIBITZER *b));
static int	ranksort	PROTO((KIBITZER *a, KIBITZER *b));
static int	freqsort	PROTO((KIBITZER *a, KIBITZER *b));
static int	bytesort	PROTO((KIBITZER *a, KIBITZER *b));

#undef PROTO

/* #]prototype:			*/ 
/* #[static:			*/

static	int		(*kibsort)()		= namesort;
static	KIBITZER	kibitzers[MAXKIBITZER]	= { {0} };

/* #]static:			*/ 

/* #[kib_list:			*/

void
kib_list(mode, label)
/*
 * List the kibitz info from the database
 */
int   mode;
char *label;
{
    KIBITZER *k;
    int	      i;

    switch (mode) {
	case 0:	kibsort = namesort; break;
	case 1: kibsort = ranksort; break;
	case 2: kibsort = freqsort; break;
	case 3: kibsort = bytesort; break;
    }
    for (i = 0; i < MAXKIBITZER; i++) {
	k = kibitzers+i;
	if (k->kib_name) k->kib_avrg = (int)((k->kib_tot *100)/k->kib_n);
    }
    qsort(kibitzers, MAXKIBITZER, sizeof(KIBITZER), kibsort);

    for (i = 0; i < MAXKIBITZER; i++) {
	k = kibitzers+i;
	if (k->kib_name)
	    fprintf(stdout, "%s{%3d}{%-12s}{%-3s}{%3d}{%6ld}{%3d.%02d}\n",
		    label,
		    i+1,
		    k->kib_name,
		    rank2str(k->kib_rank),
		    k->kib_n,
		    k->kib_tot,
		    k->kib_avrg/100,
		    k->kib_avrg - 100*(k->kib_avrg/100));
    }
}

/* #]kib_list:			*/ 
/* #[kib_add:			*/

void
kib_add(text)
/*
 * Add kibitz info to the database
 */
TEXT *text;
{
    KIBITZER *k;
    TEXT     *t;
    int	      i;

    for (t = text; t != (TEXT *) 0; t = t->cc_next) {
	if (!t->cc_name) continue;
	for (i = 0; i < MAXKIBITZER; i++) {
	    k = kibitzers+i;
	    if (!k->kib_name || !strcmp(k->kib_name, t->cc_name)) {
		/*
		 * Either a free slot or we found the user
		 */
		if (!k->kib_name) {
		    k->kib_name = t->cc_name;
		    k->kib_rank = t->cc_spec;
		}
		k->kib_tot += strlen(t->cc_text);
		k->kib_n++;
		break;
	    }
	}
	if (strcmp(k->kib_name, t->cc_name))
	    message("Internal error: User table full in kib_add()\n");
    }
}

/* #]kib_add:			*/ 

/* #[namesort:			*/

static int
namesort(a, b)
KIBITZER *a;
KIBITZER *b;
{
    if (a->kib_name == (char *) 0) return  1;
    if (b->kib_name == (char *) 0) return -1;

    return strcmp(a->kib_name, b->kib_name);
}

/* #]namesort:			*/ 
/* #[ranksort:			*/

static int
ranksort(a, b)
KIBITZER *a;
KIBITZER *b;
{
    long cmp;
    long r1;
    long r2;

    if (a->kib_name == (char *) 0) return  1;
    if (b->kib_name == (char *) 0) return -1;

    r1 = (a->kib_rank) > 0 ? a->kib_rank : -1 * a->kib_rank;
    r2 = (b->kib_rank) > 0 ? b->kib_rank : -1 * b->kib_rank;
    if ((cmp = r2 - r1) != 0) return (cmp > 0) ? 1 : -1;

    /*
     * We trust rated players slightly better than ranked players
     */
    if (b->kib_rank > 0 && a->kib_rank < 0) return -1;
    if (b->kib_rank < 0 && a->kib_rank > 0) return  1;

    /*
     * If all is equal we sort by name
     */
    return namesort(a, b);
}

/* #]ranksort:			*/ 
/* #[freqsort:			*/

static int
freqsort(a, b)
KIBITZER *a;
KIBITZER *b;
{
    int cmp;

    if (a->kib_name == (char *) 0) return  1;
    if (b->kib_name == (char *) 0) return -1;

    if ((cmp =        b->kib_n   - a->kib_n   ) != 0) return cmp;
    if ((cmp = (int) (b->kib_tot - a->kib_tot)) != 0) return cmp;

    return namesort(a, b);
}

/* #]freqsort:			*/ 
/* #[bytesort:			*/

static int
bytesort(a, b)
KIBITZER *a;
KIBITZER *b;
{
    int cmp;

    if (a->kib_name == (char *) 0) return  1;
    if (b->kib_name == (char *) 0) return -1;

    if ((cmp = (int) (b->kib_tot - a->kib_tot)) != 0) return cmp;
    if ((cmp =        b->kib_n   - a->kib_n   ) != 0) return cmp;

    return namesort(a, b);
}

/* #]bytesort:			*/ 
