/* #[info:		*/
/************************************************************************
*									*
*		     ,							*
*		    ||							*
*	      _-_, =||=	 _-_ \\ \\ ,._-_  _-_,	_-_   _-_,		*
*	     ||_.   ||	||   || ||  ||	 ||_.  || \\ ||_.		*
*	      ~ ||  ||	||   || ||  ||	  ~ || ||/    ~ ||		*
*	     ,-_-   \\, \\,/ \\/\\  \\,	 ,-_-  \\,/  ,-_-		*
*									*
*									*
*									*
*-----------------------------------------------------------------------*
* File	  : stcurses.h							*
* Purpose : Curses for the ATARI-ST (author see info below)		*
* Version : 1.3								*
* Modified: 7/24/93 17:08:53						*
* Fixes	  : Jan van der Steen (jansteen@cwi.nl), Geertjan van Opdorp	*
************************************************************************/
/* #]info:		*/ 

#if !defined(__CURSESH)
#define __CURSESH

/* ************************************************************************ */
/*                                                                          */
/*   CURSES.H                      include file for programs using CURSES   */
/*                                                                          */
/* ************************************************************************ */
/* 									    */
/* This source and resulting object may be modified, used or distributed by */
/* anyone who so desires under the following conditions :                   */
/*									    */
/*	1) This notice and the copyright notice shall not be removed or     */
/*	   changed.							    */
/*	2) No credit shall be taken for this source and resulting objects   */
/*	3) This source or resulting objects is not to be traded, sold or    */
/*	   used for personal gain or profit.				    */
/*	4) Inclusion of this source or resulting objects in commercially    */
/*	   available packages is forbidden without written consent of the   */
/*	   author of this source.					    */
/*                                                                          */
/* ************************************************************************ */

/* #[include:		*/

#include "portab.h"
#include "stdio.h"

/* #]include:		*/
/* #[define:		*/

#define reg		register
#define bool		BYTE
#define TRUE		1
#define FALSE       	0

#define _SUBWIN		01
#define _ENDLINE	02
#define _FULLWIN	04
#define	_SCROLLWIN	010
#define	_STANDOUT	0x100	/* 0200 */
#define	WINDOW		struct _win_st

#define TOUCHED		0x0200
#define STANDOUT	0x0100

/* #]define:		*/
/* #[prototype:								    */

#ifdef __STDC__
#	define	PROTO(s) s
#else
#	define	PROTO(s) ()
#endif

void		initscr		PROTO((void ));
void		endwin		PROTO((void ));
WINDOW		*newwin		PROTO((int l , int c , int by , int bx ));
WINDOW		*subwin		PROTO((WINDOW *w , int l , int c , int by , int bx ));
void		waddch		PROTO((WINDOW *w , int c ));
void		waddstr		PROTO((WINDOW *w , unsigned char *s ));
void		box		PROTO((WINDOW *w , int v , int h ));
void		wclear		PROTO((WINDOW *w ));
void		wclrtobot	PROTO((WINDOW *w ));
void		wclrtoeol	PROTO((WINDOW *w ));
void		wdelch		PROTO((WINDOW *w ));
void		wdeleteln	PROTO((WINDOW *w ));
void		werase		PROTO((WINDOW *w ));
void		winsch		PROTO((WINDOW *w , int c ));
void		winsertln	PROTO((WINDOW *w ));
void		wmove		PROTO((WINDOW *w , int y , int x ));
void		woverlay	PROTO((WINDOW *v , WINDOW *w ));
void		overwrite	PROTO((WINDOW *v , WINDOW *w ));
void		wstandout	PROTO((WINDOW *w ));
void		wstandend	PROTO((WINDOW *w ));
LONG		wgetch		PROTO((WINDOW *w ));
void		wgetstr		PROTO((WINDOW *w , char *s ));
void		wrefresh	PROTO((WINDOW *w ));
void		delwin		PROTO((WINDOW *w ));
void		mvwin		PROTO((WINDOW *w , int y , int x ));
void		touchwin	PROTO((WINDOW *w ));
void		leaveok		PROTO((WINDOW *w , bool f ));
void		scrollok	PROTO((WINDOW *w , bool f ));
void		nl		PROTO((void ));
void		nonl		PROTO((void ));
void		longname	PROTO((char *termbuf , char *name ));
void		raw		PROTO((void ));
void		noraw		PROTO((void ));
void		__sshft		PROTO((void ));
void		__ushft		PROTO((void ));
void		crmode		PROTO((void ));
void		nocrmode	PROTO((void ));
void		echo		PROTO((void ));
void		noecho		PROTO((void ));
void		mvcur		PROTO((int ly , int lx , int ny , int nx ));
void		_movcur		PROTO((WORD y , WORD x ));
void		scroll		PROTO((WINDOW *w ));
void		wprintw		PROTO((WINDOW *w, char *format , ...));
void		printw		PROTO((char *format , ...));
void		scanw		PROTO((char *format , ...));

#undef PROTO

/* #]prototype:								    */

/* #[macros:		*/

#define addch(c)		waddch(stdscr, c)
#define mvaddch(y,x,c)		{ wmove(stdscr,y,x) ; waddch(stdscr,c) ; }
#define mvwaddch(w,y,x,c)	{ wmove(w,y,x) ; waddch(w,c) ; }
#define addstr(s)		waddstr(stdscr,s)
#define mvaddstr(y,x,s)		{ wmove(stdscr,y,x) ; waddstr(stdscr,s) ; }
#define mvwaddstr(w,y,x,s)	{ wmove(w,y,x) ; waddstr(w,s) ; }
#define clear()			wclear(stdscr)
#define clearok(w,f)		{ w->_clear = (w->_flags & _FULLWIN) ? f : w->_clear ; }
#define clrtobot()		wclrtobot(stdscr)
#define clrtoeol()		wclrtoeol(stdscr)
#define delch()			wdelch(stdscr)
#define mvdelch(y,x)		{ wmove(stdscr,y,x) ; wdelch(stdscr) ; }
#define mvwdelch(w,y,x)		{ wmove(w,y,x) ; wdelch(w) ; }
#define deleteln()		wdeleteln(stdscr)
#define mvdeleteln(y,x)		{ wmove(stdscr,y,x) ; wdeleteln(stdscr) ; }
#define mvwdeleteln(w,y,x)	{ wmove(w,y,x) ; wdeleteln(w) ; }
#define erase()			werase(stdscr)
#define insch(c)		winsch(stdscr,c)
#define mvinsch(y,x,c)		{ wmove(stdscr,y,x) ; winsch(stdscr,c) ; }
#define mvwinsch(w,y,x,c)	{ wmove(w,y,x) ; winsch(w,c) ; }
#define insertln()		winsertln(stdscr)
#define mvinsertln(y,x)		{ wmove(stdscr,y,x) ; winsertln(stdscr) ; }
#define mvwinsertln(w,y,x)	{ wmove(w,y,x) ; winsertln(w) ; }
#define move(y,x)		wmove(stdscr,y,x)
#define refresh()		wrefresh(stdscr)
#define standout()		wstandout(stdscr)
#define standend()		wstandend(stdscr)
#define getch()			wgetch(stdscr)
#define mvgetch(y,x)		( wmove(stdscr,y,x) , wgetch(stdscr) )
#define mvwgetch(w,y,x)		( wmove(w,y,x) , wgetch(w) )
#define getstr(s)		wgetstr(stdscr,s)
#define mvgetstr(y,x,s)		{ wmove(stdscr,y,x) ; wgetstr(stdscr,s) ; }
#define mvwgetstr(w,y,x,s)	{ wmove(w,y,x) ; wgetstr(w,s) ; }
#define getyx(w,y,x)		{ y = w->_cury ; x = w->_curx ; }
#define inch()			(stdscr->_y[stdscr->_cury][stdscr->_curx])
#define mvinch(y,x)		( wmove(stdscr,y,x) , stdscr->_y[stdscr->_cury][stdscr->_curx])
#define mvwinch(w,y,x)		( wmove(w,y,x) , w4->_y[w->_cury][w->_curx])
#define winch(w)		(w->_y[w->_cury][w->_curx])

/* #]macros:		*/
/* #[intern:		*/

struct _win_st {
	WORD	_cury, _curx ;
	WORD	_maxy, _maxx ;
	WORD	_begy, _begx ;
	WORD	_flags ;
	BYTE	_clear ;
	BYTE	_leave ;
	BYTE	_scroll ;
	WORD	**_y ;
	WORD	*_firstch ;
	WORD	*_lastch ;
} ;

/* #]intern:		*/
/* #[extern:		*/

extern WINDOW *curscr ;
extern WINDOW *stdscr ;
extern char	*Def_term ;
extern bool	My_term ;
extern char	*ttytype ;
extern int	LINES ;
extern int	COLS ;
extern int	ERR ;
extern int	OK ;

/* #]extern:		*/

#endif
