/* #[info:		*/
/************************************************************************
*									*
*		 ,		  ,		 ,,			*
*		||	  ,	 ||		 ||			*
*	       =||=  _-_  \\ /` =||=  /'\\  /'\\ ||  _-_,		*
*		||  || \\  \\	 ||  || || || || || ||_.		*
*		||  ||/	   /\\	 ||  || || || || ||  ~ ||		*
*		\\, \\,/  /  \;	 \\, \\,/  \\,/	 \\ ,-_-		*
*									*
*									*
*									*
*		  Copyright (c) 1993  Jan van der Steen			*
*	       Centre for Mathematics and Computer Science		*
*		       Amsterdam, the Netherlands			*
*									*
*-----------------------------------------------------------------------*
*   You are not allowed to use this software in commercial software.	*
*   You are allowed to use this software in free software under the	*
*   condition that you maintain this integral message in the source.	*
*-----------------------------------------------------------------------*
* File	  : textools.c							*
* Purpose : Write a Go data structure in PS diagrams and TeX text	*
* Version : 1.21							*
* Modified: 22 Mar 1995 09:43:16					*
* Author  : Jan van der Steen (jansteen@cwi.nl)				*
************************************************************************/
/* #]info:		*/ 
/* #[include:		*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "sgf2misc.h"
#include "gogame.h"
#include "textools.h"
#include "tools.h"

/* #]include:		*/ 
/* #[define:		*/

#ifndef	GOMACROS
#if defined (ATARI)
#define	GOMACROS	"c:\\lib\\sgf2misc\\sgf"
#else
#define	GOMACROS	"/ufs/jansteen/lib/sgf2misc/sgf"
#endif
#endif
#ifndef	GOPROLOG
#if defined (ATARI)
#define	GOPROLOG	"c:\\lib\\sgf2misc\\ps"
#else
#define	GOPROLOG	"/ufs/jansteen/lib/sgf2misc/ps"
#endif
#endif

/* #]define:		*/ 
/* #[extern:		*/

extern	FILE *		fw;		/* Output stream		*/
extern	int		prn_dev;	/* The device number		*/
extern	int		paper_size;	/* The paper size		*/
extern	EPSFINFO	epsf;		/* The eps filename/directory	*/
extern	char *		program;	/* Name of this program		*/
extern	int		statout;	/* Statistics report ?		*/
extern	int		kibitz;		/* Are we processing kibitz ?	*/
extern	int		caching;	/* Save on output ?		*/
extern	int		coords;		/* Diagrams with coordinates ?	*/
extern	int		labels;		/* Labels with move numbers	*/
extern	int		zhuge;		/* Apply the "zhuge" conversion	*/

/* #]extern:		*/ 

/* #[typedef:		*/

typedef struct paper_info {
    int		figure_size;	/* Maximum epsf figure width for twocolumn */
    char	*style_file;	/* Style file to include (with leading `,') */
} PAPER_TYPE;

/* #]typedef:		*/ 
/* #[static:		*/

static PAPER_TYPE paper_types[] = {
	/*#[DEFAULT:	*/
	{
    	    MAXWIDTH_DEFAULT, "",
	},
	/*#]DEFAULT:	*/ 
	/*#[LETTER:	*/
	{
    	    MAXWIDTH_LETTER, "",
	},
	/*#]LETTER:	*/ 
	/*#[A4WIDE:	*/
	{
    	    MAXWIDTH_A4WIDE, "a4wide,",
	},
	/*#]A4WIDE:	*/ 
	/*#[A4:	    	*/
	{
    	    MAXWIDTH_A4, "a4,",
	},
	/*#]A4:	    	*/ 
};

/* #]static:		*/ 
 
/* #[tex_figuresize:	*/

int
tex_figuresize(paper)
int paper;
{
    return paper_types[paper].figure_size;
}

/* #]tex_figuresize:	*/ 
/* #[tex_header:	*/

void
tex_header(g)
GOGAME *g;
{
    static int done = 0;
    char *gomacros;
    char *goprolog;

    if (done) return;

    /*
     * Settings
     */
    if ((gomacros = getenv("SGFMACROS")) == NULL) gomacros = GOMACROS;
    if ((goprolog = getenv("SGFPROLOG")) == NULL) goprolog = GOPROLOG;

    /*
     * Document header
     */
    switch (prn_dev) {
	case DEVICE_PSTEX:
		fprintf(fw, "\\documentstyle[%s%s%c%s]{article}\n",
			paper_types[paper_size].style_file,
			gomacros, DIRSEP, "sgf");
		fprintf(fw, "\\special{header=%s%c%s}\n", goprolog, DIRSEP, "gopro.eps");
		fprintf(fw, "\\makeatletter\n");
		if (epsf.eps_width <= paper_types[paper_size].figure_size) {
		    if (kibitz) {
			/* Extra intercolumn gap for usernames */
			fprintf(fw, "\\setlength{\\columnsep}{2.25cm}\n");
		    } else {
			/* Small intercolumn gap */
			fprintf(fw, "\\setlength{\\columnsep}{1.0cm}\n");
		    }
		    /* No intercolumn seperator */
		    fprintf(fw, "\\setlength{\\columnseprule}{0.0pt}\n");
		}
		fprintf(fw, "\\makeatother\n");
		break;
	case DEVICE_TEX:
		fprintf(fw, "\\documentstyle[sgf,go]{article}\n");
		fprintf(fw, "\\gofontsize{8}\n");
		break;
    }

    /*
     * Title page
     */
    fprintf(fw, "\\title{%%\n");
	fprintf(fw, "    \\centerline{");
	    text2tex((g->gm_pb) ? g->gm_pb : "NN");
	    fprintf(fw, ", ");
	    text2tex((g->gm_br) ? g->gm_br : "NR");
	fprintf(fw, "}\n");
	fprintf(fw, "    \\centerline{versus}\n");
	fprintf(fw, "    \\centerline{");
	    text2tex((g->gm_pw) ? g->gm_pw : "NN");
	    fprintf(fw, ", ");
	    text2tex((g->gm_wr) ? g->gm_wr : "NR");
	fprintf(fw, "}\n");
    fprintf(fw, "}\n");
    fprintf(fw, "\\author{(generated by sgf2misc)}\n");
    fprintf(fw, "\\date  {");
	    text2tex((g->gm_dt) ? g->gm_dt : "");
    fprintf(fw, "}\n");
    fprintf(fw, "\\begin{document}\n");
    fprintf(fw, "\\maketitle\n");
    fprintf(fw, "\\thispagestyle{empty}\n");
    fprintf(fw, "\\clearpage\n");

    /*
     * One or twocolumn depends on the mode and the diagram size
     */
    switch (prn_dev) {
	case DEVICE_PSTEX:
	    if (epsf.eps_width <= paper_types[paper_size].figure_size)
    	    	fprintf(fw, "\\twocolumn\n");
	    break;
	case DEVICE_TEX:
	    fprintf(fw, "\\twocolumn\n");
	    break;
    }

    /*
     * Section header
     */
    fprintf(fw, "\\section{The Game}\n");
    fprintf(fw, "\\vskip 5pt\n");

    /*
     * Game header (players, rank, date, place and result)
     */
    if (!kibitz) fprintf(fw, "\\centerline{\\parbox{.5\\hsize}{%%\n");
    fprintf(fw, "\\Black{");
		    text2tex((g->gm_pb) ? g->gm_pb : "NN");
		    fprintf(fw, "}{");
		    text2tex((g->gm_br) ? g->gm_br : "NR");
    fprintf(fw, "}\n");
    fprintf(fw, "\\White{");
		    text2tex((g->gm_pw) ? g->gm_pw : "NN");
		    fprintf(fw, "}{");
		    text2tex((g->gm_wr) ? g->gm_wr : "NR");
    fprintf(fw, "}\n");
    fprintf(fw, "\\Date{");
		    text2tex((g->gm_dt) ? g->gm_dt : "");
    fprintf(fw, "}\n");
    fprintf(fw, "\\Place{");
		    text2tex((g->gm_pc) ? g->gm_pc : "");
    fprintf(fw, "}\n");
    if (g->gm_ha) {
        fprintf(fw, "\\Handicap{");
                  text2tex(g->gm_ha);
        fprintf(fw, "}\n");
    }
    fprintf(fw, "\\Komi{");
                  text2tex((g->gm_km) ? g->gm_km : "none");
    fprintf(fw, "}\n");
    fprintf(fw, "\\Result{");
		    text2tex((g->gm_re) ? g->gm_re : "");
    fprintf(fw, "}\n");
    if (!kibitz) fprintf(fw, "}}\n");

    done = 1;
}

/* #]tex_header:	*/ 
/* #[tex_trailer:	*/

void
tex_trailer()
{
    static int done = 0;

    if (done) return;

    if (statout) {
	/*#[Name:*/

	fprintf(fw, "%%%% #[By name:\n");
	fprintf(fw, "\\clearpage\n");
	fprintf(fw, "\\section{Kibitzers sorted by name}\n");
	kib_list(0, "\\kdict");
	fprintf(fw, "%%%% #]By name: \n");

	/*#]Name:*/ 
	/*#[Rank:*/

	fprintf(fw, "%%%% #[By rank:\n");
	fprintf(fw, "\\vskip1000pt\n");
	fprintf(fw, "\\section{Kibitzers sorted by rank}\n");
	kib_list(1, "\\krank");
	fprintf(fw, "%%%% #]By rank: \n");

	/*#]Rank:*/ 
	/*#[Freq:*/

	fprintf(fw, "%%%% #[By frequency:\n");
	fprintf(fw, "\\vskip1000pt\n");
	fprintf(fw, "\\section{Kibitzers sorted by frequency}\n");
	kib_list(2, "\\kfreq");
	fprintf(fw, "%%%% #]By frequency: \n");

	/*#]Freq:*/ 
	/*#[Byte:*/

	fprintf(fw, "%%%% #[By amount:\n");
	fprintf(fw, "\\vskip1000pt\n");
	fprintf(fw, "\\section{Kibitzers sorted by amount}\n");
	kib_list(3, "\\kbyte");
	fprintf(fw, "%%%% #]By amount: \n");

	/*#]Byte:*/ 
    }
    fprintf(fw, "\\end{document}\n");
    fputc('\n', stderr);	/* To flush the messages on stderr */

    done = 1;
}

/* #]tex_trailer:	*/ 
/* #[tex_text:		*/

void
tex_text(t1, t2, n1, n2)
GOMOVE *t1;
GOMOVE *t2;
int	n1;
int	n2;
/*
 * Print the text associated with a position
 */
{
    GOMOVE *m;
    char    username[MAXUSER];		/* Name of last kibitzer */

    if (!t1) return;

    for (m = t1; m != t2; m = m->mv_next) {
	TEXT   *t;

	if (!m->mv_text) continue;

/*
	  #[statistics:
*/
	if (kibitz) kib_add(m->mv_text);
/*
	  #]statistics: 
	  #[labels:
*/
	if (m->mv_nr) {
	    username[0] = 0;
	    if (labels && n1 != n2) {
		fprintf(fw, "\\%cn{%d}",
		    (m->mv_side == BLACK) ? 'b' : 'w', m->mv_nr);
		fprintf(fw, "%s\n", (kibitz) ? "\\par" : "");
	    }
	}
/*
	  #]labels: 
	  #[text:
*/
	for (t = m->mv_text; t; t = t->cc_next) {
	    char line[1024];
/*
	  #[preparation:
*/
	    strcpy(line, t->cc_text);
	    if (*(line) &&
		*(line + strlen(line)-1) == ' ')
		*(line + strlen(line)-1) = '\n';
	    if (kibitz	&&
		*(line) &&
		*(line + strlen(line)-1) == '\n')
		*(line + strlen(line)-1) = 0;
/*
	  #]preparation: 
	  #[kibitz:
*/
	    if (kibitz) {
		char rankstr[10];
		char *r;

		if (strcmp(username, t->cc_name)) {
		    strcpy(username, t->cc_name);
		    strcpy(rankstr, rank2str(t->cc_spec));
		    for (r = rankstr; *r == ' '; r++);
		    fprintf(fw, "\\kk{%s}{(%s)}{", t->cc_name, r);
		    text2tex(line);
		    fprintf(fw, "}\n");
		} else {
		    fprintf(fw, "\\kk{}{}{");
		    text2tex(line);
		    fprintf(fw, "}\n");
		}
		continue;
	    }
/*
	  #]kibitz: 
	  #[non-zhuge:
*/
	    if (!zhuge) { text2tex(line); continue; }
/*
	  #]non-zhuge: 
	  #[zhuge:
*/
	    {
		/*
		 * Author's notes are in "(*" "*)" blocks
		 */
		char *s		 = line;
		char *note_start = strstr(line, "(*");
		char *note_end	 = strstr(line, "*)");
		int   restore    = 0;

		if (note_start) {
		    fprintf(fw, "\\begin{em}\n");
		    s = note_start+2;
		    if (*s == ' ') s++;
		}
		if (note_end) {
		if (*(note_end-1) == ' ') note_end--;
		    restore = *note_end;
		    *note_end = 0;
		}
		text2tex(s);
		if (note_end) {
		    fprintf(fw, "\n\\end{em}\n");
		    *note_end = restore; /* restore line */
		}
	    }
/*
	  #]zhuge: 
*/
	}
	fputc('\n', fw);	/* create a paragraph */
/*
	  #]text: 
*/
    }
}

/* #]tex_text:		*/ 
/* #[text2tex:		*/

void
text2tex(t)
/*
 * Copy t to fw while taking care of LaTeX special characters.
 */
char *t;
{
    for (; *t; t++) {
	/*
	 * Special characters
	 */
	if (*t == '_' || *t == '#' || *t == '{' || *t == '}' ||
	    *t == '&' || *t == '%' || *t == '$') {
		fprintf(fw, "\\%c", *t);
	} else

	/*
	 * Accent characters
	 */
	if (*t == '^') {
		fprintf(fw, "\\%c{}", *t);
	} else

	/*
	 * Math characters
	 */
	if (*t == '+' /* || *t == '-' */ ||
	    *t == '<' || *t == '>' ||
	    *t == '|' || *t == '=') {
		fprintf(fw, "$%c$", *t);
	} else

	/*
	 * Backslash
	 */
	if (*t == '~') {
	    fprintf(fw, "\\Tilde%c", (*(t+1) == ' ') ? '\\' : ' ');
	} else

	/*
	 * Backslash
	 */
	if (*t == '\\')
		fprintf(fw, "$\\backslash$");

	else  putc(*t, fw);
    }
}

/* #]text2tex:		*/ 
